searchNodes=[{"doc":"Fast and flexible runtime type-checking. The main way to use TypeCheck is by adding use TypeCheck in your modules. This will allow you to use the macros of TypeCheck.Macros in your module, which are versions of the normal type-specification module attributes with an extra exclamation mark at the end: @type! , @spec! , @typep! and @opaque! . It will also bring all functions in TypeCheck.Builtin in scope, which is usually what you want as this allows you to use all types and special syntax that are built-in to Elixir in the TypeCheck specifications. Using these, you're able to add function-specifications to your functions which will wrap them with runtime type-checks. You'll also be able to create your own type-specifications that can be used in other type- and function-specifications in the same or other modules later on: defmodule User do use TypeCheck defstruct [ :name , :age ] @type! age :: non_neg_integer ( ) @type! t :: % User { name : binary ( ) , age : age ( ) } @spec! new ( binary ( ) , age ( ) ) :: t ( ) def new ( name , age ) do % User { name : name , age : age } end @spec! old_enough? ( t ( ) , age ( ) ) :: boolean ( ) def old_enough? ( user , limit ) do user . age &gt;= limit end end Finally, you can test whether your functions correctly adhere to their specs, by adding a spectest in your testing suite. See TypeCheck.ExUnit.spectest/2 for details. Types and their syntax TypeCheck allows types written using (essentially) the same syntax as Elixir's builtin typespecs . This means the following: literal values like :ok , 10.0 or &quot;my string&quot; desugar to a call to TypeCheck.Builtin.literal/1 , which is a type that matches only exactly that value. Basic types like integer() , float() , atom() etc. are directly supported (and exist as functions in TypeCheck.Builtin ). tuples of types like {atom(), integer()} are supported (and desugar to TypeCheck.Builtin.fixed_tuple/1 ) maps where keys are literal values and the values are types like %{a: integer(), b: integer(), 42 =&gt; float()} desugar to calls to TypeCheck.Builtin.fixed_map/1 . The same happens with structs like %User{name: binary(), age: non_neg_integer()} sum types like integer() | string() | atom() are supported, and desugar to calls to TypeCheck.Builtin.one_of/1 . Ranges like lower..higher are supported, matching integers within the given range. This desugars into a call to TypeCheck.Builtin.range/1 . Currently unsupported features The following typespec syntax can not currently be used in TypeCheck. This will hopefully change in future versions of the library. Literal maps with required(...) and optional(...) keys. (TypeCheck does already support literal maps with a fixed set of keys, as well as maps with any number of key-value-pairs of fixed types. It is the special syntax that might mix these approaches that is not supported yet.) Extensions TypeCheck adds the following extensions on Elixir's builtin typespec syntax: fixed-size lists containing types like [1, 2, integer()] are supported, and desugar to TypeCheck.Builtin.fixed_list/1 . This example matches only lists of 3 elements where the first element is the literal 1 , the second the literal 2 and the last element any integer. Elixir's builtin typespecs do not support fixed-size lists. named types like x :: integer() are supported; these are useful in combination with &quot;type guards&quot; (see the section below). &quot;type guards&quot; using the syntax some_type when arbitrary_code are supported, to add extra arbitrary checks to a value for it to match the type. (See the section about type guards below.) lazy(some_type) , which defers type-expansion until during runtime. This is required to be able to expand recursive types. C.f. TypeCheck.Builtin.lazy/1 Named Types Type Guards To add extra custom checks to a type, you can use a so-called 'type guard'. This is arbitrary code that is executed during a type-check once the type itself already matches. You can use &quot;named types&quot; to refer to (parts of) the value that matched the type, and refer to these from a type-guard: type sorted_pair :: { lower :: number ( ) , higher :: number ( ) } when lower &lt;= higher iex&gt; TypeCheck . conforms! ( { 10 , 20 } , sorted_pair ) { 10 , 20 } iex&gt; TypeCheck . conforms! ( { 20 , 10 } , sorted_pair ) ** (TypeCheck.TypeError) `{20, 10}` does not match the definition of the named type `TypeCheckTest.TypeGuardExample.sorted_pair` which is: `TypeCheckTest.TypeGuardExample.sorted_pair :: (sorted_pair :: {lower :: number(), higher :: number()} when lower &lt;= higher)`. Reason: `{20, 10}` does not check against `(sorted_pair :: {lower :: number(), higher :: number()} when lower &lt;= higher)`. Reason: type guard: `lower &lt;= higher` evaluated to false or nil. bound values: %{higher: 10, lower: 20, sorted_pair: {20, 10}} Named types are available in your guard even from the (both local and remote) types that you are using in your time, as long as those types are not defined as opaque types. Manual type-checking If you want to check values against a type outside of the checks the @spec! macro wraps a function with, you can use the conforms/2 / conforms?/2 / conforms!/2 macros in this module directly in your code. These are evaluated at compile time which means the resulting checks will be optimized by the compiler. Unfortunately it also means that the types passed to them have to be known at compile time. If you have a type that is constructed dynamically at runtime, you can resort to dynamic_conforms/2 and variants. Because these variants have to evaluate the type-checking code at runtime, these checks are not optimized by the compiler. Introspection To allow checking what types and specs exist, the introspection function __type_check__/1 will be added to a module when use TypeCheck is used. YourModule.__type_check__(:types) returns a keyword list of all {type_name, arity} pairs for all types defined in the module using @type! / @typep! or @opaque! . YourModule.__type_check__(:specs) returns a keyword list of all {function_name, arity} pairs for all functions in the module wrapped with a spec using @spec! . Note that these lists might also contain private types / private function names.","ref":"TypeCheck.html","title":"TypeCheck","type":"module"},{"doc":"Makes sure value typechecks the type description type . If it typechecks, we return {:ok, value} Otherwise, we return {:error, %TypeCheck.TypeError{}} which contains information about why the value failed the check. conforms is a macro and expands the type check at compile-time, allowing it to be optimized by the compiler. C.f. TypeCheck.Type.build/1 for more information on what type-expressions are allowed as type parameter. Note: usually you'll want to import TypeCheck.Builtin in the context where you use conforms , which will bring Elixir's builtin types into scope. (Calling use TypeCheck will already do this; see the module documentation of TypeCheck for more information))","ref":"TypeCheck.html#conforms/3","title":"TypeCheck.conforms/3","type":"macro"},{"doc":"Similar to conforms/2 , but returns value if the value typechecked and raises TypeCheck.TypeError if it did not. The same features and restrictions apply to this function as to conforms/2 .","ref":"TypeCheck.html#conforms!/3","title":"TypeCheck.conforms!/3","type":"macro"},{"doc":"Similar to conforms/2 , but returns true if the value typechecked and false if it did not. The same features and restrictions apply to this function as to conforms/2 .","ref":"TypeCheck.html#conforms?/3","title":"TypeCheck.conforms?/3","type":"macro"},{"doc":"Makes sure value typechecks the type type . Evaluated at runtime . Because dynamic_conforms/2 is evaluated at runtime: The typecheck cannot be optimized by the compiler, which makes it slower. You must pass an already-expanded type as type . This can be done by using one of your custom types directly (e.g. YourModule.typename() ), or by calling TypeCheck.Type.build . Use dynamic_conforms only when you cannot use the normal conforms/2 , for instance when you're only able to construct the type to check against at runtime. iex&gt; fourty_two = TypeCheck.Type . build ( 42 ) iex&gt; TypeCheck . dynamic_conforms ( 42 , fourty_two ) { :ok , 42 } iex&gt; { :error , type_error } = TypeCheck . dynamic_conforms ( 20 , fourty_two ) iex&gt; type_error . message &quot;At (for doctest at) lib/type_check.ex:281: `20` is not the same value as `42`.&quot;","ref":"TypeCheck.html#dynamic_conforms/3","title":"TypeCheck.dynamic_conforms/3","type":"function"},{"doc":"Similar to dynamic_conforms/2 , but returns value if the value typechecked and raises TypeCheck.TypeError if it did not. The same features and restrictions apply to this function as to dynamic_conforms/2 . iex&gt; fourty_two = TypeCheck.Type . build ( 42 ) iex&gt; TypeCheck . dynamic_conforms! ( 42 , fourty_two ) 42 iex&gt; TypeCheck . dynamic_conforms! ( 20 , fourty_two ) ** (TypeCheck.TypeError) At lib/type_check.ex:349: `20` is not the same value as `42`.","ref":"TypeCheck.html#dynamic_conforms!/3","title":"TypeCheck.dynamic_conforms!/3","type":"function"},{"doc":"Similar to dynamic_conforms/2 , but returns true if the value typechecked and false if it did not. The same features and restrictions apply to this function as to dynamic_conforms/2 . iex&gt; fourty_two = TypeCheck.Type . build ( 42 ) iex&gt; TypeCheck . dynamic_conforms? ( 42 , fourty_two ) true iex&gt; TypeCheck . dynamic_conforms? ( 20 , fourty_two ) false","ref":"TypeCheck.html#dynamic_conforms?/3","title":"TypeCheck.dynamic_conforms?/3","type":"function"},{"doc":"","ref":"TypeCheck.html#t:value/0","title":"TypeCheck.value/0","type":"type"},{"doc":"Implements the defstruct! macro. To use this macro: Ensure you use TypeCheck in your module Also use TypeCheck.Defstruct And now call defstruct!/1 when you want to define a struct.","ref":"TypeCheck.Defstruct.html","title":"TypeCheck.Defstruct","type":"module"},{"doc":"Defines a struct and a TypeCheck type at the same time. Example: defmodule User do use TypeCheck use TypeCheck.Defstruct defstruct! ( name : &quot;Guest&quot; :: String . t ( ) , age : _ :: non_neg_integer ( ) ) end This is syntactic sugar for: defmodule User do use TypeCheck use TypeCheck.Defstruct @type! t ( ) :: % User { name : String . t ( ) , age : non_neg_integer ( ) } @enforce_keys [ :age ] defstruct [ :age , name : &quot;Guest&quot; ] end Optional and required keys A key is considered optional if it uses the syntax name : default_value :: type A key is considered required if it uses one of the following syntaxes: :name :: type name : _ :: type In this case, it will be added to the @enforce_keys list. (c.f. Kernel.defstruct ).","ref":"TypeCheck.Defstruct.html#defstruct!/1","title":"TypeCheck.Defstruct.defstruct!/1","type":"macro"},{"doc":"Provides macros for 'spectests': spec-automated property-testing. The core macro exposed by this module is spectest/2 , which will will check for all function-specs in the given module, whether those functions correctly follow the spec. To use this functionality, add use TypeCheck.ExUnit to your testing module. Currently, spectesting uses StreamData under the hood. This means that to use the spectest functionality, you need to add the :stream_data dependency to your application (it is an optional dependency of TypeCheck .) In the future, support for other property-generating libraries might be added. What is a spectest? A 'function-specification test' is a property-based test in which we check whether the function adheres to its invariants (also known as the function's contract or preconditions and postconditions ). We generate a large amount of possible function inputs, and for each of these, check whether the function: Does not raise an exception. Returns a result that type-checks against the spec's return-type. (To be precise, if an incorrect result is returned, the function is wrapped in will end up raising an exception for this.) While @spec! s themselves ensure that callers do not mis-use your function, a spectest ensures¹ that the function itself is working correctly. Spectests are given its own test-category in ExUnit, for easier recognition (Just like 'doctests' and 'properties' are different from normal tests, so are 'spectests'.) ¹: Because of the nature of property-based testing, we can never know for 100% sure that a function is correct. However, with every new randomly-generated test-case, the level of confidence grows a little. So while we can never by fully sure, we are able to get asymptotically close to it.","ref":"TypeCheck.ExUnit.html","title":"TypeCheck.ExUnit","type":"module"},{"doc":"Sets up a testing module for spectesting. Not normally invoked directly, but rather by calling use TypeCheck.ExUnit . Currently does not accept any options, but this might change in the future.","ref":"TypeCheck.ExUnit.html#__using__/1","title":"TypeCheck.ExUnit.__using__/1","type":"macro"},{"doc":"Tests the functions in module against their @spec! s. spectest will look at all functions which have a TypeCheck spec in module , and will for each of them run a 'spectest'. See the module documentation for more information on spectests in general. Examples defmodule MyModuleTest do use ExUnit.Case , async : true use TypeCheck.ExUnit # Test all functions that have `@spec!`s in `MyModule` spectest MyModule # Test all functions that have `@spec!`s in `MyOtherModule`, # except `MyOtherModule.bar/2` and `MyOtherModule.baz/0` spectest MyOtherModule , except : [ { :bar , 2 } , { :baz , 0 } ] # Test only `OneMoreModule.foo/2` and `MyOtherModule.qux/0` spectest OneMoreModule , only : [ { :foo , 2 } , { :qux , 0 } ] end Options :except :only :initial_seed :generator Except and Only By default, all functions in the module (that have an associated @spec! ) will be tested. If any of them need to be skipped, you can add them as a list of {name, arity} -pairs under except: . If instead of excluding a few functions, you want to only test a small subset of functions, you can add them as {name, arity} -pairs under only: . Initial seed The :initial_seed -option can be used to seed the property-generation. It expects an integer value. This option is passed on to the generator automatically (without requiring the usage of generator-specific options). By default, the seed of the ExUnit configuration is used (which by default differs every test run). Generator The generator: option expects either the name of a property-testing library, or a {name, options} -tuple. (If only the name is specified, this is a shorthand for {Name, []} ). For now, only StreamData is supported, and this is its default value. If you want to pass extra options to the library, the notation {StreamData, list_of_options} can be passed. For the list of options supported by StreamData, see StreamData.check_all/3 .","ref":"TypeCheck.ExUnit.html#spectest/2","title":"TypeCheck.ExUnit.spectest/2","type":"macro"},{"doc":"Working with regular Elixir and Erlang @spec and @type definitions. Experimental This module is experimental. The API of the various functions in this module are not yet stable; they might change slightly in future minor versions of the library. If you encounter a problem, please, open an issue .","ref":"TypeCheck.External.html","title":"TypeCheck.External","type":"module"},{"doc":"Non-raising version of TypeCheck.External.apply!/4 . Ensures values conform to the given type. Returns {:error, reason} on failure, {:ok, function_call_result} otherwise. iex&gt; alias TypeCheck.Builtin , as : B iex&gt; type = B . function ( [ B . number ] , B . number ) iex&gt; TypeCheck.External . apply ( type , Kernel , :abs , [ - 13 ] ) { :ok , 13 } iex&gt; { :error , err } = TypeCheck.External . apply ( type , Kernel , :abs , [ false ] ) iex&gt; err . message &quot;At lib/type_check/external.ex:176: \\n `false` is not a number.&quot;","ref":"TypeCheck.External.html#apply/4","title":"TypeCheck.External.apply/4","type":"function"},{"doc":"The same as Kernel.apply/3 but ensures that values conform the given type. The first argument must be a function type of the function to be called. In case of type error, raises an exception. Examples iex&gt; alias TypeCheck.Builtin , as : B iex&gt; type = B . function ( [ B . number ] , B . number ) iex&gt; TypeCheck.External . apply! ( type , Kernel , :abs , [ - 13 ] ) 13 iex&gt; TypeCheck.External . apply! ( type , Kernel , :abs , [ &quot;hello&quot; ] ) ** (TypeCheck.TypeError) At lib/type_check/external.ex:176: `&quot;hello&quot;` is not a number.","ref":"TypeCheck.External.html#apply!/4","title":"TypeCheck.External.apply!/4","type":"function"},{"doc":"Ensure at runtime that arguments at result of function call conform the function spec. The function spec is extracted at compile time from the regular Elixir (or Erlang) @spec . Examples iex&gt; import TypeCheck.External iex&gt; enforce_spec! ( Kernel . abs ( - 13 ) ) 13 iex&gt; enforce_spec! ( Kernel . abs ( &quot;hi&quot; ) ) ** (TypeCheck.TypeError) At lib/type_check/external.ex:176: `&quot;hi&quot;` is not a number.","ref":"TypeCheck.External.html#enforce_spec!/1","title":"TypeCheck.External.enforce_spec!/1","type":"macro"},{"doc":"Extract TypeCheck type from the regular Elixir (or Erlang) @spec of the given function. Examples iex&gt; import TypeCheck.External iex&gt; { :ok , type } = fetch_spec ( Kernel , :abs , 1 ) iex&gt; type # TypeCheck.Type &lt; ( number ( ) -&gt; number ( ) ) &gt; iex&gt; { :ok , type } = fetch_spec ( Atom , :to_string , 1 ) iex&gt; type # TypeCheck.Type &lt; ( atom ( ) -&gt; binary ( ) ) &gt; iex&gt; fetch_spec ( Kernel , :non_existent , 1 ) { :error , &quot;cannot find spec for function&quot; }","ref":"TypeCheck.External.html#fetch_spec/3","title":"TypeCheck.External.fetch_spec/3","type":"function"},{"doc":"Extract TypeCheck type from the regular Elixir (or Erlang) @type with the given name. To fetch a generic type, you must pass a list of types to be placed instead of generic variables. If you don't know these types, just punch in any() as many times as you need. For example, to fetch the type of Range.t(left, right) , you need to pass [number(), number()] or [any(), any()] . Examples Fetching a regular type: iex&gt; import TypeCheck.External iex&gt; { :ok , type } = fetch_type ( String , :t ) iex&gt; type # TypeCheck.Type &lt; binary ( ) &gt; Fetching a generic type: iex&gt; import TypeCheck.External iex&gt; import TypeCheck.Builtin iex&gt; { :ok , type } = fetch_type ( :elixir , :keyword , [ number ( ) ] ) iex&gt; type # TypeCheck.Type &lt; list ( { atom ( ) , number ( ) } ) &gt; iex&gt; { :ok , type } = fetch_type ( Range , :t , [ integer ( ) , integer ( ) ] ) iex&gt; type # TypeCheck.Type &lt; % Range { first : integer ( ) , last : integer ( ) , step : pos_integer ( ) | neg_integer ( ) } &gt; Fetching non-existent type causes an error: iex&gt; import TypeCheck.External iex&gt; fetch_type ( String , :non_existent ) { :error , &quot;cannot find type with the given name&quot; }","ref":"TypeCheck.External.html#fetch_type/3","title":"TypeCheck.External.fetch_type/3","type":"function"},{"doc":"Contains the @spec! , @type! , @typep! , @opaque! macros to define runtime-checked function- and type-specifications. Usage This module is included by calling use TypeCheck . This will set up the module to use the special macros. Usually you'll want to use the module attribute-style of the macros, like @spec! and @type! . Using these forms has two advantages over using the direct calls: Syntax highlighting will highlight the types correctly and the Elixir formatter will not mess with the way you write your type. It is clear to people who have not heard of TypeCheck before that @type! and @spec! will work similarly to resp. @type and @spec . Avoiding naming conflicts with TypeCheck.Builtin If you want to define a type with the same name as one in TypeCheck.Builtin, (which is not particularly recommended) , you should hide those particular functions from TypeCheck.Builtin by adding import TypeCheck.Builtin, except: [...] below use TypeCheck manually. Calling the explicit implementations In case you are working in an environment where the @/1 is already overridden by another library, you can still use this library, by simply adding import TypeCheck.Macros, except: [@: 1] to your module and calling the direct versions of the macros instead. TypeCheck and metaprogramming In certain cases you might want to use TypeCheck to dynamically generate types or functions, such as to add @spec! -s to functions that themselves are dynamically generated. TypeCheck's macros support 'unquote fragments', just like many builtin 'definition' constructs like def , but also @type do. (c.f. Elixir.Kernel.SpecialForms.quote/2 for more details about unquote fragments.) An example: defmodule MetaExample do use TypeCheck people = ~w[joe robert mike]a for name &lt;- people do @type! unquote ( name ) ( ) :: %{ name : unquote ( name ) , coolness_level : :high } end end iex&gt; MetaExample . joe # TypeCheck.Type &lt; TypeCheck.MacrosTest.MetaExample . joe ( ) :: %{ coolness_level : :high , name : :joe } &gt; iex&gt; MetaExample . mike # TypeCheck.Type &lt; TypeCheck.MacrosTest.MetaExample . mike ( ) :: %{ coolness_level : :high , name : :mike } &gt; Macros Inside macros, we use unquote fragments in the same way. There is however one more thing to keep in mind: You'll need to add a call to import Kernel, except: [@: 1] in your macro (before the quote) to make sure you can call @type! , @spec! etc. This is a subtle consequence of Elixir's macro-hygiene rules. See this issue on Elixir's GitHub repository for more info (Alternatively, directly calls to type! , spec! etc. are possible without overriding the import.) An example: defmodule GreeterMacro do defmacro generate_greeter ( greeting ) do import Kernel , except : [ @ : 1 ] # Ensures TypeSpec&#39;s overridden `@` is used in the quote quote do @spec! unquote ( greeting ) ( binary ) :: binary def unquote ( greeting ) ( name ) do &quot; \#{ greeting } , \#{ name } !&quot; end end end end defmodule GreeterExample do use TypeCheck require GreeterMacro GreeterMacro . generate_greeter ( :hi ) GreeterMacro . generate_greeter ( :hello ) end iex&gt; GreeterExample . hi ( &quot;John&quot; ) &quot;hi, John!&quot; iex&gt; GreeterExample . hello ( &quot;Frank&quot; ) &quot;hello, Frank!&quot; iex&gt; GreeterExample . hi ( 42 ) ** (TypeCheck.TypeError) At test/type_check/macros_test.exs:32: The call to `hi/1` failed, because parameter no. 1 does not adhere to the spec `binary()`. Rather, its value is: `42`. Details: The call `hi(42)` does not adhere to spec `hi(binary()) :: binary()`. Reason: parameter no. 1: `42` is not a binary. About use TypeCheck The use TypeCheck statement adds an @before_compile -hook to the final module, which is used to wrap functions with the specified runtime type-checks. This means that some care needs to be taken to ensure that a call to use TypeCheck exists in the final module, if you're generating specs dynamically from inside macros. Hiding the autogenerated typespec By default, TypeCheck will automatically generate @type , @opaque and @spec -attributes, which will be shown in the documentation, as well as used by tools such as Dialyzer. In rare situations, TypeCheck might try to generate typespecs which are invalid. (In such case, please open a bug report! ) Or sometimes, you might want to alter the type which is exported. In such situations, you can disable the autogeneration of these attributes, by calling @autogen_typespec false just before the next @type! / @opaque! / @spec! : defmodule AutogenTypespecsExample do use TypeCheck # The typespec of `foo` is auto-generated: # A line `@type foo() :: integer()` will be visible in the documentation/Dialyzer. @type! foo ( ) :: integer ( ) # The typespec of `bar` is _not_ auto-generated. # As such, we could write a completely different `@type` (or leave it out all-together). @autogen_typespec false @type! bar ( ) :: integer ( ) end iex&gt; t AutogenTypespecsExample # Will show type of `foo` but not `bar` &quot;@type foo() :: integer()&quot;","ref":"TypeCheck.Macros.html","title":"TypeCheck.Macros","type":"module"},{"doc":"","ref":"TypeCheck.Macros.html#@/1","title":"TypeCheck.Macros.@/1","type":"macro"},{"doc":"Define a opaque type specification. Usually invoked as @opaque! This behaves similarly to Elixir's builtin @opaque attribute, and will create a type whose name is public but whose structure is private. Calling this macro will: Fill the @opaque -attribute with a Typespec-friendly representation of the TypeCheck type. Add a (or append to an already existing) @typedoc detailing that the type is managed by TypeCheck, and containing the name of the TypeCheck type. (not the definition, since it is an opaque type). Define a (hidden) public function with the same name (and arity) as the type that returns the TypeCheck.Type as a datastructure when called. This makes the type usable in calls to: definitions of other type-specifications (in the same or different modules). definitions of function-specifications (in the same or different modules). TypeCheck.conforms/2 and variants, TypeCheck.Type.build/1 opaque!/1 accepts the same typedef expression as type!/1 .","ref":"TypeCheck.Macros.html#opaque!/1","title":"TypeCheck.Macros.opaque!/1","type":"macro"},{"doc":"Define a function specification. Usually invoked as @spec! A function specification will wrap the function with checks that each of its parameters are of the types it expects. as well as checking that the return type is as expected. Usage The syntax is essentially the same as for built-in @spec attributes: @spec! function_name ( type1 , type2 ) :: return_type It is also allowed to introduce named types: @spec! days_since_epoch ( year :: integer , month :: integer , day :: integer ) :: integer Note that TypeCheck does not allow the when keyword to be used to restrict the types of recurring type variables (which Elixir's builtin Typespecs allow). This is because: Usually it is more clear to give a recurring type an explicit name. The when keyword is used instead for TypeCheck's type guards'. (See TypeCheck.Builtin.guarded_by/2 for more information.)","ref":"TypeCheck.Macros.html#spec!/1","title":"TypeCheck.Macros.spec!/1","type":"macro"},{"doc":"Define a public type specification. Usually invoked as @type! This behaves similarly to Elixir's builtin @type attribute, and will create a type whose name and definition are public. Calling this macro will: Fill the @type -attribute with a Typespec-friendly representation of the TypeCheck type. Add a (or append to an already existing) @typedoc detailing that the type is managed by TypeCheck, and containing the full definition of the TypeCheck type. Define a (hidden) public function with the same name (and arity) as the type that returns the TypeCheck.Type as a datastructure when called. This makes the type usable in calls to: definitions of other type-specifications (in the same or different modules). definitions of function-specifications (in the same or different modules). TypeCheck.conforms/2 and variants, TypeCheck.Type.build/1 Usage The syntax is essentially the same as for the built-in @type attribute: @type! type_name :: type_description It is possible to create parameterized types as well: @type! dict ( key , value ) :: [ { key , value } ] Named types You can also introduce named types: @type! color :: { red :: integer , green :: integer , blue :: integer } Not only is this nice to document that the same type is being used for different purposes, it can also be used with a 'type guard' to add custom checks to your type specifications: @type! sorted_pair ( a , b ) :: { first :: a , second :: b } when first &lt;= second","ref":"TypeCheck.Macros.html#type!/1","title":"TypeCheck.Macros.type!/1","type":"macro"},{"doc":"Define a private type specification. Usually invoked as @typep! This behaves similarly to Elixir's builtin @typep attribute, and will create a type whose name and structure is private (therefore only usable in the current module). Fill the @typep -attribute with a Typespec-friendly representation of the TypeCheck type. Define a private function with the same name (and arity) as the type that returns the TypeCheck.Type as a datastructure when called. This makes the type usable in calls (in the same module) to: definitions of other type-specifications definitions of function-specifications TypeCheck.conforms/2 and variants, TypeCheck.Type.build/1 typep!/1 accepts the same typedef expression as type!/1 .","ref":"TypeCheck.Macros.html#typep!/1","title":"TypeCheck.Macros.typep!/1","type":"macro"},{"doc":"Defines the options that TypeCheck supports on calls to use TypeCheck . Supported options: :overrides : A list of overrides for remote types. (default: [] ) :default_overrides : A boolean. If false, will not include any of the overrides of the types of Elixir's standard library (c.f. TypeCheck.DefaultOverrides.default_overrides/0 ). (default: true ) :enable_runtime_checks : When true, functions that contain a @spec! will be wrapped with a runtime check which will check the input to and result returned from the function. (Default: true ). :debug : When true, will (at compile-time) print the generated TypeCheck-checking code. (Default: false ) These options are usually specified as passed to use TypeCheck , although they may also be passed in direct calls to TypeCheck.conforms/3 (and its variants). These options are module-specific and are read/used at compile-time. Setting different options for different dependencies Different default options can be set for each of the different dependencies, by supplying the desired options in the configuration of that dependency under the :type_check key: config :your_dependency_name , :type_check , [ enable_runtime_checks : false # etc. ] Note that these values might (by design) be overridden by module-specific options that might be specified at the location where the dependency calls use TypeCheck . The supported options in detail Overrides: The :overrides field contains a list of remote types to be overridden by a replacement. This is useful to be able to specify TypeCheck-types for types that you do not have control over (because they are for instance defined in a library that is not itself using TypeCheck). For obvious reasons, using TypeCheck directly should be preferred over overriding types. Each of the elements in the :overrides list should be written as {original_type, replacement_type} . Both of these can take the shape of either &amp;Module.type/arity or the longer form {Module, :type, arity} . An example: use TypeCheck , overrides : [ { &amp; Ecto.Schema . t / 0 , &amp; MyProject.TypeCheckOverrides.Ecto.Schema . t / 0 } ] Enabling/Disabling runtime checks By default, runtime checks are enabled. In the case where the runtime checks turn out to be too slow (for instance, because of working with very large or deeply nested collections) in a particular module, they can be turned off completely. It is recommended to: Only turn them off after benchmarking has shown that this will make a significant difference. Only turn them off in e.g. the production environment, keeping them on in the development and test environments. An example: use TypeCheck , enable_runtime_checks : Mix . env ( ) != :prod Debugging Passing the option debug: true will at compile-time print the generated code for all added @spec s, as well as TypeCheck.conforms/3 / TypeCheck.conforms?/3 / TypeCheck.conforms!/3 calls.","ref":"TypeCheck.Options.html","title":"TypeCheck.Options","type":"module"},{"doc":"","ref":"TypeCheck.Options.html#check_overrides!/1","title":"TypeCheck.Options.check_overrides!/1","type":"function"},{"doc":"","ref":"TypeCheck.Options.html#new/0","title":"TypeCheck.Options.new/0","type":"function"},{"doc":"","ref":"TypeCheck.Options.html#new/1","title":"TypeCheck.Options.new/1","type":"function"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: remote_type ( ) :: mfa ( ) | function","ref":"TypeCheck.Options.html#t:remote_type/0","title":"TypeCheck.Options.remote_type/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t :: % TypeCheck.Options { overrides : type_overrides ( ) , default_overrides : boolean ( ) , enable_runtime_checks : boolean ( ) , debug : boolean ( ) }","ref":"TypeCheck.Options.html#t:t/0","title":"TypeCheck.Options.t/0","type":"type"},{"doc":"An extra check is performed to ensure that the original type and the replacement type have the same arity. (This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: type_override :: { original :: remote_type ( ) , replacement :: remote_type ( ) }","ref":"TypeCheck.Options.html#t:type_override/0","title":"TypeCheck.Options.type_override/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: type_overrides :: list ( type_override ( ) )","ref":"TypeCheck.Options.html#t:type_overrides/0","title":"TypeCheck.Options.type_overrides/0","type":"type"},{"doc":"","ref":"TypeCheck.Spec.html","title":"TypeCheck.Spec","type":"module"},{"doc":"True if a spec was added to {module, function, arity} . c.f. lookup/3 . iex(1)&gt; defmodule Example3 do ...(2)&gt; use TypeCheck ...(3)&gt; @spec! greeter ( name :: binary ( ) ) :: binary ( ) ...(4)&gt; def greeter ( name ) , do : &quot;Hello, \#{ name } !&quot; ...(5)&gt; end ...(6)&gt; ...(7)&gt; TypeCheck.Spec . defined? ( Example3 , :greeter , 1 ) true ...(8)&gt; TypeCheck.Spec . defined? ( Example3 , :nonexistent , 0 ) false","ref":"TypeCheck.Spec.html#defined?/3","title":"TypeCheck.Spec.defined?/3","type":"function"},{"doc":"Looks up the spec for a particular {module, function, arity} . On success, returns {:ok, spec} . On failure, returns {:error, :not_found} . This is quite an advanced low-level function, which you usually won't need to interact with directly. c.f. lookup!/3 . iex(1)&gt; defmodule Example do ...(2)&gt; use TypeCheck ...(3)&gt; @spec! greeter ( name :: binary ( ) ) :: binary ( ) ...(4)&gt; def greeter ( name ) , do : &quot;Hello, \#{ name } !&quot; ...(5)&gt; end ...(6)&gt; ...(7)&gt; { :ok , spec } = TypeCheck.Spec . lookup ( Example , :greeter , 1 ) ...(8)&gt; spec # TypeCheck.Spec &lt; greeter ( name :: binary ( ) ) :: binary ( ) &gt; iex&gt; TypeCheck.Spec . lookup ( Example , :nonexistent , 0 ) { :error , :not_found }","ref":"TypeCheck.Spec.html#lookup/3","title":"TypeCheck.Spec.lookup/3","type":"function"},{"doc":"Looks up the spec for a particular {module, function, arity} . On success, returns spec . Raises when the spec cannot be found. c.f. lookup/3 . iex(1)&gt; defmodule Example2 do ...(2)&gt; use TypeCheck ...(3)&gt; @spec! greeter ( name :: binary ( ) ) :: binary ( ) ...(4)&gt; def greeter ( name ) , do : &quot;Hello, \#{ name } !&quot; ...(5)&gt; end ...(6)&gt; ...(7)&gt; TypeCheck.Spec . lookup! ( Example2 , :greeter , 1 ) # TypeCheck.Spec &lt; greeter ( name :: binary ( ) ) :: binary ( ) &gt; iex&gt; TypeCheck.Spec . lookup! ( Example2 , :nonexistent , 0 ) ** (ArgumentError) No spec found for `Example2.nonexistent/0`","ref":"TypeCheck.Spec.html#lookup!/3","title":"TypeCheck.Spec.lookup!/3","type":"function"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: problem_tuple :: { t ( ) , :param_error , %{ index : non_neg_integer ( ) , problem : lazy ( TypeCheck.TypeError.Formatter . problem_tuple ( ) ) } , list ( any ( ) ) } | { t ( ) , :return_error , %{ arguments : list ( term ( ) ) , problem : lazy ( TypeCheck.TypeError.Formatter . problem_tuple ( ) ) } , list ( any ( ) ) }","ref":"TypeCheck.Spec.html#t:problem_tuple/0","title":"TypeCheck.Spec.problem_tuple/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t ( ) :: % TypeCheck.Spec { name : TypeCheck.DefaultOverrides.String . t ( ) , param_types : list ( TypeCheck.Type . t ( ) ) , return_type : TypeCheck.Type . t ( ) , location : [ ] | list ( { :file , TypeCheck.DefaultOverrides.String . t ( ) } | { :line , non_neg_integer ( ) } ) }","ref":"TypeCheck.Spec.html#t:t/0","title":"TypeCheck.Spec.t/0","type":"type"},{"doc":"TODO","ref":"TypeCheck.Type.html","title":"TypeCheck.Type","type":"module"},{"doc":"Constructs a concrete type from the given type_ast . This means that you can pass type-syntax to this macro, which will be transformed into explicit calls to the functions in TypeCheck.Builtin . iex&gt; res = TypeCheck.Type . build ( :ok | :error ) iex&gt; res # TypeCheck.Type &lt; :ok | :error &gt; iex&gt; # This is the same as: iex&gt; import TypeCheck.Builtin , only : [ one_of : 2 , literal : 1 ] iex&gt; explicit = one_of ( literal ( :ok ) , literal ( :error ) ) iex&gt; res == explicit true iex&gt; res = TypeCheck.Type . build ( { a :: number ( ) , b :: number ( ) } when a &lt;= b ) iex&gt; res # TypeCheck.Type &lt; ( { a :: number ( ) , b :: number ( ) } when a &lt;= b ) &gt; iex&gt; # This is the same as: iex&gt; import TypeCheck.Builtin , only : [ fixed_tuple : 1 , number : 0 , guarded_by : 2 , named_type : 2 ] iex&gt; explicit = guarded_by ( fixed_tuple ( [ named_type ( :a , number ( ) ) , named_type ( :b , number ( ) ) ] ) , quote do a &lt;= b end ) iex&gt; explicit # TypeCheck.Type &lt; ( { a :: number ( ) , b :: number ( ) } when a &lt;= b ) &gt; Of course, you can refer to your own local and remote types as well.","ref":"TypeCheck.Type.html#build/2","title":"TypeCheck.Type.build/2","type":"macro"},{"doc":"","ref":"TypeCheck.Type.html#type?/1","title":"TypeCheck.Type.type?/1","type":"function"},{"doc":"Indicates that we expect a 'type AST' that will be expanded to a proper type. This means that it might contain essentially the full syntax that Elixir Typespecs allow, which will be rewritten to calls to the functions in TypeCheck.Builtin . See TypeCheck.Builtin for the precise syntax you are allowed to use.","ref":"TypeCheck.Type.html#t:expandable_type/0","title":"TypeCheck.Type.expandable_type/0","type":"type"},{"doc":"Something is a TypeCheck.Type if it implements the TypeCheck.Protocols.ToCheck protocol. It is also expected to implement the TypeCheck.Protocols.Inspect protocol (although that has an Any fallback). In practice, this type means 'any of the' structs in the TypeCheck.Builtin.* modules. (This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t ( ) :: wrap_with_gen ( x :: any ( ) when TypeCheck.Type . type? ( x ) , &amp; TypeCheck.Type.StreamData . arbitrary_type_gen / 0 )","ref":"TypeCheck.Type.html#t:t/0","title":"TypeCheck.Type.t/0","type":"type"},{"doc":"Exception to be returned or raised when a value is not of the expected type. This exception has two fields: :raw , which will contain the problem tuple of the type check failure. :message , which will contain a the humanly-readable representation of the raw problem_tuple :message is constructed from :raw using the TypeCheck.TypeError.DefaultFormatter. (TODO at some point this might be configured to use your custom formatter instead)","ref":"TypeCheck.TypeError.html","title":"TypeCheck.TypeError","type":"exception"},{"doc":"The name of the particular check. Might be :no_match for simple types, but for more complex types that have multiple checks, it disambugates between them. For instance, for TypeCheck.Builtin.List we have :not_a_list , :different_length , and :element_error .","ref":"TypeCheck.TypeError.html#t:check_name/0","title":"TypeCheck.TypeError.check_name/0","type":"type"},{"doc":"An extra map with any information related to the check that failed. For instance, if the check was a compound check, will contain the field problem: with the child problem tuple as well as :index or :key etc. to indicate _where in the compound structure the check failed.","ref":"TypeCheck.TypeError.html#t:extra_information/0","title":"TypeCheck.TypeError.extra_information/0","type":"type"},{"doc":"","ref":"TypeCheck.TypeError.html#t:location/0","title":"TypeCheck.TypeError.location/0","type":"type"},{"doc":"A problem_tuple contains all information about a failed type check. c.f. TypeCheck.TypeError.Formatter.problem_tuple for a more precise definition","ref":"TypeCheck.TypeError.html#t:problem_tuple/0","title":"TypeCheck.TypeError.problem_tuple/0","type":"type"},{"doc":"The value that was passed in which failed the check. It is included for the easy creation of value did not match y -style messages.","ref":"TypeCheck.TypeError.html#t:problematic_value/0","title":"TypeCheck.TypeError.problematic_value/0","type":"type"},{"doc":"","ref":"TypeCheck.TypeError.html#t:t/0","title":"TypeCheck.TypeError.t/0","type":"type"},{"doc":"Any built-in TypeCheck struct (c.f. TypeCheck.Builtin.* ), whose check(s) failed.","ref":"TypeCheck.TypeError.html#t:type_checked_against/0","title":"TypeCheck.TypeError.type_checked_against/0","type":"type"},{"doc":"","ref":"TypeCheck.TypeError.DefaultFormatter.html","title":"TypeCheck.TypeError.DefaultFormatter","type":"module"},{"doc":"Transforms a problem_tuple into a humanly-readable explanation string. C.f. TypeCheck.TypeError.Formatter for more information about problem tuples.","ref":"TypeCheck.TypeError.DefaultFormatter.html#do_format/1","title":"TypeCheck.TypeError.DefaultFormatter.do_format/1","type":"function"},{"doc":"Callback implementation for TypeCheck.TypeError.Formatter.format/2 .","ref":"TypeCheck.TypeError.DefaultFormatter.html#format/2","title":"TypeCheck.TypeError.DefaultFormatter.format/2","type":"function"},{"doc":"Behaviour to format your own type errors","ref":"TypeCheck.TypeError.Formatter.html","title":"TypeCheck.TypeError.Formatter","type":"behaviour"},{"doc":"A formatter is expected to turn a problem_tuple into a string that can be used as :message of the TypeCheck.TypeError exception.","ref":"TypeCheck.TypeError.Formatter.html#c:format/2","title":"TypeCheck.TypeError.Formatter.format/2","type":"callback"},{"doc":"A problem tuple contains four fields: the module of the type for which a check did not pass an atom describing the exact error; for many types there are multiple checks a map with fields containing extra information about the error. in the cases of a compound type, this often contains information about the deeper problem that happened as well. the datastructure that did not pass the check See the module documentation of all TypeCheck.Builtin.* modules for more information about the checks that they perform and the problem tuples they might return. (This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: problem_tuple :: TypeCheck.Builtin.Any . problem_tuple ( ) | TypeCheck.Builtin.Atom . problem_tuple ( ) | TypeCheck.Builtin.Binary . problem_tuple ( ) | TypeCheck.Builtin.Bitstring . problem_tuple ( ) | TypeCheck.Builtin.Boolean . problem_tuple ( ) | TypeCheck.Builtin.FixedList . problem_tuple ( ) | TypeCheck.Builtin.FixedMap . problem_tuple ( ) | TypeCheck.Builtin.CompoundFixedMap . problem_tuple ( ) | TypeCheck.Builtin.FixedTuple . problem_tuple ( ) | TypeCheck.Builtin.Float . problem_tuple ( ) | TypeCheck.Builtin.Integer . problem_tuple ( ) | TypeCheck.Builtin.List . problem_tuple ( ) | TypeCheck.Builtin.MaybeImproperList . problem_tuple ( ) | TypeCheck.Builtin.Literal . problem_tuple ( ) | TypeCheck.Builtin.Map . problem_tuple ( ) | TypeCheck.Builtin.NamedType . problem_tuple ( ) | TypeCheck.Builtin.NegInteger . problem_tuple ( ) | TypeCheck.Builtin.NonNegInteger . problem_tuple ( ) | TypeCheck.Builtin.None . problem_tuple ( ) | TypeCheck.Builtin.Number . problem_tuple ( ) | TypeCheck.Builtin.OneOf . problem_tuple ( ) | TypeCheck.Builtin.PosInteger . problem_tuple ( ) | TypeCheck.Builtin.PID . problem_tuple ( ) | TypeCheck.Builtin.Port . problem_tuple ( ) | TypeCheck.Builtin.Reference . problem_tuple ( ) | TypeCheck.Builtin.Range . problem_tuple ( ) | TypeCheck.Builtin.Tuple . problem_tuple ( ) | TypeCheck.Spec . problem_tuple ( )","ref":"TypeCheck.TypeError.Formatter.html#t:problem_tuple/0","title":"TypeCheck.TypeError.Formatter.problem_tuple/0","type":"type"},{"doc":"Transforms types into StreamData generators. With the exception of wrap_with_gen/2 , methods in this module are only compiled when the optional dependency :stream_data is added to your project's dependencies.","ref":"TypeCheck.Type.StreamData.html","title":"TypeCheck.Type.StreamData","type":"module"},{"doc":"","ref":"TypeCheck.Type.StreamData.html#arbitrary_primitive_type_gen/0","title":"TypeCheck.Type.StreamData.arbitrary_primitive_type_gen/0","type":"function"},{"doc":"","ref":"TypeCheck.Type.StreamData.html#arbitrary_type_gen/0","title":"TypeCheck.Type.StreamData.arbitrary_type_gen/0","type":"function"},{"doc":"","ref":"TypeCheck.Type.StreamData.html#fixed_list_gen/0","title":"TypeCheck.Type.StreamData.fixed_list_gen/0","type":"function"},{"doc":"When given a type, it is transformed to a StreamData generator that can be used in a property test. iex&gt; import TypeCheck.Type.StreamData iex&gt; generator = TypeCheck.Type . build ( { :ok | :error , integer ( ) } ) |&gt; to_gen ( ) iex&gt; StreamData . seeded ( generator , 42 ) |&gt; Enum . take ( 10 ) [ { :ok , - 1 } , { :ok , 2 } , { :ok , - 2 } , { :ok , - 4 } , { :ok , 1 } , { :ok , 1 } , { :ok , 2 } , { :ok , 4 } , { :ok , - 7 } , { :ok , 5 } ]","ref":"TypeCheck.Type.StreamData.html#to_gen/1","title":"TypeCheck.Type.StreamData.to_gen/1","type":"function"},{"doc":"Customizes a type with a custom generator. generator_function can be a arity-zero function (in which case it should simply return a StreamData generator) or a arity-one function, in which case it is passed the value that would be generated by default and it can be altered by e.g. using StreamData.map/2 or StreamData.bind/2 . Note that these functions must be of the form &amp;Module.function/arity because this is the only form of function capture that can be stored at compile-time. Example: iex&gt; defmodule IntString do ...&gt; use TypeCheck ...&gt; import TypeCheck.Type.StreamData ...&gt; @type! t ( ) :: ( ( val :: binary ( ) ) when Integer . parse ( val ) != :error ) ...&gt; |&gt; wrap_with_gen ( &amp; IntString . gen / 0 ) ...&gt; ...&gt; def gen ( ) do ...&gt; StreamData . integer ( ) ...&gt; |&gt; StreamData . map ( &amp; to_string / 1 ) ...&gt; end ...&gt; end ...&gt; ...&gt; IntString . t ( ) |&gt; TypeCheck.Type.StreamData . to_gen ( ) |&gt; StreamData . seeded ( 42 ) |&gt; Enum . take ( 10 ) [ &quot;0&quot; , &quot;2&quot; , &quot;1&quot; , &quot;-3&quot; , &quot;-5&quot; , &quot;-4&quot; , &quot;-3&quot; , &quot;-4&quot; , &quot;3&quot; , &quot;-6&quot; ]","ref":"TypeCheck.Type.StreamData.html#wrap_with_gen/2","title":"TypeCheck.Type.StreamData.wrap_with_gen/2","type":"function"},{"doc":"Contains TypeCheck specifications for all 'built-in' Elixir types. These are all the types described on the 'Basic Types', 'Literals' and 'Builtin Types' sections of the Elixir 'Typespecs' documentation page. See TypeCheck.DefaultOverrides for the 'Remote Types' supported by TypeCheck. Usually you'd want to import this module when you're using TypeCheck. This is done automatically when calling use TypeCheck . If necessary, feel free to hide (using import ... except: ) the things you don't need. Ommissions TypeCheck strives to implement all of the syntax and builtin types which Elixir itself also supports. Most of them are supported today. The rest will hopefully be supported in the near future. For an up-to-date comparison of what types TypeCheck does and does not support w.r.t. Elixir's builtin typespecs, see Comparison to Plain Typespecs .","ref":"TypeCheck.Builtin.html","title":"TypeCheck.Builtin","type":"module"},{"doc":"Any Elixir value. Will always succeed. c.f. TypeCheck.Builtin.Any iex&gt; TypeCheck . conforms! ( 10 , any ( ) ) 10 iex&gt; TypeCheck . conforms! ( &quot;foobar&quot; , any ( ) ) &quot;foobar&quot;","ref":"TypeCheck.Builtin.html#any/0","title":"TypeCheck.Builtin.any/0","type":"function"},{"doc":"Shorthand for range(0..255) iex&gt; TypeCheck . conforms! ( 1 , arity ( ) ) 1 iex&gt; TypeCheck . conforms! ( 1000 , arity ( ) ) ** (TypeCheck.TypeError) `1000` does not check against `0..255`. Reason: `1000` falls outside the range 0..255.","ref":"TypeCheck.Builtin.html#arity/0","title":"TypeCheck.Builtin.arity/0","type":"function"},{"doc":"The same as type , but indicates that the result will be used as a boolean. iex&gt; TypeCheck . conforms! ( :ok , as_boolean ( atom ( ) ) ) :ok iex&gt; TypeCheck . conforms! ( 10 , as_boolean ( atom ( ) ) ) ** (TypeCheck.TypeError) `10` is not an atom.","ref":"TypeCheck.Builtin.html#as_boolean/1","title":"TypeCheck.Builtin.as_boolean/1","type":"function"},{"doc":"Any Elixir atom. c.f. TypeCheck.Builtin.Atom iex&gt; TypeCheck . conforms! ( :ok , atom ( ) ) :ok iex&gt; TypeCheck . conforms! ( :foo , atom ( ) ) :foo iex&gt; TypeCheck . conforms! ( 10 , atom ( ) ) ** (TypeCheck.TypeError) `10` is not an atom.","ref":"TypeCheck.Builtin.html#atom/0","title":"TypeCheck.Builtin.atom/0","type":"function"},{"doc":"Any binary. A binary is a bitstring with a bitsize divisible by eight. c.f. TypeCheck.Builtin.Binary","ref":"TypeCheck.Builtin.html#binary/0","title":"TypeCheck.Builtin.binary/0","type":"function"},{"doc":"Any bitstring c.f. TypeCheck.Builtin.Bitstring","ref":"TypeCheck.Builtin.html#bitstring/0","title":"TypeCheck.Builtin.bitstring/0","type":"function"},{"doc":"Any boolean (either true or false .) c.f. TypeCheck.Builtin.Boolean","ref":"TypeCheck.Builtin.html#boolean/0","title":"TypeCheck.Builtin.boolean/0","type":"function"},{"doc":"A byte; shorthand for range(0..255) c.f. range/1 iex&gt; TypeCheck . conforms! ( 1 , byte ( ) ) 1 iex&gt; TypeCheck . conforms! ( 255 , byte ( ) ) 255 iex&gt; TypeCheck . conforms! ( 256 , byte ( ) ) ** (TypeCheck.TypeError) `256` does not check against `0..255`. Reason: `256` falls outside the range 0..255.","ref":"TypeCheck.Builtin.html#byte/0","title":"TypeCheck.Builtin.byte/0","type":"function"},{"doc":"A char; shorthand for range(0..0x10FFFF) c.f. range/1 iex&gt; TypeCheck . conforms! ( ?a , char ( ) ) 97 iex&gt; TypeCheck . conforms! ( - 1 , char ( ) ) ** (TypeCheck.TypeError) `-1` does not check against `0..1114111`. Reason: `-1` falls outside the range 0..1114111.","ref":"TypeCheck.Builtin.html#char/0","title":"TypeCheck.Builtin.char/0","type":"function"},{"doc":"A list filled with characters; exactly list(char()) c.f. list/1 and char/0 iex&gt; TypeCheck . conforms! ( &#39;hello world&#39; , charlist ( ) ) &#39;hello world&#39; iex&gt; TypeCheck . conforms! ( &quot;hello world&quot; , charlist ( ) ) ** (TypeCheck.TypeError) `&quot;hello world&quot;` does not check against `list(0..1114111)`. Reason: `&quot;hello world&quot;` is not a list.","ref":"TypeCheck.Builtin.html#charlist/0","title":"TypeCheck.Builtin.charlist/0","type":"function"},{"doc":"Allows constructing map types containing a combination of fixed, required and optional keys (and their associatied type-values) Desugaring of most ways of map syntaxes. Note that because of reasons of efficiency and implementation difficulty, not all possibilities are supported by TypeCheck currently. Supported are: maps with only fixed keys ( %{a: 1, b: 2, &quot;foo&quot; =&gt; number()} ) maps with a single required keypair ( %{required(key_type) =&gt; value_type} ) maps with a single optional keypair ( %{optional(key_type) =&gt; value_type} ) maps with only fixed keys and one optional keypair ( %{:a =&gt; 1, :b =&gt; 2, &quot;foo&quot; =&gt; number(), optional(integer()) =&gt; boolean()} ) Help with extending this support is very welcome. c.f. https://github.com/Qqwy/elixir-type_check/issues/7","ref":"TypeCheck.Builtin.html#fancy_map/3","title":"TypeCheck.Builtin.fancy_map/3","type":"function"},{"doc":"A list of fixed size where element_types dictates the types of each of the respective elements. Desugaring of literal lists like [:a, 10, &quot;foo&quot;] . Cannot directly be represented in Elixir's builtin Typespecs, and is thus represented as [any()] instead.","ref":"TypeCheck.Builtin.html#fixed_list/1","title":"TypeCheck.Builtin.fixed_list/1","type":"function"},{"doc":"A map with exactly the key-value-pairs indicated by keywords where all keys are required to be literal values, and the values are a type specification. Desugaring of literal maps like %{a_key: value_type, &quot;other_key&quot; =&gt; value_type2} . Represented in Elixir's builtin Typespecs as %{ required ( :a_key ) =&gt; value_type1 , required ( &quot;other key&quot; ) =&gt; value_type2 } (for e.g. a call to fixed_map([a_key: value_type1, {&quot;other key&quot;, value_type2}]) )","ref":"TypeCheck.Builtin.html#fixed_map/1","title":"TypeCheck.Builtin.fixed_map/1","type":"function"},{"doc":"A tuple whose elements are of the types given by list_of_element_types . Desugaring of writing tuples directly in your types: {a, b, c} desugars to fixed_tuple([a, b, c]) . Represented in Elixir's builtin Typespecs as a plain tuple, where each of the elements are the respective element of list_of_types . C.f. TypeCheck.Builtin.Tuple","ref":"TypeCheck.Builtin.html#fixed_tuple/1","title":"TypeCheck.Builtin.fixed_tuple/1","type":"function"},{"doc":"Any float. C.f. TypeCheck.Builtin.Float","ref":"TypeCheck.Builtin.html#float/0","title":"TypeCheck.Builtin.float/0","type":"function"},{"doc":"Alias for function/0 . iex&gt; TypeCheck . conforms! ( &amp; div / 2 , fun ( ) ) &amp; :erlang . div / 2","ref":"TypeCheck.Builtin.html#fun/0","title":"TypeCheck.Builtin.fun/0","type":"function"},{"doc":"Any function (of any arity), regardless of input or output types c.f. TypeCheck.Builtin.Function iex&gt; TypeCheck . conforms! ( &amp; div / 2 , function ( ) ) &amp; :erlang . div / 2 iex&gt; TypeCheck . conforms! ( &amp; Application . get_env / 3 , function ( ) ) &amp; Application . get_env / 3 iex&gt; TypeCheck . conforms! ( 42 , function ( ) ) ** (TypeCheck.TypeError) `42` is not a function.","ref":"TypeCheck.Builtin.html#function/0","title":"TypeCheck.Builtin.function/0","type":"function"},{"doc":"A function (of any arity) returning return_type . Desugaring of (... -&gt; return_type) See function/2 for more info. c.f. TypeCheck.Builtin.Function","ref":"TypeCheck.Builtin.html#function/1","title":"TypeCheck.Builtin.function/1","type":"function"},{"doc":"A function taking param_types as parameters, returning return_type . Desugaring of (param_type -&gt; return_type) , (param_type, param_type2 -&gt; return_type) , (param_type, param_type2, param_type3 -&gt; return_type) etc. Type-checking a function value against a function-type works a bit differently from most other types. The reason for this is that we can only ascertain whether the function-value works correctly when the function-value is called. Specifically: When a call to TypeCheck.conforms/3 (and variants) or a function wrapped with a @spec! is called, we can immediately check whether a particular parameter: is a function accepts the expected arity Then, the parameter-which-is-a-function is wrapped in a 'wrapper function' which, when called: typechecks whether the passed parameters are of the expected types ( This checks whether your function uses the parameter-function correctly .) calls the original function with the parameters. typechecks whether the result is of the expected type. ( This checks whether the parameter-function works correctly .) returns the result. In other words, the 'wrapper function' which is added for a type (param_type, param_type2 -&gt; result_type) works similarly to a named function with the spec @spec! myfunction(param_type, param_type2) :: result_type . iex&gt; # The following passes the first check... iex&gt; fun = TypeCheck . conforms! ( &amp; div / 2 , ( integer ( ) , integer ( ) -&gt; boolean ( ) ) ) iex&gt; # ... but once the function returns, the wrapper will raise iex&gt; fun . ( 20 , 5 ) ** (TypeCheck.TypeError) The call to `#Function&lt;...&gt;/2` failed, because the returned result does not adhere to the spec `boolean()`. Rather, its value is: `4`. Details: The result of calling `#Function&lt;...&gt;.(20, 5)` does not adhere to spec `(integer(), integer() -&gt; boolean())`. Reason: Returned result: `4` is not a boolean. c.f. TypeCheck.Builtin.Function","ref":"TypeCheck.Builtin.html#function/2","title":"TypeCheck.Builtin.function/2","type":"function"},{"doc":"Adds a 'type guard' to the type, which is an extra check written using arbitrary Elixir code. Desugaring of some_type when guard_code . The type guard is a check written using any Elixir code, which may refer to names set in the type using named_type/2 . If this type guard fails (by returning a non-truthy value), the type will not check. For user-friendly error-handling, don't let your type guards throw exceptions. C.f. TypeCheck.Builtin.Guarded Cannot be represented in Elixir's builtin Typespecs, and is thus represented as type (without the guard) instead.","ref":"TypeCheck.Builtin.html#guarded_by/3","title":"TypeCheck.Builtin.guarded_by/3","type":"function"},{"doc":"Syntactic sugar for pid() | port() | reference() iex&gt; TypeCheck . conforms? ( self ( ) , identifier ( ) ) true","ref":"TypeCheck.Builtin.html#identifier/0","title":"TypeCheck.Builtin.identifier/0","type":"function"},{"doc":"Checks whether the given value implements the particular protocol. For this type-check to work, Protocol Consolidation needs to be active. Data generation TypeCheck tries to generate values of any type implementing the protocol. These generators can generate any built-in type for which the protocol is implemented (with the exception of functions, and datetimes). It can also generate your custom structs, as long as: They contain a TypeCheck type called t . In this case, any values adhering to t will be generated. They don't have a t TypeCheck type, but contain a new/0 function. In this case, a single value is generated each time: the result of calling YourStructModule.new/0 . A deliberate choice was made not to automatically generate values for any module by using struct/0 , because this would not respect the @enforce_keys option that might be given to structs.","ref":"TypeCheck.Builtin.html#impl/1","title":"TypeCheck.Builtin.impl/1","type":"function"},{"doc":"Any integer. C.f. TypeCheck.Builtin.Integer iex&gt; TypeCheck . conforms! ( 42 , integer ( ) ) 42 iex&gt; TypeCheck . conforms! ( 42.0 , integer ( ) ) ** (TypeCheck.TypeError) `42.0` is not an integer. iex&gt; TypeCheck . conforms! ( &quot;hello&quot; , integer ( ) ) ** (TypeCheck.TypeError) `&quot;hello&quot;` is not an integer.","ref":"TypeCheck.Builtin.html#integer/0","title":"TypeCheck.Builtin.integer/0","type":"function"},{"doc":"Syntactic sugar for binary() | iolist()","ref":"TypeCheck.Builtin.html#iodata/0","title":"TypeCheck.Builtin.iodata/0","type":"function"},{"doc":"A potentially-improper list containing binaries, single characters, or nested iolists. Syntactic sugar for maybe_improper_list(byte() | binary() | iolist(), binary() | [])","ref":"TypeCheck.Builtin.html#iolist/0","title":"TypeCheck.Builtin.iolist/0","type":"function"},{"doc":"A list of pairs with atoms as 'keys' and anything allowed as as 'values'. Shorthand for list({atom(), any()}) iex&gt; x = [ a : 1 , b : 2 ] iex&gt; TypeCheck . conforms! ( x , keyword ( ) ) [ a : 1 , b : 2 ] iex&gt; y = [ a : 1 , b : 2 ] ++ [ 3 , 4 ] iex&gt; TypeCheck . conforms! ( y , keyword ( ) ) ** (TypeCheck.TypeError) `[{:a, 1}, {:b, 2}, 3, 4]` does not check against `list({atom(), any()})`. Reason: at index 2: `3` does not check against `{atom(), any()}`. Reason: `3` is not a tuple.","ref":"TypeCheck.Builtin.html#keyword/0","title":"TypeCheck.Builtin.keyword/0","type":"function"},{"doc":"A list of pairs with atoms as 'keys' and t's as 'values'. Shorthand for list({atom(), t})","ref":"TypeCheck.Builtin.html#keyword/1","title":"TypeCheck.Builtin.keyword/1","type":"function"},{"doc":"Defers type-expansion until the last possible moment. This is used to be able to expand recursive types. For instance, if you have the following: defmodule MyBrokenlist do type empty :: nil type cons ( a ) :: { a , mylist ( a ) } type mylist ( a ) :: empty ( ) | cons ( a ) spec new_list ( ) :: mylist ( any ( ) ) def new_list ( ) do nil end spec cons_val ( mylist ( any ( ) ) , any ( ) ) :: mylist ( any ) def cons_val ( list , val ) do { val , list } end end then when TypeCheck is expanding the spec s at compile-time to build the type-checking code, mylist(a) will call cons(a) which will call mylist(a) which will call cons(a) etc. until infinity. This makes compilation hang indefinitely. To be able to handle types like this, use lazy : defmodule MyFixedList do type empty :: nil type cons ( a ) :: { a , lazy ( mylist ( a ) ) } type mylist ( a ) :: empty ( ) | cons ( a ) spec new_list ( ) :: mylist ( any ( ) ) def new_list ( ) do nil end spec cons_val ( mylist ( any ( ) ) , any ( ) ) :: mylist ( any ) def cons_val ( list , val ) do { val , list } end end This will work as intended. Since lazy/1 defers type-expansion (and check-code-generation) until runtime, the compiler is not able to optimize the type-checking code. Thus, you should only use it when necessary, since it will be slower than when using the inner type direcly. In builtin typespecs lazy/1 does not exist in Elixir's builtin typespecs (since builtin typespecs does not expand types it does not need to handle recursive types in a special way). Therefore, lazy(some_type) is represented as some_type directly in ELixir's builtin typespecs.","ref":"TypeCheck.Builtin.html#lazy/1","title":"TypeCheck.Builtin.lazy/1","type":"macro"},{"doc":"A (proper) list with any type of elements; shorthand for list(any()) C.f. list/1 and any/0","ref":"TypeCheck.Builtin.html#list/0","title":"TypeCheck.Builtin.list/0","type":"function"},{"doc":"A (proper) list containing only elements of type a . C.f. TypeCheck.Builtin.List iex&gt; TypeCheck . conforms! ( [ 1 , 2 , 3 ] , list ( integer ( ) ) ) [ 1 , 2 , 3 ] iex&gt; TypeCheck . conforms! ( :foo , list ( integer ( ) ) ) ** (TypeCheck.TypeError) `:foo` does not check against `list(integer())`. Reason: `:foo` is not a list. iex&gt; TypeCheck . conforms! ( [ 1 , 2 , 3.3 ] , list ( integer ( ) ) ) ** (TypeCheck.TypeError) `[1, 2, 3.3]` does not check against `list(integer())`. Reason: at index 2: `3.3` is not an integer.","ref":"TypeCheck.Builtin.html#list/1","title":"TypeCheck.Builtin.list/1","type":"function"},{"doc":"A literal value. Desugaring of using any literal primitive value (like a particular integer, float, atom, binary or bitstring) directly a type. For instance, 10 desugars to literal(10) . Represented in Elixir's builtin Typespecs as for integers, atoms and booleans: the primitive value itself. for binaries, a more general binary() is used as Elixir's builtin typespecs do not support literal UTF-8 binaries as literal values. For other kinds of values which Elixir's builtin typespecs do not support as literals, we similarly represent it as a more general type. C.f. TypeCheck.Builtin.Literal","ref":"TypeCheck.Builtin.html#literal/1","title":"TypeCheck.Builtin.literal/1","type":"function"},{"doc":"Any Elixir map with any types as keys and any types as values. C.f. TypeCheck.Builtin.Map","ref":"TypeCheck.Builtin.html#map/0","title":"TypeCheck.Builtin.map/0","type":"function"},{"doc":"Any map containing zero or more keys of key_type and values of value_type . Represented in Elixir's builtin Typespecs as %{optional(key_type) =&gt; value_type} , and indeed a desugaring of this. Note that multiple optional keypairs are not (yet) supported. C.f. TypeCheck.Builtin.Map","ref":"TypeCheck.Builtin.html#map/2","title":"TypeCheck.Builtin.map/2","type":"function"},{"doc":"Syntactic sugar for maybe_improper_list(any(), any())","ref":"TypeCheck.Builtin.html#maybe_improper_list/0","title":"TypeCheck.Builtin.maybe_improper_list/0","type":"function"},{"doc":"WIP","ref":"TypeCheck.Builtin.html#maybe_improper_list/2","title":"TypeCheck.Builtin.maybe_improper_list/2","type":"function"},{"doc":"A module-function-arity tuple Module is a module/0 function is an atom/0 Arity is an arity/0 C.f. fixed_tuple/1","ref":"TypeCheck.Builtin.html#mfa/0","title":"TypeCheck.Builtin.mfa/0","type":"function"},{"doc":"Any Elixir atom, but indicates that the atom is expected to be used as a module. iex&gt; TypeCheck . conforms! ( String , module ( ) ) String iex&gt; TypeCheck . conforms! ( :array , module ( ) ) :array iex&gt; TypeCheck . conforms! ( &quot;hello&quot; , module ( ) ) ** (TypeCheck.TypeError) `&quot;hello&quot;` is not an atom. c.f. atom/0","ref":"TypeCheck.Builtin.html#module/0","title":"TypeCheck.Builtin.module/0","type":"function"},{"doc":"A type with a local name, which can be referred to from a 'type guard'. This name can be used in 'type guards'. See the module documentation and guarded_by/2 for more information. Desugaring of name :: type (when :: is used inside a type.). Cannot directly be represented in Elixir's builtin Typespecs, and is thus represented as type (without the name) instead.","ref":"TypeCheck.Builtin.html#named_type/4","title":"TypeCheck.Builtin.named_type/4","type":"function"},{"doc":"Any integer smaller than zero. C.f. TypeCheck.Builtin.NegInteger","ref":"TypeCheck.Builtin.html#neg_integer/0","title":"TypeCheck.Builtin.neg_integer/0","type":"function"},{"doc":"See none/0 .","ref":"TypeCheck.Builtin.html#no_return/0","title":"TypeCheck.Builtin.no_return/0","type":"function"},{"doc":"Any integer zero or larger. C.f. TypeCheck.Builtin.NonNegInteger","ref":"TypeCheck.Builtin.html#non_neg_integer/0","title":"TypeCheck.Builtin.non_neg_integer/0","type":"function"},{"doc":"Matches no value at all. none() is not very useful on its own, but it is a useful default in certain circumstances, as well as to indicate that you expect some place to not return at all. (instead for instance throwing an exception or looping forever.) C.f. TypeCheck.Builtin.None .","ref":"TypeCheck.Builtin.html#none/0","title":"TypeCheck.Builtin.none/0","type":"function"},{"doc":"A binary which contains at least one byte. Shorthand for sized_bitstring(8, 8) .","ref":"TypeCheck.Builtin.html#nonempty_binary/0","title":"TypeCheck.Builtin.nonempty_binary/0","type":"function"},{"doc":"A bitstring which contains at least one bit. Shorthand for sized_bitstring(1, 1) .","ref":"TypeCheck.Builtin.html#nonempty_bitstring/0","title":"TypeCheck.Builtin.nonempty_bitstring/0","type":"function"},{"doc":"Shorthand for nonempty_list(char()).","ref":"TypeCheck.Builtin.html#nonempty_charlist/0","title":"TypeCheck.Builtin.nonempty_charlist/0","type":"function"},{"doc":"Any list with at least one element, which has to be terminated by something else than [] . To be precise, the list needs to be terminated with terminator_type .","ref":"TypeCheck.Builtin.html#nonempty_improper_list/2","title":"TypeCheck.Builtin.nonempty_improper_list/2","type":"function"},{"doc":"Shorthand for nonempty_list(any()).","ref":"TypeCheck.Builtin.html#nonempty_list/0","title":"TypeCheck.Builtin.nonempty_list/0","type":"function"},{"doc":"A nonempty_list is any list with at least one element.","ref":"TypeCheck.Builtin.html#nonempty_list/1","title":"TypeCheck.Builtin.nonempty_list/1","type":"function"},{"doc":"Any list with at least one element, which might be terminated by something else than [] . To be precise, the list needs to be terminated with either [] or terminator_type","ref":"TypeCheck.Builtin.html#nonempty_maybe_improper_list/2","title":"TypeCheck.Builtin.nonempty_maybe_improper_list/2","type":"function"},{"doc":"Any number (either a float or an integer) Matches the same as integer | float but is more efficient. C.f. TypeCheck.Builtin.Number","ref":"TypeCheck.Builtin.html#number/0","title":"TypeCheck.Builtin.number/0","type":"function"},{"doc":"Version of one_of that allows passing many possibilities at once. A union of multiple types (also known as a 'sum type') Desugaring of types separated by | like a | b or a | b | c | d . (and represented that way in Elixir's builtin Typespecs). c.f. one_of/2 .","ref":"TypeCheck.Builtin.html#one_of/1","title":"TypeCheck.Builtin.one_of/1","type":"function"},{"doc":"A union of multiple types (also known as a 'sum type') Desugaring of types separated by | like a | b or a | b | c | d . (and represented that way in Elixir's builtin Typespecs).","ref":"TypeCheck.Builtin.html#one_of/2","title":"TypeCheck.Builtin.one_of/2","type":"function"},{"doc":"Matches any process-identifier. Note that no checks are made to see whether the process is alive or not. Also, the current property-generator will generate arbitrary PIDs, most of which will not point to alive processes.","ref":"TypeCheck.Builtin.html#pid/0","title":"TypeCheck.Builtin.pid/0","type":"function"},{"doc":"Matches any port. c.f. TypeCheck.Builtin.Port iex&gt; TypeCheck . conforms? ( Kernel . make_ref ( ) , reference ( ) ) true iex&gt; some_port = Port . open ( { :spawn , &quot;cat&quot; } , [ :binary ] ) ...&gt; TypeCheck . conforms? ( some_port , port ( ) ) true","ref":"TypeCheck.Builtin.html#port/0","title":"TypeCheck.Builtin.port/0","type":"function"},{"doc":"Any integer larger than zero. C.f. TypeCheck.Builtin.PosInteger","ref":"TypeCheck.Builtin.html#pos_integer/0","title":"TypeCheck.Builtin.pos_integer/0","type":"function"},{"doc":"Any integer in the half-open range range . Desugaring of a..b . (And represented that way in Elixir's builtin Typespecs.) C.f. TypeCheck.Builtin.Range","ref":"TypeCheck.Builtin.html#range/1","title":"TypeCheck.Builtin.range/1","type":"function"},{"doc":"Any integer between lower (includsive) and higher (exclusive). Desugaring of lower..higher . (And represented that way in Elixir's builtin Typespecs.) C.f. range/1","ref":"TypeCheck.Builtin.html#range/2","title":"TypeCheck.Builtin.range/2","type":"function"},{"doc":"Matches any reference. c.f. TypeCheck.Builtin.Reference iex&gt; TypeCheck . conforms? ( Kernel . make_ref ( ) , reference ( ) ) true iex&gt; some_ref = IEx.Helpers . ref ( 0 , 749884137 , 111673345 , 43386 ) ...&gt; TypeCheck . conforms! ( some_ref , reference ( ) ) # Reference &lt; 0.749884137 . 111673345.43386 &gt;","ref":"TypeCheck.Builtin.html#reference/0","title":"TypeCheck.Builtin.reference/0","type":"function"},{"doc":"A bitstring of fixed size. Desugaring of bitstring types like &lt;&lt; _ :: size&gt;&gt; . c.f. TypeCheck.Builtin.SizedBitstring .","ref":"TypeCheck.Builtin.html#sized_bitstring/1","title":"TypeCheck.Builtin.sized_bitstring/1","type":"function"},{"doc":"A bitstring with a fixed prefix_size (which might be 0 ), followed by zero or repetitions of unit_size . Desugaring of bitstring types like &lt;&lt; _ :: _ * unit_size&gt;&gt; and &lt;&lt; _ :: prefix_size, _ :: _ * unit_size&gt;&gt; . iex&gt; TypeCheck . conforms! ( &quot;hi&quot; , &lt;&lt; _ :: 16 &gt;&gt; ) &quot;hi&quot; iex&gt; TypeCheck . conforms! ( &quot;bye&quot; , &lt;&lt; _ :: 16 &gt;&gt; ) ** (TypeCheck.TypeError) `&quot;bye&quot;` has a different bit_size (24) than expected (16). iex&gt; TypeCheck . conforms! ( &lt;&lt; 1 :: size ( 2 ) &gt;&gt; , &lt;&lt; _ :: 2 &gt;&gt; ) &lt;&lt; 1 :: size ( 2 ) &gt;&gt; iex&gt; TypeCheck . conforms! ( &lt;&lt; 1 :: size ( 3 ) &gt;&gt; , &lt;&lt; _ :: 2 &gt;&gt; ) ** (TypeCheck.TypeError) `&lt;&lt;1::size(3)&gt;&gt;` has a different bit_size (3) than expected (2). iex&gt; [ &quot;ab&quot; , &quot;abcd&quot; , &quot;abcdef&quot; ] |&gt; Enum . map ( &amp; TypeCheck . conforms! ( &amp;1 , &lt;&lt; _ :: _ * 16 &gt;&gt; ) ) [ &quot;ab&quot; , &quot;abcd&quot; , &quot;abcdef&quot; ] iex&gt; TypeCheck . conforms! ( &quot;abc&quot; , &lt;&lt; _ :: _ * 16 &gt;&gt; ) ** (TypeCheck.TypeError) `&quot;abc&quot;` has a different bit_size (24) than expected (_ * 16). iex&gt; [ &quot;a&quot; , &quot;abc&quot; , &quot;abcde&quot; ] |&gt; Enum . map ( &amp; TypeCheck . conforms! ( &amp;1 , &lt;&lt; _ :: 8 , _ :: _ * 16 &gt;&gt; ) ) [ &quot;a&quot; , &quot;abc&quot; , &quot;abcde&quot; ] iex&gt; TypeCheck . conforms! ( &quot;ab&quot; , &lt;&lt; _ :: 8 , _ :: _ * 16 &gt;&gt; ) ** (TypeCheck.TypeError) `&quot;ab&quot;` has a different bit_size (16) than expected (8 + _ * 16). c.f. TypeCheck.Builtin.SizedBitstring .","ref":"TypeCheck.Builtin.html#sized_bitstring/2","title":"TypeCheck.Builtin.sized_bitstring/2","type":"function"},{"doc":"Any kind of struct. Syntactic sugar for %{: struct =&gt; atom(), optional(atom()) =&gt; any()}","ref":"TypeCheck.Builtin.html#struct/0","title":"TypeCheck.Builtin.struct/0","type":"function"},{"doc":"alias for any/0","ref":"TypeCheck.Builtin.html#term/0","title":"TypeCheck.Builtin.term/0","type":"function"},{"doc":"Builtin type. Syntactic sugar for :infinity | non_neg_integer()","ref":"TypeCheck.Builtin.html#timeout/0","title":"TypeCheck.Builtin.timeout/0","type":"function"},{"doc":"A tuple of any size (with any elements). C.f. TypeCheck.Builtin.Tuple","ref":"TypeCheck.Builtin.html#tuple/0","title":"TypeCheck.Builtin.tuple/0","type":"function"},{"doc":"A tuple whose elements have any types, but which has exactly size elements. Represented in Elixir's builtin Typespecs as a plain tuple, with size elements, where each of the element types is any() . For instance, tuple(3) is represented as {any(), any(), any()} .","ref":"TypeCheck.Builtin.html#tuple/1","title":"TypeCheck.Builtin.tuple/1","type":"function"},{"doc":"","ref":"TypeCheck.Builtin.Any.html","title":"TypeCheck.Builtin.Any","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: problem_tuple :: none ( )","ref":"TypeCheck.Builtin.Any.html#t:problem_tuple/0","title":"TypeCheck.Builtin.Any.problem_tuple/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t (opaque type)","ref":"TypeCheck.Builtin.Any.html#t:t/0","title":"TypeCheck.Builtin.Any.t/0","type":"opaque"},{"doc":"Checks whether the value is any atom. Returns a problem tuple with the reason :no_match otherwise.","ref":"TypeCheck.Builtin.Atom.html","title":"TypeCheck.Builtin.Atom","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: problem_tuple :: { t ( ) , :no_match , %{ } , any ( ) }","ref":"TypeCheck.Builtin.Atom.html#t:problem_tuple/0","title":"TypeCheck.Builtin.Atom.problem_tuple/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t :: % TypeCheck.Builtin.Atom { }","ref":"TypeCheck.Builtin.Atom.html#t:t/0","title":"TypeCheck.Builtin.Atom.t/0","type":"type"},{"doc":"","ref":"TypeCheck.Builtin.Binary.html","title":"TypeCheck.Builtin.Binary","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: problem_tuple :: { t ( ) , :no_match , %{ } , any ( ) }","ref":"TypeCheck.Builtin.Binary.html#t:problem_tuple/0","title":"TypeCheck.Builtin.Binary.problem_tuple/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t :: % TypeCheck.Builtin.Binary { }","ref":"TypeCheck.Builtin.Binary.html#t:t/0","title":"TypeCheck.Builtin.Binary.t/0","type":"type"},{"doc":"","ref":"TypeCheck.Builtin.Bitstring.html","title":"TypeCheck.Builtin.Bitstring","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: problem_tuple :: { t ( ) , :no_match , %{ } , any ( ) }","ref":"TypeCheck.Builtin.Bitstring.html#t:problem_tuple/0","title":"TypeCheck.Builtin.Bitstring.problem_tuple/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t :: % TypeCheck.Builtin.Bitstring { }","ref":"TypeCheck.Builtin.Bitstring.html#t:t/0","title":"TypeCheck.Builtin.Bitstring.t/0","type":"type"},{"doc":"","ref":"TypeCheck.Builtin.Boolean.html","title":"TypeCheck.Builtin.Boolean","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: problem_tuple :: { t ( ) , :no_match , %{ } , any ( ) }","ref":"TypeCheck.Builtin.Boolean.html#t:problem_tuple/0","title":"TypeCheck.Builtin.Boolean.problem_tuple/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t :: % TypeCheck.Builtin.Boolean { }","ref":"TypeCheck.Builtin.Boolean.html#t:t/0","title":"TypeCheck.Builtin.Boolean.t/0","type":"type"},{"doc":"Special type for map-typespecs that contain a combination of fixed keys as well as an optional(...) or required(...) part. Its checks compile down to a combination of TypeCheck.Builtin.FixedMap and TypeCheck.Builtin.Map for the fixed resp. non-fixed parts.","ref":"TypeCheck.Builtin.CompoundFixedMap.html","title":"TypeCheck.Builtin.CompoundFixedMap","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: problem_tuple :: { t ( ) , :not_a_map , %{ } , any ( ) } | { t ( ) , :missing_keys , %{ keys : list ( atom ( ) ) } , map ( ) } | { t ( ) , :superfluous_keys , %{ keys : list ( atom ( ) ) } , map ( ) } | { t ( ) , :value_error , %{ problem : lazy ( TypeCheck.TypeError.Formatter . problem_tuple ( ) ) , key : any ( ) } , map ( ) } | { t ( ) , :key_error , %{ problem : lazy ( TypeCheck.TypeError.Formatter . problem_tuple ( ) ) , key : any ( ) } , map ( ) }","ref":"TypeCheck.Builtin.CompoundFixedMap.html#t:problem_tuple/0","title":"TypeCheck.Builtin.CompoundFixedMap.problem_tuple/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t :: % TypeCheck.Builtin.CompoundFixedMap { fixed : TypeCheck.Builtin.FixedMap . t ( ) , flexible : TypeCheck.Builtin.Map . t ( ) | TypeCheck.Builtin.OptionalFixedMap . t ( ) }","ref":"TypeCheck.Builtin.CompoundFixedMap.html#t:t/0","title":"TypeCheck.Builtin.CompoundFixedMap.t/0","type":"type"},{"doc":"Checks whether the value is a list with the expected elements On failure returns a problem tuple with: :not_a_list if the value is not a list :different_length if the value is a list but not of equal size. :element_error if one of the elements does not match. The extra information contains in this case :problem and :index to indicate what and where the problem occured.","ref":"TypeCheck.Builtin.FixedList.html","title":"TypeCheck.Builtin.FixedList","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: problem_tuple :: { t ( ) , :not_a_list , %{ } , any ( ) } | { t ( ) , :different_length , %{ expected_length : non_neg_integer ( ) } , list ( ) } | { t ( ) , :element_error , %{ problem : lazy ( TypeCheck.TypeError.Formatter . problem_tuple ( ) ) , index : non_neg_integer ( ) } , list ( ) }","ref":"TypeCheck.Builtin.FixedList.html#t:problem_tuple/0","title":"TypeCheck.Builtin.FixedList.problem_tuple/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t :: % TypeCheck.Builtin.FixedList { element_types : list ( TypeCheck.Type . t ( ) ) }","ref":"TypeCheck.Builtin.FixedList.html#t:t/0","title":"TypeCheck.Builtin.FixedList.t/0","type":"type"},{"doc":"Checks whether the value is a map with a required set of keys On failure returns a problem tuple with: :not_a_map if the value is not a map :missing_keys if the value does not have all of the expected keys. The extra information contains in this case :keys with a list of keys that are missing. :superfluous_keys if the value have any keys other than the expected keys. The extra information contains in this case :keys with a list of keys that are superfluous. :value_error if one of the elements does not match. The extra information contains in this case :problem and :key to indicate what and where the problem occured.","ref":"TypeCheck.Builtin.FixedMap.html","title":"TypeCheck.Builtin.FixedMap","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: problem_tuple :: { t ( ) , :not_a_map , %{ } , any ( ) } | { t ( ) , :missing_keys , %{ keys : list ( atom ( ) ) } , map ( ) } | { t ( ) , :superfluous_keys , %{ keys : list ( atom ( ) ) } , map ( ) } | { t ( ) , :value_error , %{ problem : lazy ( TypeCheck.TypeError.Formatter . problem_tuple ( ) ) , key : any ( ) } , map ( ) }","ref":"TypeCheck.Builtin.FixedMap.html#t:problem_tuple/0","title":"TypeCheck.Builtin.FixedMap.problem_tuple/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t :: % TypeCheck.Builtin.FixedMap { keypairs : list ( { term ( ) , TypeCheck.Type . t ( ) } ) }","ref":"TypeCheck.Builtin.FixedMap.html#t:t/0","title":"TypeCheck.Builtin.FixedMap.t/0","type":"type"},{"doc":"","ref":"TypeCheck.Builtin.FixedTuple.html","title":"TypeCheck.Builtin.FixedTuple","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: problem_tuple :: { t ( ) , :not_a_tuple , %{ } , any ( ) } | { t ( ) , :different_size , %{ expected_size : integer ( ) } , tuple ( ) } | { t ( ) , :element_error , %{ problem : lazy ( TypeCheck.TypeError.Formatter . problem_tuple ( ) ) , index : integer ( ) } , tuple ( ) }","ref":"TypeCheck.Builtin.FixedTuple.html#t:problem_tuple/0","title":"TypeCheck.Builtin.FixedTuple.problem_tuple/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t :: % TypeCheck.Builtin.FixedTuple { element_types : list ( TypeCheck.Type . t ( ) ) }","ref":"TypeCheck.Builtin.FixedTuple.html#t:t/0","title":"TypeCheck.Builtin.FixedTuple.t/0","type":"type"},{"doc":"","ref":"TypeCheck.Builtin.Float.html","title":"TypeCheck.Builtin.Float","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: problem_tuple :: { t ( ) , :no_match , %{ } , any ( ) }","ref":"TypeCheck.Builtin.Float.html#t:problem_tuple/0","title":"TypeCheck.Builtin.Float.problem_tuple/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t :: % TypeCheck.Builtin.Float { }","ref":"TypeCheck.Builtin.Float.html#t:t/0","title":"TypeCheck.Builtin.Float.t/0","type":"type"},{"doc":"","ref":"TypeCheck.Builtin.Function.html","title":"TypeCheck.Builtin.Function","type":"module"},{"doc":"","ref":"TypeCheck.Builtin.Function.html#contravariant_wrapper/2","title":"TypeCheck.Builtin.Function.contravariant_wrapper/2","type":"function"},{"doc":"","ref":"TypeCheck.Builtin.Function.html#param_check_code/3","title":"TypeCheck.Builtin.Function.param_check_code/3","type":"function"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: problem_tuple :: { t ( ) , :no_match , %{ } , any ( ) }","ref":"TypeCheck.Builtin.Function.html#t:problem_tuple/0","title":"TypeCheck.Builtin.Function.problem_tuple/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t (opaque type)","ref":"TypeCheck.Builtin.Function.html#t:t/0","title":"TypeCheck.Builtin.Function.t/0","type":"opaque"},{"doc":"","ref":"TypeCheck.Builtin.Guarded.html","title":"TypeCheck.Builtin.Guarded","type":"module"},{"doc":"","ref":"TypeCheck.Builtin.Guarded.html#ast_gen/1","title":"TypeCheck.Builtin.Guarded.ast_gen/1","type":"function"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: ast ( ) :: wrap_with_gen ( term ( ) , &amp; TypeCheck.Builtin.Guarded . ast_gen / 1 )","ref":"TypeCheck.Builtin.Guarded.html#t:ast/0","title":"TypeCheck.Builtin.Guarded.ast/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t ( ) :: % TypeCheck.Builtin.Guarded { type : TypeCheck.Type . t ( ) , guard : ast ( ) , original_module : module ( ) | nil }","ref":"TypeCheck.Builtin.Guarded.html#t:t/0","title":"TypeCheck.Builtin.Guarded.t/0","type":"type"},{"doc":"Checks whether there is a protocol implementation for this value. Returns a problem tuple with the reason :no_match otherwise.","ref":"TypeCheck.Builtin.ImplementsProtocol.html","title":"TypeCheck.Builtin.ImplementsProtocol","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: problem_tuple :: { t ( ) , :no_match , %{ } , any ( ) }","ref":"TypeCheck.Builtin.ImplementsProtocol.html#t:problem_tuple/0","title":"TypeCheck.Builtin.ImplementsProtocol.problem_tuple/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t :: % TypeCheck.Builtin.ImplementsProtocol { protocol : module ( ) }","ref":"TypeCheck.Builtin.ImplementsProtocol.html#t:t/0","title":"TypeCheck.Builtin.ImplementsProtocol.t/0","type":"type"},{"doc":"","ref":"TypeCheck.Builtin.Integer.html","title":"TypeCheck.Builtin.Integer","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: problem_tuple :: { t ( ) , :no_match , %{ } , any ( ) }","ref":"TypeCheck.Builtin.Integer.html#t:problem_tuple/0","title":"TypeCheck.Builtin.Integer.problem_tuple/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t :: % TypeCheck.Builtin.Integer { }","ref":"TypeCheck.Builtin.Integer.html#t:t/0","title":"TypeCheck.Builtin.Integer.t/0","type":"type"},{"doc":"","ref":"TypeCheck.Builtin.Lazy.html","title":"TypeCheck.Builtin.Lazy","type":"module"},{"doc":"","ref":"TypeCheck.Builtin.Lazy.html#lazily_expand_type/1","title":"TypeCheck.Builtin.Lazy.lazily_expand_type/1","type":"function"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: problem_tuple :: TypeCheck.TypeError.Formatter . problem_tuple ( )","ref":"TypeCheck.Builtin.Lazy.html#t:problem_tuple/0","title":"TypeCheck.Builtin.Lazy.problem_tuple/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t :: % TypeCheck.Builtin.Lazy { module : module ( ) , function : atom ( ) , arguments : list ( term ( ) ) }","ref":"TypeCheck.Builtin.Lazy.html#t:t/0","title":"TypeCheck.Builtin.Lazy.t/0","type":"type"},{"doc":"","ref":"TypeCheck.Builtin.List.html","title":"TypeCheck.Builtin.List","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: problem_tuple :: { t ( ) , :not_a_list , %{ } , any ( ) } | { t ( ) , :element_error , %{ problem : lazy ( TypeCheck.TypeError.Formatter . problem_tuple ( ) ) , index : non_neg_integer ( ) } , any ( ) }","ref":"TypeCheck.Builtin.List.html#t:problem_tuple/0","title":"TypeCheck.Builtin.List.problem_tuple/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t :: t ( TypeCheck.Type . t ( ) )","ref":"TypeCheck.Builtin.List.html#t:t/0","title":"TypeCheck.Builtin.List.t/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t ( element_type ) :: % TypeCheck.Builtin.List { element_type : element_type }","ref":"TypeCheck.Builtin.List.html#t:t/1","title":"TypeCheck.Builtin.List.t/1","type":"type"},{"doc":"","ref":"TypeCheck.Builtin.Literal.html","title":"TypeCheck.Builtin.Literal","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: problem_tuple :: { t ( ) , :not_same_value , %{ } , value :: term ( ) }","ref":"TypeCheck.Builtin.Literal.html#t:problem_tuple/0","title":"TypeCheck.Builtin.Literal.problem_tuple/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t :: % TypeCheck.Builtin.Literal { value : term ( ) }","ref":"TypeCheck.Builtin.Literal.html#t:t/0","title":"TypeCheck.Builtin.Literal.t/0","type":"type"},{"doc":"","ref":"TypeCheck.Builtin.Map.html","title":"TypeCheck.Builtin.Map","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: problem_tuple :: { t ( ) , :not_a_map , %{ } , any ( ) } | { t ( ) , :key_error , %{ problem : lazy ( TypeCheck.TypeError.Formatter . problem_tuple ( ) ) , key : any ( ) } , any ( ) } | { t ( ) , :value_error , %{ problem : lazy ( TypeCheck.TypeError.Formatter . problem_tuple ( ) ) , key : any ( ) } , any ( ) }","ref":"TypeCheck.Builtin.Map.html#t:problem_tuple/0","title":"TypeCheck.Builtin.Map.problem_tuple/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t (opaque type)","ref":"TypeCheck.Builtin.Map.html#t:t/0","title":"TypeCheck.Builtin.Map.t/0","type":"opaque"},{"doc":"","ref":"TypeCheck.Builtin.MaybeImproperList.html","title":"TypeCheck.Builtin.MaybeImproperList","type":"module"},{"doc":"","ref":"TypeCheck.Builtin.MaybeImproperList.html#empty?/1","title":"TypeCheck.Builtin.MaybeImproperList.empty?/1","type":"function"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: problem_tuple :: { t ( ) , :not_a_list , %{ } , any ( ) } | { t ( ) , :element_error , %{ problem : lazy ( TypeCheck.TypeError.Formatter . problem_tuple ( ) ) , index : non_neg_integer ( ) } , any ( ) } | { t ( ) , :terminator_error , %{ problem : lazy ( TypeCheck.TypeError.Formatter . problem_tuple ( ) ) } , any ( ) }","ref":"TypeCheck.Builtin.MaybeImproperList.html#t:problem_tuple/0","title":"TypeCheck.Builtin.MaybeImproperList.problem_tuple/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t ( ) :: t ( TypeCheck.Type . t ( ) , TypeCheck.Type . t ( ) )","ref":"TypeCheck.Builtin.MaybeImproperList.html#t:t/0","title":"TypeCheck.Builtin.MaybeImproperList.t/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t ( element_type , terminator_type ) :: % TypeCheck.Builtin.MaybeImproperList { element_type : element_type , terminator_type : terminator_type }","ref":"TypeCheck.Builtin.MaybeImproperList.html#t:t/2","title":"TypeCheck.Builtin.MaybeImproperList.t/2","type":"type"},{"doc":"","ref":"TypeCheck.Builtin.NamedType.html","title":"TypeCheck.Builtin.NamedType","type":"module"},{"doc":"","ref":"TypeCheck.Builtin.NamedType.html#stringify_name/2","title":"TypeCheck.Builtin.NamedType.stringify_name/2","type":"function"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: problem_tuple :: { t ( ) , :named_type , %{ problem : lazy ( TypeCheck.TypeError.Formatter . problem_tuple ( ) ) } , any ( ) }","ref":"TypeCheck.Builtin.NamedType.html#t:problem_tuple/0","title":"TypeCheck.Builtin.NamedType.problem_tuple/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t :: % TypeCheck.Builtin.NamedType { name : atom ( ) , type : TypeCheck.Type . t ( ) , local : boolean ( ) , type_kind : :type | :typep | :opaque , called_as : nil | { atom ( ) , list ( any ( ) ) } }","ref":"TypeCheck.Builtin.NamedType.html#t:t/0","title":"TypeCheck.Builtin.NamedType.t/0","type":"type"},{"doc":"","ref":"TypeCheck.Builtin.NegInteger.html","title":"TypeCheck.Builtin.NegInteger","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: problem_tuple :: { t ( ) , :no_match , %{ } , any ( ) }","ref":"TypeCheck.Builtin.NegInteger.html#t:problem_tuple/0","title":"TypeCheck.Builtin.NegInteger.problem_tuple/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t :: % TypeCheck.Builtin.NegInteger { }","ref":"TypeCheck.Builtin.NegInteger.html#t:t/0","title":"TypeCheck.Builtin.NegInteger.t/0","type":"type"},{"doc":"","ref":"TypeCheck.Builtin.NonNegInteger.html","title":"TypeCheck.Builtin.NonNegInteger","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: problem_tuple :: { t ( ) , :no_match , %{ } , any ( ) }","ref":"TypeCheck.Builtin.NonNegInteger.html#t:problem_tuple/0","title":"TypeCheck.Builtin.NonNegInteger.problem_tuple/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t :: % TypeCheck.Builtin.NonNegInteger { }","ref":"TypeCheck.Builtin.NonNegInteger.html#t:t/0","title":"TypeCheck.Builtin.NonNegInteger.t/0","type":"type"},{"doc":"The 'none' type has no inhabitants. In other words, no value will typecheck against this type. This means that we always return a problem tuple with :no_match as reason in its check. It also means that the StreamData-generator will not generate any values; instead, it will filter away values that would have been produced by none() , meaning that if you attempt to use none() directly in a generator, you might get a StreamData.FilterTooNarrowError . However, it's still possible to combine it with other types like :ok | :error | none() and e.g. use the resulting generator of that.","ref":"TypeCheck.Builtin.None.html","title":"TypeCheck.Builtin.None","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: problem_tuple :: { t ( ) , :no_match , %{ } , val :: any ( ) }","ref":"TypeCheck.Builtin.None.html#t:problem_tuple/0","title":"TypeCheck.Builtin.None.problem_tuple/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t :: % TypeCheck.Builtin.None { }","ref":"TypeCheck.Builtin.None.html#t:t/0","title":"TypeCheck.Builtin.None.t/0","type":"type"},{"doc":"","ref":"TypeCheck.Builtin.Number.html","title":"TypeCheck.Builtin.Number","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: problem_tuple :: { t ( ) , :no_match , %{ } , val :: any ( ) }","ref":"TypeCheck.Builtin.Number.html#t:problem_tuple/0","title":"TypeCheck.Builtin.Number.problem_tuple/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t :: % TypeCheck.Builtin.Number { }","ref":"TypeCheck.Builtin.Number.html#t:t/0","title":"TypeCheck.Builtin.Number.t/0","type":"type"},{"doc":"","ref":"TypeCheck.Builtin.OneOf.html","title":"TypeCheck.Builtin.OneOf","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: problem_tuple :: { t ( ) , :all_failed , %{ problems : list ( lazy ( TypeCheck.TypeError.Formatter . problem_tuple ( ) ) ) } , term ( ) }","ref":"TypeCheck.Builtin.OneOf.html#t:problem_tuple/0","title":"TypeCheck.Builtin.OneOf.problem_tuple/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t ( ) :: % TypeCheck.Builtin.OneOf { choices : list ( TypeCheck.Type . t ( ) ) }","ref":"TypeCheck.Builtin.OneOf.html#t:t/0","title":"TypeCheck.Builtin.OneOf.t/0","type":"type"},{"doc":"Checks whether the value is a map with a optional set of keys On failure returns a problem tuple with: :not_a_map if the value is not a map :superfluous_keys if the value have any keys other than the expected keys. The extra information contains in this case :keys with a list of keys that are superfluous. :value_error if one of the elements does not match. The extra information contains in this case :problem and :key to indicate what and where the problem occured.","ref":"TypeCheck.Builtin.OptionalFixedMap.html","title":"TypeCheck.Builtin.OptionalFixedMap","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: problem_tuple :: { t ( ) , :not_a_map , %{ } , any ( ) } | { t ( ) , :superfluous_keys , %{ keys : list ( atom ( ) ) } , map ( ) } | { t ( ) , :value_error , %{ problem : lazy ( TypeCheck.TypeError.Formatter . problem_tuple ( ) ) , key : any ( ) } , map ( ) }","ref":"TypeCheck.Builtin.OptionalFixedMap.html#t:problem_tuple/0","title":"TypeCheck.Builtin.OptionalFixedMap.problem_tuple/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t :: % TypeCheck.Builtin.OptionalFixedMap { keypairs : list ( { term ( ) , TypeCheck.Type . t ( ) } ) }","ref":"TypeCheck.Builtin.OptionalFixedMap.html#t:t/0","title":"TypeCheck.Builtin.OptionalFixedMap.t/0","type":"type"},{"doc":"","ref":"TypeCheck.Builtin.PID.html","title":"TypeCheck.Builtin.PID","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: problem_tuple :: { t ( ) , :no_match , %{ } , any ( ) }","ref":"TypeCheck.Builtin.PID.html#t:problem_tuple/0","title":"TypeCheck.Builtin.PID.problem_tuple/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t :: % TypeCheck.Builtin.PID { }","ref":"TypeCheck.Builtin.PID.html#t:t/0","title":"TypeCheck.Builtin.PID.t/0","type":"type"},{"doc":"Type to check whether the given input is any port. Check Port for more information on ports. NOTE: The property testing generator will generate ports of the cat binary which is a sensible default as it will send back any binaries sent to it exactly. However, note that generated ports are not automatically closed.","ref":"TypeCheck.Builtin.Port.html","title":"TypeCheck.Builtin.Port","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: problem_tuple :: { t ( ) , :no_match , %{ } , any ( ) }","ref":"TypeCheck.Builtin.Port.html#t:problem_tuple/0","title":"TypeCheck.Builtin.Port.problem_tuple/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t :: % TypeCheck.Builtin.Port { }","ref":"TypeCheck.Builtin.Port.html#t:t/0","title":"TypeCheck.Builtin.Port.t/0","type":"type"},{"doc":"","ref":"TypeCheck.Builtin.PosInteger.html","title":"TypeCheck.Builtin.PosInteger","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: problem_tuple :: { t ( ) , :no_match , %{ } , any ( ) }","ref":"TypeCheck.Builtin.PosInteger.html#t:problem_tuple/0","title":"TypeCheck.Builtin.PosInteger.problem_tuple/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t :: % TypeCheck.Builtin.PosInteger { }","ref":"TypeCheck.Builtin.PosInteger.html#t:t/0","title":"TypeCheck.Builtin.PosInteger.t/0","type":"type"},{"doc":"","ref":"TypeCheck.Builtin.Range.html","title":"TypeCheck.Builtin.Range","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: problem_tuple :: { t ( ) , :not_an_integer , %{ } , any ( ) } | { t ( ) , :not_in_range , %{ } , integer ( ) }","ref":"TypeCheck.Builtin.Range.html#t:problem_tuple/0","title":"TypeCheck.Builtin.Range.problem_tuple/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t :: % TypeCheck.Builtin.Range { range : % Range { first : integer ( ) , last : integer ( ) , step : 1 } }","ref":"TypeCheck.Builtin.Range.html#t:t/0","title":"TypeCheck.Builtin.Range.t/0","type":"type"},{"doc":"Type to check whether the given input is a reference. Elixir/Erlang uses references for two use-cases: As unique identifiers. To refer to resources created and returned by NIFs (to be passed to other NIFs of the same NIF module). The property testing generator will generate arbitrary references using Kernel.make_ref() . To property-test the second kind of data, you should create your own kind of generator that calls the appropriate NIF.","ref":"TypeCheck.Builtin.Reference.html","title":"TypeCheck.Builtin.Reference","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: problem_tuple :: { t ( ) , :no_match , %{ } , any ( ) }","ref":"TypeCheck.Builtin.Reference.html#t:problem_tuple/0","title":"TypeCheck.Builtin.Reference.problem_tuple/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t :: % TypeCheck.Builtin.Reference { }","ref":"TypeCheck.Builtin.Reference.html#t:t/0","title":"TypeCheck.Builtin.Reference.t/0","type":"type"},{"doc":"","ref":"TypeCheck.Builtin.SizedBitstring.html","title":"TypeCheck.Builtin.SizedBitstring","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: problem_tuple :: { t ( ) , :no_match , %{ } , any ( ) } | { t ( ) , :wrong_size , %{ } , any ( ) }","ref":"TypeCheck.Builtin.SizedBitstring.html#t:problem_tuple/0","title":"TypeCheck.Builtin.SizedBitstring.problem_tuple/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t :: % TypeCheck.Builtin.SizedBitstring { prefix_size : non_neg_integer ( ) , unit_size : nil | %{ __struct__ : Range , first : 1 , last : 256 , step : 1 } }","ref":"TypeCheck.Builtin.SizedBitstring.html#t:t/0","title":"TypeCheck.Builtin.SizedBitstring.t/0","type":"type"},{"doc":"Checks whether the value is any tuple. Returns a problem tuple with the reason :no_match otherwise.","ref":"TypeCheck.Builtin.Tuple.html","title":"TypeCheck.Builtin.Tuple","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: problem_tuple :: { t ( ) , :no_match , %{ } , any ( ) }","ref":"TypeCheck.Builtin.Tuple.html#t:problem_tuple/0","title":"TypeCheck.Builtin.Tuple.problem_tuple/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t :: % TypeCheck.Builtin.Tuple { }","ref":"TypeCheck.Builtin.Tuple.html#t:t/0","title":"TypeCheck.Builtin.Tuple.t/0","type":"type"},{"doc":"Contains a many common types that can be used as overrides for Elixir's standard library's 'Remote Types'. This module complements TypeCheck.Builtin , contains all 'built-in' types of Elixir. Implementing TypeSpecs for all types of Elixir's standard library is a work-in-progress. Some TypeCheck-versions of the types are a little more general than the original version, to make up for functionality in TypeCheck which does not exist yet. Simply put, this means that you will never get a 'false positive' (a correct value not being accepted by a function), but in very rare cases you might get a 'false negative' (an improper value passing the type-check.)","ref":"TypeCheck.DefaultOverrides.html","title":"TypeCheck.DefaultOverrides","type":"module"},{"doc":"Lists all overridden types in {module, function, arity} format.","ref":"TypeCheck.DefaultOverrides.html#default_overrides/0","title":"TypeCheck.DefaultOverrides.default_overrides/0","type":"function"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Access.html","title":"TypeCheck.DefaultOverrides.Access","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: access_fun ( data , current_value ) :: get_fun ( data ) | get_and_update_fun ( data , current_value )","ref":"TypeCheck.DefaultOverrides.Access.html#t:access_fun/2","title":"TypeCheck.DefaultOverrides.Access.access_fun/2","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: any_container ( ) :: any ( )","ref":"TypeCheck.DefaultOverrides.Access.html#t:any_container/0","title":"TypeCheck.DefaultOverrides.Access.any_container/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: container ( ) :: keyword ( ) | struct ( ) | map ( )","ref":"TypeCheck.DefaultOverrides.Access.html#t:container/0","title":"TypeCheck.DefaultOverrides.Access.container/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: get_and_update_fun ( data , current_value ) :: ( :get_and_update , data , ( term ( ) -&gt; term ( ) ) -&gt; { current_value , new_data :: container ( ) } | :pop )","ref":"TypeCheck.DefaultOverrides.Access.html#t:get_and_update_fun/2","title":"TypeCheck.DefaultOverrides.Access.get_and_update_fun/2","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: get_fun ( data ) :: ( :get , data , ( term ( ) -&gt; term ( ) ) -&gt; new_data :: container ( ) )","ref":"TypeCheck.DefaultOverrides.Access.html#t:get_fun/1","title":"TypeCheck.DefaultOverrides.Access.get_fun/1","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: key ( ) :: any ( )","ref":"TypeCheck.DefaultOverrides.Access.html#t:key/0","title":"TypeCheck.DefaultOverrides.Access.key/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: nil_container ( ) :: nil","ref":"TypeCheck.DefaultOverrides.Access.html#t:nil_container/0","title":"TypeCheck.DefaultOverrides.Access.nil_container/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t ( ) :: container ( ) | nil_container ( ) | any_container ( )","ref":"TypeCheck.DefaultOverrides.Access.html#t:t/0","title":"TypeCheck.DefaultOverrides.Access.t/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: value ( ) :: any ( )","ref":"TypeCheck.DefaultOverrides.Access.html#t:value/0","title":"TypeCheck.DefaultOverrides.Access.value/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Agent.html","title":"TypeCheck.DefaultOverrides.Agent","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: agent ( ) :: pid ( ) | { atom ( ) , node ( ) } | name ( )","ref":"TypeCheck.DefaultOverrides.Agent.html#t:agent/0","title":"TypeCheck.DefaultOverrides.Agent.agent/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: name ( ) :: atom ( ) | { :global , term ( ) } | { :via , module ( ) , term ( ) }","ref":"TypeCheck.DefaultOverrides.Agent.html#t:name/0","title":"TypeCheck.DefaultOverrides.Agent.name/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: on_start ( ) :: { :ok , pid ( ) } | { :error , { :already_started , pid ( ) } | term ( ) }","ref":"TypeCheck.DefaultOverrides.Agent.html#t:on_start/0","title":"TypeCheck.DefaultOverrides.Agent.on_start/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: state ( ) :: term ( )","ref":"TypeCheck.DefaultOverrides.Agent.html#t:state/0","title":"TypeCheck.DefaultOverrides.Agent.state/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Application.html","title":"TypeCheck.DefaultOverrides.Application","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: app ( ) :: atom ( )","ref":"TypeCheck.DefaultOverrides.Application.html#t:app/0","title":"TypeCheck.DefaultOverrides.Application.app/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: application_key ( ) :: :start_phases | :mod | :applications | :optional_applications | :included_applications | :registered | :maxT | :maxP | :modules | :vsn | :id | :description","ref":"TypeCheck.DefaultOverrides.Application.html#t:application_key/0","title":"TypeCheck.DefaultOverrides.Application.application_key/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: key ( ) :: atom ( )","ref":"TypeCheck.DefaultOverrides.Application.html#t:key/0","title":"TypeCheck.DefaultOverrides.Application.key/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: restart_type ( ) :: :permanent | :transient | :temporary","ref":"TypeCheck.DefaultOverrides.Application.html#t:restart_type/0","title":"TypeCheck.DefaultOverrides.Application.restart_type/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: start_type ( ) :: :normal | { :takeover , node ( ) } | { :failover , node ( ) }","ref":"TypeCheck.DefaultOverrides.Application.html#t:start_type/0","title":"TypeCheck.DefaultOverrides.Application.start_type/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: state ( ) :: term ( )","ref":"TypeCheck.DefaultOverrides.Application.html#t:state/0","title":"TypeCheck.DefaultOverrides.Application.state/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: value ( ) :: term ( )","ref":"TypeCheck.DefaultOverrides.Application.html#t:value/0","title":"TypeCheck.DefaultOverrides.Application.value/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Base.html","title":"TypeCheck.DefaultOverrides.Base","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: decode_case ( ) :: :upper | :lower | :mixed","ref":"TypeCheck.DefaultOverrides.Base.html#t:decode_case/0","title":"TypeCheck.DefaultOverrides.Base.decode_case/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: encode_case ( ) :: :upper | :lower","ref":"TypeCheck.DefaultOverrides.Base.html#t:encode_case/0","title":"TypeCheck.DefaultOverrides.Base.encode_case/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Calendar.html","title":"TypeCheck.DefaultOverrides.Calendar","type":"module"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Calendar.html#calendar_gen/0","title":"TypeCheck.DefaultOverrides.Calendar.calendar_gen/0","type":"function"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: calendar ( ) :: wrap_with_gen ( module ( ) , &amp; TypeCheck.DefaultOverrides.Calendar . calendar_gen / 0 )","ref":"TypeCheck.DefaultOverrides.Calendar.html#t:calendar/0","title":"TypeCheck.DefaultOverrides.Calendar.calendar/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: date ( ) :: %{ optional ( any ( ) ) =&gt; any ( ) , calendar : calendar ( ) , year : year ( ) , month : month ( ) , day : day ( ) }","ref":"TypeCheck.DefaultOverrides.Calendar.html#t:date/0","title":"TypeCheck.DefaultOverrides.Calendar.date/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: datetime ( ) :: %{ optional ( any ( ) ) =&gt; any ( ) , calendar : calendar ( ) , year : year ( ) , month : month ( ) , day : day ( ) , hour : hour ( ) , minute : minute ( ) , second : second ( ) , microsecond : microsecond ( ) , time_zone : time_zone ( ) , zone_abbr : zone_abbr ( ) , utc_offset : utc_offset ( ) , std_offset : std_offset ( ) }","ref":"TypeCheck.DefaultOverrides.Calendar.html#t:datetime/0","title":"TypeCheck.DefaultOverrides.Calendar.datetime/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: day ( ) :: pos_integer ( )","ref":"TypeCheck.DefaultOverrides.Calendar.html#t:day/0","title":"TypeCheck.DefaultOverrides.Calendar.day/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: day_fraction ( ) :: { parts_in_day :: non_neg_integer ( ) , parts_per_day :: pos_integer ( ) }","ref":"TypeCheck.DefaultOverrides.Calendar.html#t:day_fraction/0","title":"TypeCheck.DefaultOverrides.Calendar.day_fraction/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: day_of_era ( ) :: { day :: non_neg_integer ( ) , era ( ) }","ref":"TypeCheck.DefaultOverrides.Calendar.html#t:day_of_era/0","title":"TypeCheck.DefaultOverrides.Calendar.day_of_era/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: day_of_week ( ) :: non_neg_integer ( )","ref":"TypeCheck.DefaultOverrides.Calendar.html#t:day_of_week/0","title":"TypeCheck.DefaultOverrides.Calendar.day_of_week/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: era ( ) :: non_neg_integer ( )","ref":"TypeCheck.DefaultOverrides.Calendar.html#t:era/0","title":"TypeCheck.DefaultOverrides.Calendar.era/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: hour ( ) :: non_neg_integer ( )","ref":"TypeCheck.DefaultOverrides.Calendar.html#t:hour/0","title":"TypeCheck.DefaultOverrides.Calendar.hour/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: iso_days ( ) :: { days :: integer ( ) , day_fraction ( ) }","ref":"TypeCheck.DefaultOverrides.Calendar.html#t:iso_days/0","title":"TypeCheck.DefaultOverrides.Calendar.iso_days/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Calendar.html#t:microsecond/0","title":"TypeCheck.DefaultOverrides.Calendar.microsecond/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: minute ( ) :: non_neg_integer ( )","ref":"TypeCheck.DefaultOverrides.Calendar.html#t:minute/0","title":"TypeCheck.DefaultOverrides.Calendar.minute/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: month ( ) :: pos_integer ( )","ref":"TypeCheck.DefaultOverrides.Calendar.html#t:month/0","title":"TypeCheck.DefaultOverrides.Calendar.month/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: naive_datetime ( ) :: %{ optional ( any ( ) ) =&gt; any ( ) , calendar : calendar ( ) , year : year ( ) , month : month ( ) , day : day ( ) , hour : hour ( ) , minute : minute ( ) , second : second ( ) , microsecond : microsecond ( ) }","ref":"TypeCheck.DefaultOverrides.Calendar.html#t:naive_datetime/0","title":"TypeCheck.DefaultOverrides.Calendar.naive_datetime/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: second ( ) :: non_neg_integer ( )","ref":"TypeCheck.DefaultOverrides.Calendar.html#t:second/0","title":"TypeCheck.DefaultOverrides.Calendar.second/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: std_offset ( ) :: integer ( )","ref":"TypeCheck.DefaultOverrides.Calendar.html#t:std_offset/0","title":"TypeCheck.DefaultOverrides.Calendar.std_offset/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: time ( ) :: %{ optional ( any ( ) ) =&gt; any ( ) , hour : hour ( ) , minute : minute ( ) , second : second ( ) , microsecond : microsecond ( ) }","ref":"TypeCheck.DefaultOverrides.Calendar.html#t:time/0","title":"TypeCheck.DefaultOverrides.Calendar.time/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: time_zone ( ) :: TypeCheck.DefaultOverrides.String . t ( )","ref":"TypeCheck.DefaultOverrides.Calendar.html#t:time_zone/0","title":"TypeCheck.DefaultOverrides.Calendar.time_zone/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: time_zone_database ( ) :: module ( )","ref":"TypeCheck.DefaultOverrides.Calendar.html#t:time_zone_database/0","title":"TypeCheck.DefaultOverrides.Calendar.time_zone_database/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: utc_offset ( ) :: integer ( )","ref":"TypeCheck.DefaultOverrides.Calendar.html#t:utc_offset/0","title":"TypeCheck.DefaultOverrides.Calendar.utc_offset/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: week ( ) :: pos_integer ( )","ref":"TypeCheck.DefaultOverrides.Calendar.html#t:week/0","title":"TypeCheck.DefaultOverrides.Calendar.week/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: year ( ) :: integer ( )","ref":"TypeCheck.DefaultOverrides.Calendar.html#t:year/0","title":"TypeCheck.DefaultOverrides.Calendar.year/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: zone_abbr ( ) :: TypeCheck.DefaultOverrides.String . t ( )","ref":"TypeCheck.DefaultOverrides.Calendar.html#t:zone_abbr/0","title":"TypeCheck.DefaultOverrides.Calendar.zone_abbr/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Calendar.ISO.html","title":"TypeCheck.DefaultOverrides.Calendar.ISO","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: bce ( ) :: 0","ref":"TypeCheck.DefaultOverrides.Calendar.ISO.html#t:bce/0","title":"TypeCheck.DefaultOverrides.Calendar.ISO.bce/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: ce ( ) :: 1","ref":"TypeCheck.DefaultOverrides.Calendar.ISO.html#t:ce/0","title":"TypeCheck.DefaultOverrides.Calendar.ISO.ce/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: day ( ) :: %{ __struct__ : Range , first : 1 , last : 31 , step : 1 }","ref":"TypeCheck.DefaultOverrides.Calendar.ISO.html#t:day/0","title":"TypeCheck.DefaultOverrides.Calendar.ISO.day/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: day_of_week ( ) :: %{ __struct__ : Range , first : 1 , last : 7 , step : 1 }","ref":"TypeCheck.DefaultOverrides.Calendar.ISO.html#t:day_of_week/0","title":"TypeCheck.DefaultOverrides.Calendar.ISO.day_of_week/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: day_of_year ( ) :: %{ __struct__ : Range , first : 1 , last : 366 , step : 1 }","ref":"TypeCheck.DefaultOverrides.Calendar.ISO.html#t:day_of_year/0","title":"TypeCheck.DefaultOverrides.Calendar.ISO.day_of_year/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: era ( ) :: bce ( ) | ce ( )","ref":"TypeCheck.DefaultOverrides.Calendar.ISO.html#t:era/0","title":"TypeCheck.DefaultOverrides.Calendar.ISO.era/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: hour ( ) :: %{ __struct__ : Range , first : 0 , last : 23 , step : 1 }","ref":"TypeCheck.DefaultOverrides.Calendar.ISO.html#t:hour/0","title":"TypeCheck.DefaultOverrides.Calendar.ISO.hour/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: microsecond ( ) :: { %{ __struct__ : Range , first : 0 , last : 999_999 , step : 1 } , %{ __struct__ : Range , first : 0 , last : 6 , step : 1 } }","ref":"TypeCheck.DefaultOverrides.Calendar.ISO.html#t:microsecond/0","title":"TypeCheck.DefaultOverrides.Calendar.ISO.microsecond/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: minute ( ) :: %{ __struct__ : Range , first : 0 , last : 59 , step : 1 }","ref":"TypeCheck.DefaultOverrides.Calendar.ISO.html#t:minute/0","title":"TypeCheck.DefaultOverrides.Calendar.ISO.minute/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: month ( ) :: %{ __struct__ : Range , first : 1 , last : 12 , step : 1 }","ref":"TypeCheck.DefaultOverrides.Calendar.ISO.html#t:month/0","title":"TypeCheck.DefaultOverrides.Calendar.ISO.month/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: quarter_of_year ( ) :: %{ __struct__ : Range , first : 1 , last : 4 , step : 1 }","ref":"TypeCheck.DefaultOverrides.Calendar.ISO.html#t:quarter_of_year/0","title":"TypeCheck.DefaultOverrides.Calendar.ISO.quarter_of_year/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: second ( ) :: %{ __struct__ : Range , first : 0 , last : 59 , step : 1 }","ref":"TypeCheck.DefaultOverrides.Calendar.ISO.html#t:second/0","title":"TypeCheck.DefaultOverrides.Calendar.ISO.second/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: weekday ( ) :: :monday | :tuesday | :wednesday | :thursday | :friday | :saturday | :sunday","ref":"TypeCheck.DefaultOverrides.Calendar.ISO.html#t:weekday/0","title":"TypeCheck.DefaultOverrides.Calendar.ISO.weekday/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: year ( ) :: %{ __struct__ : Range , first : - 9999 , last : 9999 , step : 1 }","ref":"TypeCheck.DefaultOverrides.Calendar.ISO.html#t:year/0","title":"TypeCheck.DefaultOverrides.Calendar.ISO.year/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: year_of_era ( ) :: { %{ __struct__ : Range , first : 1 , last : 10000 , step : 1 } , era ( ) }","ref":"TypeCheck.DefaultOverrides.Calendar.ISO.html#t:year_of_era/0","title":"TypeCheck.DefaultOverrides.Calendar.ISO.year_of_era/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Calendar.TimeZoneDatabase.html","title":"TypeCheck.DefaultOverrides.Calendar.TimeZoneDatabase","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: time_zone_period ( ) :: %{ optional ( any ( ) ) =&gt; any ( ) , utc_offset : TypeCheck.DefaultOverrides.Calendar . utc_offset ( ) , std_offset : TypeCheck.DefaultOverrides.Calendar . std_offset ( ) , zone_abbr : TypeCheck.DefaultOverrides.Calendar . zone_abbr ( ) }","ref":"TypeCheck.DefaultOverrides.Calendar.TimeZoneDatabase.html#t:time_zone_period/0","title":"TypeCheck.DefaultOverrides.Calendar.TimeZoneDatabase.time_zone_period/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: time_zone_period_limit ( ) :: TypeCheck.DefaultOverrides.Calendar . naive_datetime ( )","ref":"TypeCheck.DefaultOverrides.Calendar.TimeZoneDatabase.html#t:time_zone_period_limit/0","title":"TypeCheck.DefaultOverrides.Calendar.TimeZoneDatabase.time_zone_period_limit/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Code.html","title":"TypeCheck.DefaultOverrides.Code","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: binding ( ) :: [ { atom ( ) | tuple ( ) , any ( ) } ]","ref":"TypeCheck.DefaultOverrides.Code.html#t:binding/0","title":"TypeCheck.DefaultOverrides.Code.binding/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Code.Fragment.html","title":"TypeCheck.DefaultOverrides.Code.Fragment","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: binding ( ) :: [ { atom ( ) | tuple ( ) , any ( ) } ]","ref":"TypeCheck.DefaultOverrides.Code.Fragment.html#t:binding/0","title":"TypeCheck.DefaultOverrides.Code.Fragment.binding/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Collectable.html","title":"TypeCheck.DefaultOverrides.Collectable","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: command ( ) :: { :cont , term ( ) } | :done | :halt","ref":"TypeCheck.DefaultOverrides.Collectable.html#t:command/0","title":"TypeCheck.DefaultOverrides.Collectable.command/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t ( ) :: impl ( Collectable )","ref":"TypeCheck.DefaultOverrides.Collectable.html#t:t/0","title":"TypeCheck.DefaultOverrides.Collectable.t/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Config.Provider.html","title":"TypeCheck.DefaultOverrides.Config.Provider","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: config ( ) :: keyword ( )","ref":"TypeCheck.DefaultOverrides.Config.Provider.html#t:config/0","title":"TypeCheck.DefaultOverrides.Config.Provider.config/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: config_path ( ) :: { :system , binary ( ) , binary ( ) } | binary ( )","ref":"TypeCheck.DefaultOverrides.Config.Provider.html#t:config_path/0","title":"TypeCheck.DefaultOverrides.Config.Provider.config_path/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: state ( ) :: term ( )","ref":"TypeCheck.DefaultOverrides.Config.Provider.html#t:state/0","title":"TypeCheck.DefaultOverrides.Config.Provider.state/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Date.html","title":"TypeCheck.DefaultOverrides.Date","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t ( ) :: % Date { calendar : TypeCheck.DefaultOverrides.Calendar . calendar ( ) , day : TypeCheck.DefaultOverrides.Calendar . day ( ) , month : TypeCheck.DefaultOverrides.Calendar . month ( ) , year : TypeCheck.DefaultOverrides.Calendar . year ( ) }","ref":"TypeCheck.DefaultOverrides.Date.html#t:t/0","title":"TypeCheck.DefaultOverrides.Date.t/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Date.Range.html","title":"TypeCheck.DefaultOverrides.Date.Range","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: iso_days() (opaque type)","ref":"TypeCheck.DefaultOverrides.Date.Range.html#t:iso_days/0","title":"TypeCheck.DefaultOverrides.Date.Range.iso_days/0","type":"opaque"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t ( ) :: % Date.Range { first : TypeCheck.DefaultOverrides.Date . t ( ) , first_in_iso_days : iso_days ( ) , last : TypeCheck.DefaultOverrides.Date . t ( ) , last_in_iso_days : iso_days ( ) , step : pos_integer ( ) | neg_integer ( ) }","ref":"TypeCheck.DefaultOverrides.Date.Range.html#t:t/0","title":"TypeCheck.DefaultOverrides.Date.Range.t/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.DateTime.html","title":"TypeCheck.DefaultOverrides.DateTime","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t ( ) :: % DateTime { calendar : TypeCheck.DefaultOverrides.Calendar . calendar ( ) , day : TypeCheck.DefaultOverrides.Calendar . day ( ) , hour : TypeCheck.DefaultOverrides.Calendar . hour ( ) , microsecond : TypeCheck.DefaultOverrides.Calendar . microsecond ( ) , minute : TypeCheck.DefaultOverrides.Calendar . minute ( ) , month : TypeCheck.DefaultOverrides.Calendar . month ( ) , second : TypeCheck.DefaultOverrides.Calendar . second ( ) , std_offset : TypeCheck.DefaultOverrides.Calendar . std_offset ( ) , time_zone : TypeCheck.DefaultOverrides.Calendar . time_zone ( ) , utc_offset : TypeCheck.DefaultOverrides.Calendar . utc_offset ( ) , year : TypeCheck.DefaultOverrides.Calendar . year ( ) , zone_abbr : TypeCheck.DefaultOverrides.Calendar . zone_abbr ( ) }","ref":"TypeCheck.DefaultOverrides.DateTime.html#t:t/0","title":"TypeCheck.DefaultOverrides.DateTime.t/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.DynamicSupervisor.html","title":"TypeCheck.DefaultOverrides.DynamicSupervisor","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: init_option ( ) :: { :strategy , strategy ( ) } | { :max_restarts , non_neg_integer ( ) } | { :max_seconds , pos_integer ( ) } | { :max_children , non_neg_integer ( ) | :infinity } | { :extra_arguments , [ term ( ) ] }","ref":"TypeCheck.DefaultOverrides.DynamicSupervisor.html#t:init_option/0","title":"TypeCheck.DefaultOverrides.DynamicSupervisor.init_option/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: on_start_child ( ) :: { :ok , pid ( ) } | { :ok , pid ( ) , info :: term ( ) } | :ignore | { :error , { :already_started , pid ( ) } | :max_children | term ( ) }","ref":"TypeCheck.DefaultOverrides.DynamicSupervisor.html#t:on_start_child/0","title":"TypeCheck.DefaultOverrides.DynamicSupervisor.on_start_child/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: option ( ) :: TypeCheck.DefaultOverrides.GenServer . option ( )","ref":"TypeCheck.DefaultOverrides.DynamicSupervisor.html#t:option/0","title":"TypeCheck.DefaultOverrides.DynamicSupervisor.option/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: strategy ( ) :: :one_for_one","ref":"TypeCheck.DefaultOverrides.DynamicSupervisor.html#t:strategy/0","title":"TypeCheck.DefaultOverrides.DynamicSupervisor.strategy/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: sup_flags ( ) :: %{ strategy : strategy ( ) , intensity : non_neg_integer ( ) , period : pos_integer ( ) , max_children : non_neg_integer ( ) | :infinity , extra_arguments : [ term ( ) ] }","ref":"TypeCheck.DefaultOverrides.DynamicSupervisor.html#t:sup_flags/0","title":"TypeCheck.DefaultOverrides.DynamicSupervisor.sup_flags/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Enum.html","title":"TypeCheck.DefaultOverrides.Enum","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: acc ( ) :: any ( )","ref":"TypeCheck.DefaultOverrides.Enum.html#t:acc/0","title":"TypeCheck.DefaultOverrides.Enum.acc/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: default ( ) :: any ( )","ref":"TypeCheck.DefaultOverrides.Enum.html#t:default/0","title":"TypeCheck.DefaultOverrides.Enum.default/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: element ( ) :: any ( )","ref":"TypeCheck.DefaultOverrides.Enum.html#t:element/0","title":"TypeCheck.DefaultOverrides.Enum.element/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: index ( ) :: integer ( )","ref":"TypeCheck.DefaultOverrides.Enum.html#t:index/0","title":"TypeCheck.DefaultOverrides.Enum.index/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t ( ) :: TypeCheck.DefaultOverrides.Enumerable . t ( )","ref":"TypeCheck.DefaultOverrides.Enum.html#t:t/0","title":"TypeCheck.DefaultOverrides.Enum.t/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Enumerable.html","title":"TypeCheck.DefaultOverrides.Enumerable","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: acc ( ) :: { :cont , term ( ) } | { :halt , term ( ) } | { :suspend , term ( ) }","ref":"TypeCheck.DefaultOverrides.Enumerable.html#t:acc/0","title":"TypeCheck.DefaultOverrides.Enumerable.acc/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: continuation ( ) :: ( acc ( ) -&gt; result ( ) )","ref":"TypeCheck.DefaultOverrides.Enumerable.html#t:continuation/0","title":"TypeCheck.DefaultOverrides.Enumerable.continuation/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: reducer ( ) :: ( element :: term ( ) , current_acc :: acc ( ) -&gt; updated_acc :: acc ( ) )","ref":"TypeCheck.DefaultOverrides.Enumerable.html#t:reducer/0","title":"TypeCheck.DefaultOverrides.Enumerable.reducer/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: result ( ) :: { :done , term ( ) } | { :halted , term ( ) } | { :suspended , term ( ) , continuation ( ) }","ref":"TypeCheck.DefaultOverrides.Enumerable.html#t:result/0","title":"TypeCheck.DefaultOverrides.Enumerable.result/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: slicing_fun ( ) :: ( start :: non_neg_integer ( ) , length :: pos_integer ( ) -&gt; [ term ( ) ] )","ref":"TypeCheck.DefaultOverrides.Enumerable.html#t:slicing_fun/0","title":"TypeCheck.DefaultOverrides.Enumerable.slicing_fun/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t ( ) :: impl ( Enumerable )","ref":"TypeCheck.DefaultOverrides.Enumerable.html#t:t/0","title":"TypeCheck.DefaultOverrides.Enumerable.t/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Erlang.Binary.html","title":"TypeCheck.DefaultOverrides.Erlang.Binary","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: cp() (opaque type)","ref":"TypeCheck.DefaultOverrides.Erlang.Binary.html#t:cp/0","title":"TypeCheck.DefaultOverrides.Erlang.Binary.cp/0","type":"opaque"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: part() (opaque type)","ref":"TypeCheck.DefaultOverrides.Erlang.Binary.html#t:part/0","title":"TypeCheck.DefaultOverrides.Erlang.Binary.part/0","type":"opaque"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Erlang.Calendar.html","title":"TypeCheck.DefaultOverrides.Erlang.Calendar","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: date ( ) :: { year ( ) , month ( ) , day ( ) }","ref":"TypeCheck.DefaultOverrides.Erlang.Calendar.html#t:date/0","title":"TypeCheck.DefaultOverrides.Erlang.Calendar.date/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: datetime1970 ( ) :: { { year1970 ( ) , month ( ) , day ( ) } , time ( ) }","ref":"TypeCheck.DefaultOverrides.Erlang.Calendar.html#t:datetime1970/0","title":"TypeCheck.DefaultOverrides.Erlang.Calendar.datetime1970/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: datetime ( ) :: { date ( ) , time ( ) }","ref":"TypeCheck.DefaultOverrides.Erlang.Calendar.html#t:datetime/0","title":"TypeCheck.DefaultOverrides.Erlang.Calendar.datetime/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: time ( ) :: { hour ( ) , minute ( ) , second ( ) }","ref":"TypeCheck.DefaultOverrides.Erlang.Calendar.html#t:time/0","title":"TypeCheck.DefaultOverrides.Erlang.Calendar.time/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Erlang.Inet.html","title":"TypeCheck.DefaultOverrides.Erlang.Inet","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: port_number ( ) :: %{ __struct__ : Range , first : 0 , last : 65535 , step : 1 }","ref":"TypeCheck.DefaultOverrides.Erlang.Inet.html#t:port_number/0","title":"TypeCheck.DefaultOverrides.Erlang.Inet.port_number/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Exception.html","title":"TypeCheck.DefaultOverrides.Exception","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: arity_or_args ( ) :: non_neg_integer ( ) | list ( )","ref":"TypeCheck.DefaultOverrides.Exception.html#t:arity_or_args/0","title":"TypeCheck.DefaultOverrides.Exception.arity_or_args/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: kind ( ) :: :error | non_error_kind ( )","ref":"TypeCheck.DefaultOverrides.Exception.html#t:kind/0","title":"TypeCheck.DefaultOverrides.Exception.kind/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: location ( ) :: keyword ( )","ref":"TypeCheck.DefaultOverrides.Exception.html#t:location/0","title":"TypeCheck.DefaultOverrides.Exception.location/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: non_error_kind ( ) :: :exit | :throw | { :EXIT , pid ( ) }","ref":"TypeCheck.DefaultOverrides.Exception.html#t:non_error_kind/0","title":"TypeCheck.DefaultOverrides.Exception.non_error_kind/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: stacktrace ( ) :: [ stacktrace_entry ( ) ]","ref":"TypeCheck.DefaultOverrides.Exception.html#t:stacktrace/0","title":"TypeCheck.DefaultOverrides.Exception.stacktrace/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: stacktrace_entry ( ) :: { module ( ) , atom ( ) , arity_or_args ( ) , location ( ) } | { ( ... -&gt; any ( ) ) , arity_or_args ( ) , location ( ) }","ref":"TypeCheck.DefaultOverrides.Exception.html#t:stacktrace_entry/0","title":"TypeCheck.DefaultOverrides.Exception.stacktrace_entry/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t ( ) :: %{ :__struct__ =&gt; module ( ) , :__exception__ =&gt; true , optional ( atom ( ) ) =&gt; any ( ) }","ref":"TypeCheck.DefaultOverrides.Exception.html#t:t/0","title":"TypeCheck.DefaultOverrides.Exception.t/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.File.html","title":"TypeCheck.DefaultOverrides.File","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: encoding_mode ( ) :: :utf8 | { :encoding , :latin1 | :unicode | :utf8 | :utf16 | :utf32 | { :utf16 , :big | :little } | { :utf32 , :big | :little } }","ref":"TypeCheck.DefaultOverrides.File.html#t:encoding_mode/0","title":"TypeCheck.DefaultOverrides.File.encoding_mode/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: erlang_time ( ) :: { { year :: non_neg_integer ( ) , month :: %{ __struct__ : Range , first : 1 , last : 12 , step : 1 } , day :: %{ __struct__ : Range , first : 1 , last : 31 , step : 1 } } , { hour :: %{ __struct__ : Range , first : 0 , last : 23 , step : 1 } , minute :: %{ __struct__ : Range , first : 0 , last : 59 , step : 1 } , second :: %{ __struct__ : Range , first : 0 , last : 59 , step : 1 } } }","ref":"TypeCheck.DefaultOverrides.File.html#t:erlang_time/0","title":"TypeCheck.DefaultOverrides.File.erlang_time/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: fd :: { :file_descriptor , module :: module ( ) , data :: term ( ) }","ref":"TypeCheck.DefaultOverrides.File.html#t:fd/0","title":"TypeCheck.DefaultOverrides.File.fd/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: io_device ( ) :: pid ( ) | fd ( )","ref":"TypeCheck.DefaultOverrides.File.html#t:io_device/0","title":"TypeCheck.DefaultOverrides.File.io_device/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: mode ( ) :: :append | :binary | :charlist | :compressed | :delayed_write | :exclusive | :raw | :read | :read_ahead | :sync | :write | { :read_ahead , pos_integer ( ) } | { :delayed_write , non_neg_integer ( ) , non_neg_integer ( ) } | encoding_mode ( )","ref":"TypeCheck.DefaultOverrides.File.html#t:mode/0","title":"TypeCheck.DefaultOverrides.File.mode/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: posix :: :eacces | :eagain | :ebadf | :ebadmsg | :ebusy | :edeadlk | :edeadlock | :edquot | :eexist | :efault | :efbig | :eftype | :eintr | :einval | :eio | :eisdir | :eloop | :emfile | :emlink | :emultihop | :enametoolong | :enfile | :enobufs | :enodev | :enolck | :enolink | :enoent | :enomem | :enospc | :enosr | :enostr | :enosys | :enotblk | :enotdir | :enotsup | :enxio | :eopnotsupp | :eoverflow | :eperm | :epipe | :erange | :erofs | :espipe | :esrch | :estale | :etxtbsy | :exdev","ref":"TypeCheck.DefaultOverrides.File.html#t:posix/0","title":"TypeCheck.DefaultOverrides.File.posix/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: posix_time ( ) :: integer ( )","ref":"TypeCheck.DefaultOverrides.File.html#t:posix_time/0","title":"TypeCheck.DefaultOverrides.File.posix_time/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: stat_options ( ) :: [ time : :local | :universal | :posix ]","ref":"TypeCheck.DefaultOverrides.File.html#t:stat_options/0","title":"TypeCheck.DefaultOverrides.File.stat_options/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: stream_mode ( ) :: encoding_mode ( ) | :append | :compressed | :trim_bom | { :read_ahead , pos_integer ( ) | false } | { :delayed_write , non_neg_integer ( ) , non_neg_integer ( ) }","ref":"TypeCheck.DefaultOverrides.File.html#t:stream_mode/0","title":"TypeCheck.DefaultOverrides.File.stream_mode/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.File.Stat.html","title":"TypeCheck.DefaultOverrides.File.Stat","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t ( ) :: % File.Stat { access : :read | :write | :read_write | :none , atime : TypeCheck.DefaultOverrides.Erlang.Calendar . datetime ( ) | integer ( ) , ctime : TypeCheck.DefaultOverrides.Erlang.Calendar . datetime ( ) | integer ( ) , gid : non_neg_integer ( ) , inode : non_neg_integer ( ) , links : non_neg_integer ( ) , major_device : non_neg_integer ( ) , minor_device : non_neg_integer ( ) , mode : non_neg_integer ( ) , mtime : TypeCheck.DefaultOverrides.Erlang.Calendar . datetime ( ) | integer ( ) , size : non_neg_integer ( ) , type : :device | :directory | :regular | :other | :symlink , uid : non_neg_integer ( ) }","ref":"TypeCheck.DefaultOverrides.File.Stat.html#t:t/0","title":"TypeCheck.DefaultOverrides.File.Stat.t/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.File.Stream.html","title":"TypeCheck.DefaultOverrides.File.Stream","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t ( ) :: % File.Stream { line_or_bytes : term ( ) , modes : term ( ) , path : term ( ) , raw : term ( ) }","ref":"TypeCheck.DefaultOverrides.File.Stream.html#t:t/0","title":"TypeCheck.DefaultOverrides.File.Stream.t/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Float.html","title":"TypeCheck.DefaultOverrides.Float","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: precision_range ( ) :: %{ __struct__ : Range , first : 0 , last : 15 , step : 1 }","ref":"TypeCheck.DefaultOverrides.Float.html#t:precision_range/0","title":"TypeCheck.DefaultOverrides.Float.precision_range/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Function.html","title":"TypeCheck.DefaultOverrides.Function","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: information ( ) :: :arity | :env | :index | :module | :name | :new_index | :new_uniq | :pid | :type | :uniq","ref":"TypeCheck.DefaultOverrides.Function.html#t:information/0","title":"TypeCheck.DefaultOverrides.Function.information/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.GenServer.html","title":"TypeCheck.DefaultOverrides.GenServer","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: debug ( ) :: [ :trace | :log | :statistics | { :log_to_file , TypeCheck.DefaultOverrides.Path . t ( ) } ]","ref":"TypeCheck.DefaultOverrides.GenServer.html#t:debug/0","title":"TypeCheck.DefaultOverrides.GenServer.debug/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: from ( ) :: { pid ( ) , tag :: term ( ) }","ref":"TypeCheck.DefaultOverrides.GenServer.html#t:from/0","title":"TypeCheck.DefaultOverrides.GenServer.from/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: name ( ) :: atom ( ) | { :global , term ( ) } | { :via , module ( ) , term ( ) }","ref":"TypeCheck.DefaultOverrides.GenServer.html#t:name/0","title":"TypeCheck.DefaultOverrides.GenServer.name/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: on_start ( ) :: { :ok , pid ( ) } | :ignore | { :error , { :already_started , pid ( ) } | term ( ) }","ref":"TypeCheck.DefaultOverrides.GenServer.html#t:on_start/0","title":"TypeCheck.DefaultOverrides.GenServer.on_start/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: option ( ) :: { :debug , debug ( ) } | { :name , name ( ) } | { :timeout , timeout ( ) } | { :spawn_opt , [ TypeCheck.DefaultOverrides.Process . spawn_opt ( ) ] } | { :hibernate_after , timeout ( ) }","ref":"TypeCheck.DefaultOverrides.GenServer.html#t:option/0","title":"TypeCheck.DefaultOverrides.GenServer.option/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: options ( ) :: [ option ( ) ]","ref":"TypeCheck.DefaultOverrides.GenServer.html#t:options/0","title":"TypeCheck.DefaultOverrides.GenServer.options/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: server ( ) :: pid ( ) | name ( ) | { atom ( ) , node ( ) }","ref":"TypeCheck.DefaultOverrides.GenServer.html#t:server/0","title":"TypeCheck.DefaultOverrides.GenServer.server/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.IO.html","title":"TypeCheck.DefaultOverrides.IO","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: chardata ( ) :: TypeCheck.DefaultOverrides.String . t ( ) | maybe_improper_list ( char ( ) | chardata ( ) , TypeCheck.DefaultOverrides.String . t ( ) | [ ] )","ref":"TypeCheck.DefaultOverrides.IO.html#t:chardata/0","title":"TypeCheck.DefaultOverrides.IO.chardata/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: device ( ) :: atom ( ) | pid ( )","ref":"TypeCheck.DefaultOverrides.IO.html#t:device/0","title":"TypeCheck.DefaultOverrides.IO.device/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: nodata ( ) :: { :error , term ( ) } | :eof","ref":"TypeCheck.DefaultOverrides.IO.html#t:nodata/0","title":"TypeCheck.DefaultOverrides.IO.nodata/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.IO.ANSI.html","title":"TypeCheck.DefaultOverrides.IO.ANSI","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: ansicode ( ) :: atom ( )","ref":"TypeCheck.DefaultOverrides.IO.ANSI.html#t:ansicode/0","title":"TypeCheck.DefaultOverrides.IO.ANSI.ansicode/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: ansidata ( ) :: ansilist ( ) | ansicode ( ) | binary ( )","ref":"TypeCheck.DefaultOverrides.IO.ANSI.html#t:ansidata/0","title":"TypeCheck.DefaultOverrides.IO.ANSI.ansidata/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: ansilist ( ) :: maybe_improper_list ( char ( ) | ansicode ( ) | binary ( ) | ansilist ( ) , binary ( ) | ansicode ( ) | [ ] )","ref":"TypeCheck.DefaultOverrides.IO.ANSI.html#t:ansilist/0","title":"TypeCheck.DefaultOverrides.IO.ANSI.ansilist/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.IO.Stream.html","title":"TypeCheck.DefaultOverrides.IO.Stream","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t ( ) :: % IO.Stream { device : term ( ) , line_or_bytes : term ( ) , raw : term ( ) }","ref":"TypeCheck.DefaultOverrides.IO.Stream.html#t:t/0","title":"TypeCheck.DefaultOverrides.IO.Stream.t/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Inspect.html","title":"TypeCheck.DefaultOverrides.Inspect","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t ( ) :: impl ( Inspect )","ref":"TypeCheck.DefaultOverrides.Inspect.html#t:t/0","title":"TypeCheck.DefaultOverrides.Inspect.t/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Inspect.Algebra.html","title":"TypeCheck.DefaultOverrides.Inspect.Algebra","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t ( ) :: binary ( ) | :doc_line | :doc_nil | doc_break ( ) | doc_collapse ( ) | doc_color ( ) | doc_cons ( ) | doc_fits ( ) | doc_force ( ) | doc_group ( ) | doc_nest ( ) | doc_string ( )","ref":"TypeCheck.DefaultOverrides.Inspect.Algebra.html#t:t/0","title":"TypeCheck.DefaultOverrides.Inspect.Algebra.t/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Inspect.Opts.html","title":"TypeCheck.DefaultOverrides.Inspect.Opts","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: color_key ( ) :: atom ( )","ref":"TypeCheck.DefaultOverrides.Inspect.Opts.html#t:color_key/0","title":"TypeCheck.DefaultOverrides.Inspect.Opts.color_key/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t ( ) :: % Inspect.Opts { base : :decimal | :binary | :hex | :octal , binaries : :infer | :as_binaries | :as_strings , char_lists : :infer | :as_lists | :as_char_lists , charlists : :infer | :as_lists | :as_charlists , custom_options : keyword ( ) , inspect_fun : ( any ( ) , t ( ) -&gt; TypeCheck.DefaultOverrides.Inspect.Algebra . t ( ) ) , limit : non_neg_integer ( ) | :infinity , pretty : boolean ( ) , printable_limit : non_neg_integer ( ) | :infinity , safe : boolean ( ) , structs : boolean ( ) , syntax_colors : [ { color_key ( ) , TypeCheck.DefaultOverrides.IO.ANSI . ansidata ( ) } ] , syntax_colors : [ { color_key ( ) , any ( ) } ] , width : non_neg_integer ( ) | :infinity }","ref":"TypeCheck.DefaultOverrides.Inspect.Opts.html#t:t/0","title":"TypeCheck.DefaultOverrides.Inspect.Opts.t/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Kernel.ParallelCompiler.html","title":"TypeCheck.DefaultOverrides.Kernel.ParallelCompiler","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: error ( ) :: { file :: TypeCheck.DefaultOverrides.Path . t ( ) , line ( ) , message :: TypeCheck.DefaultOverrides.String . t ( ) }","ref":"TypeCheck.DefaultOverrides.Kernel.ParallelCompiler.html#t:error/0","title":"TypeCheck.DefaultOverrides.Kernel.ParallelCompiler.error/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: line ( ) :: non_neg_integer ( )","ref":"TypeCheck.DefaultOverrides.Kernel.ParallelCompiler.html#t:line/0","title":"TypeCheck.DefaultOverrides.Kernel.ParallelCompiler.line/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: location ( ) :: line ( ) | { line ( ) , column :: non_neg_integer ( ) }","ref":"TypeCheck.DefaultOverrides.Kernel.ParallelCompiler.html#t:location/0","title":"TypeCheck.DefaultOverrides.Kernel.ParallelCompiler.location/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: warning ( ) :: { file :: TypeCheck.DefaultOverrides.Path . t ( ) , location ( ) , message :: TypeCheck.DefaultOverrides.String . t ( ) }","ref":"TypeCheck.DefaultOverrides.Kernel.ParallelCompiler.html#t:warning/0","title":"TypeCheck.DefaultOverrides.Kernel.ParallelCompiler.warning/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Keyword.html","title":"TypeCheck.DefaultOverrides.Keyword","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: key ( ) :: atom ( )","ref":"TypeCheck.DefaultOverrides.Keyword.html#t:key/0","title":"TypeCheck.DefaultOverrides.Keyword.key/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t ( ) :: [ { key ( ) , value ( ) } ]","ref":"TypeCheck.DefaultOverrides.Keyword.html#t:t/0","title":"TypeCheck.DefaultOverrides.Keyword.t/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t ( value ) :: [ { key ( ) , value } ]","ref":"TypeCheck.DefaultOverrides.Keyword.html#t:t/1","title":"TypeCheck.DefaultOverrides.Keyword.t/1","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: value ( ) :: any ( )","ref":"TypeCheck.DefaultOverrides.Keyword.html#t:value/0","title":"TypeCheck.DefaultOverrides.Keyword.value/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.List.Chars.html","title":"TypeCheck.DefaultOverrides.List.Chars","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t ( ) :: impl ( List.Chars )","ref":"TypeCheck.DefaultOverrides.List.Chars.html#t:t/0","title":"TypeCheck.DefaultOverrides.List.Chars.t/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Macro.html","title":"TypeCheck.DefaultOverrides.Macro","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: captured_remote_function ( ) :: ( ... -&gt; any ( ) )","ref":"TypeCheck.DefaultOverrides.Macro.html#t:captured_remote_function/0","title":"TypeCheck.DefaultOverrides.Macro.captured_remote_function/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: input ( ) :: input_expr ( ) | { lazy ( input ( ) ) , lazy ( input ( ) ) } | [ lazy ( input ( ) ) ] | atom ( ) | number ( ) | binary ( )","ref":"TypeCheck.DefaultOverrides.Macro.html#t:input/0","title":"TypeCheck.DefaultOverrides.Macro.input/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: metadata ( ) :: keyword ( )","ref":"TypeCheck.DefaultOverrides.Macro.html#t:metadata/0","title":"TypeCheck.DefaultOverrides.Macro.metadata/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: output ( ) :: output_expr ( ) | { lazy ( output ( ) ) , lazy ( output ( ) ) } | [ lazy ( output ( ) ) ] | atom ( ) | number ( ) | binary ( ) | captured_remote_function ( ) | pid ( )","ref":"TypeCheck.DefaultOverrides.Macro.html#t:output/0","title":"TypeCheck.DefaultOverrides.Macro.output/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t ( ) :: input ( )","ref":"TypeCheck.DefaultOverrides.Macro.html#t:t/0","title":"TypeCheck.DefaultOverrides.Macro.t/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Macro.Env.html","title":"TypeCheck.DefaultOverrides.Macro.Env","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: context ( ) :: :match | :guard | nil","ref":"TypeCheck.DefaultOverrides.Macro.Env.html#t:context/0","title":"TypeCheck.DefaultOverrides.Macro.Env.context/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: context_modules ( ) :: [ module ( ) ]","ref":"TypeCheck.DefaultOverrides.Macro.Env.html#t:context_modules/0","title":"TypeCheck.DefaultOverrides.Macro.Env.context_modules/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: file ( ) :: binary ( )","ref":"TypeCheck.DefaultOverrides.Macro.Env.html#t:file/0","title":"TypeCheck.DefaultOverrides.Macro.Env.file/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: line ( ) :: non_neg_integer ( )","ref":"TypeCheck.DefaultOverrides.Macro.Env.html#t:line/0","title":"TypeCheck.DefaultOverrides.Macro.Env.line/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: name_arity ( ) :: { atom ( ) , arity ( ) }","ref":"TypeCheck.DefaultOverrides.Macro.Env.html#t:name_arity/0","title":"TypeCheck.DefaultOverrides.Macro.Env.name_arity/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t ( ) :: % Macro.Env { aliases : aliases ( ) , context : context ( ) , context_modules : context_modules ( ) , file : file ( ) , function : name_arity ( ) | nil , functions : functions ( ) , lexical_tracker : lexical_tracker ( ) , line : line ( ) , macro_aliases : macro_aliases ( ) , macros : macros ( ) , module : module ( ) , requires : requires ( ) , tracers : tracers ( ) , versioned_vars : versioned_vars ( ) }","ref":"TypeCheck.DefaultOverrides.Macro.Env.html#t:t/0","title":"TypeCheck.DefaultOverrides.Macro.Env.t/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: variable ( ) :: { atom ( ) , atom ( ) | term ( ) }","ref":"TypeCheck.DefaultOverrides.Macro.Env.html#t:variable/0","title":"TypeCheck.DefaultOverrides.Macro.Env.variable/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Map.html","title":"TypeCheck.DefaultOverrides.Map","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: key ( ) :: any ( )","ref":"TypeCheck.DefaultOverrides.Map.html#t:key/0","title":"TypeCheck.DefaultOverrides.Map.key/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: value ( ) :: any ( )","ref":"TypeCheck.DefaultOverrides.Map.html#t:value/0","title":"TypeCheck.DefaultOverrides.Map.value/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.MapSet.html","title":"TypeCheck.DefaultOverrides.MapSet","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t ( ) :: t ( term ( ) )","ref":"TypeCheck.DefaultOverrides.MapSet.html#t:t/0","title":"TypeCheck.DefaultOverrides.MapSet.t/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t(value) (opaque type)","ref":"TypeCheck.DefaultOverrides.MapSet.html#t:t/1","title":"TypeCheck.DefaultOverrides.MapSet.t/1","type":"opaque"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: value ( ) :: term ( )","ref":"TypeCheck.DefaultOverrides.MapSet.html#t:value/0","title":"TypeCheck.DefaultOverrides.MapSet.value/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Module.html","title":"TypeCheck.DefaultOverrides.Module","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: def_kind() (opaque type)","ref":"TypeCheck.DefaultOverrides.Module.html#t:def_kind/0","title":"TypeCheck.DefaultOverrides.Module.def_kind/0","type":"opaque"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: definition() (opaque type)","ref":"TypeCheck.DefaultOverrides.Module.html#t:definition/0","title":"TypeCheck.DefaultOverrides.Module.definition/0","type":"opaque"},{"doc":"","ref":"TypeCheck.DefaultOverrides.NaiveDateTime.html","title":"TypeCheck.DefaultOverrides.NaiveDateTime","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t ( ) :: % NaiveDateTime { calendar : TypeCheck.DefaultOverrides.Calendar . calendar ( ) , day : TypeCheck.DefaultOverrides.Calendar . day ( ) , hour : TypeCheck.DefaultOverrides.Calendar . hour ( ) , microsecond : TypeCheck.DefaultOverrides.Calendar . microsecond ( ) , minute : TypeCheck.DefaultOverrides.Calendar . minute ( ) , month : TypeCheck.DefaultOverrides.Calendar . month ( ) , second : TypeCheck.DefaultOverrides.Calendar . second ( ) , year : TypeCheck.DefaultOverrides.Calendar . year ( ) }","ref":"TypeCheck.DefaultOverrides.NaiveDateTime.html#t:t/0","title":"TypeCheck.DefaultOverrides.NaiveDateTime.t/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Node.html","title":"TypeCheck.DefaultOverrides.Node","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: state ( ) :: :visible | :hidden | :connected | :this | :known","ref":"TypeCheck.DefaultOverrides.Node.html#t:state/0","title":"TypeCheck.DefaultOverrides.Node.state/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t ( ) :: node ( )","ref":"TypeCheck.DefaultOverrides.Node.html#t:t/0","title":"TypeCheck.DefaultOverrides.Node.t/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.OptionParser.html","title":"TypeCheck.DefaultOverrides.OptionParser","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: argv ( ) :: list ( TypeCheck.DefaultOverrides.String . t ( ) )","ref":"TypeCheck.DefaultOverrides.OptionParser.html#t:argv/0","title":"TypeCheck.DefaultOverrides.OptionParser.argv/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: errors ( ) :: list ( { TypeCheck.DefaultOverrides.String . t ( ) , TypeCheck.DefaultOverrides.String . t ( ) | nil } )","ref":"TypeCheck.DefaultOverrides.OptionParser.html#t:errors/0","title":"TypeCheck.DefaultOverrides.OptionParser.errors/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: options ( ) :: list ( { :switches , keyword ( ) } | { :strict , keyword ( ) } | { :aliases , keyword ( ) } )","ref":"TypeCheck.DefaultOverrides.OptionParser.html#t:options/0","title":"TypeCheck.DefaultOverrides.OptionParser.options/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: parsed ( ) :: keyword ( )","ref":"TypeCheck.DefaultOverrides.OptionParser.html#t:parsed/0","title":"TypeCheck.DefaultOverrides.OptionParser.parsed/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Path.html","title":"TypeCheck.DefaultOverrides.Path","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t ( ) :: TypeCheck.DefaultOverrides.IO . chardata ( )","ref":"TypeCheck.DefaultOverrides.Path.html#t:t/0","title":"TypeCheck.DefaultOverrides.Path.t/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Port.html","title":"TypeCheck.DefaultOverrides.Port","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: name ( ) :: { :spawn , charlist ( ) | binary ( ) } | { :spawn_driver , charlist ( ) | binary ( ) } | { :spawn_executable , charlist ( ) | atom ( ) } | { :fd , non_neg_integer ( ) , non_neg_integer ( ) }","ref":"TypeCheck.DefaultOverrides.Port.html#t:name/0","title":"TypeCheck.DefaultOverrides.Port.name/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Process.html","title":"TypeCheck.DefaultOverrides.Process","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: dest ( ) :: pid ( ) | port ( ) | ( registered_name :: atom ( ) ) | { registered_name :: atom ( ) , node ( ) }","ref":"TypeCheck.DefaultOverrides.Process.html#t:dest/0","title":"TypeCheck.DefaultOverrides.Process.dest/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: spawn_opt ( ) :: :link | :monitor | { :monitor , monitor_option ( ) } | { :priority , :low | :normal | :high } | { :fullsweep_after , non_neg_integer ( ) } | { :min_heap_size , non_neg_integer ( ) } | { :min_bin_vheap_size , non_neg_integer ( ) } | { :max_heap_size , heap_size ( ) } | { :message_queue_data , :off_heap | :on_heap }","ref":"TypeCheck.DefaultOverrides.Process.html#t:spawn_opt/0","title":"TypeCheck.DefaultOverrides.Process.spawn_opt/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: spawn_opts ( ) :: [ spawn_opt ( ) ]","ref":"TypeCheck.DefaultOverrides.Process.html#t:spawn_opts/0","title":"TypeCheck.DefaultOverrides.Process.spawn_opts/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Range.html","title":"TypeCheck.DefaultOverrides.Range","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: limit ( ) :: integer ( )","ref":"TypeCheck.DefaultOverrides.Range.html#t:limit/0","title":"TypeCheck.DefaultOverrides.Range.limit/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: step ( ) :: pos_integer ( ) | neg_integer ( )","ref":"TypeCheck.DefaultOverrides.Range.html#t:step/0","title":"TypeCheck.DefaultOverrides.Range.step/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t ( ) :: % Range { first : limit ( ) , last : limit ( ) , step : step ( ) }","ref":"TypeCheck.DefaultOverrides.Range.html#t:t/0","title":"TypeCheck.DefaultOverrides.Range.t/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t ( first , last ) :: % Range { first : first , last : last , step : step ( ) }","ref":"TypeCheck.DefaultOverrides.Range.html#t:t/2","title":"TypeCheck.DefaultOverrides.Range.t/2","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Regex.html","title":"TypeCheck.DefaultOverrides.Regex","type":"module"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Regex.html#regex_gen/0","title":"TypeCheck.DefaultOverrides.Regex.regex_gen/0","type":"function"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t ( ) :: wrap_with_gen ( % Regex { opts : binary ( ) , re_pattern : term ( ) , re_version : term ( ) , source : binary ( ) } , &amp; TypeCheck.DefaultOverrides.Regex . regex_gen / 0 )","ref":"TypeCheck.DefaultOverrides.Regex.html#t:t/0","title":"TypeCheck.DefaultOverrides.Regex.t/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Registry.html","title":"TypeCheck.DefaultOverrides.Registry","type":"module"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Registry.html#t:body/0","title":"TypeCheck.DefaultOverrides.Registry.body/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Registry.html#t:guard/0","title":"TypeCheck.DefaultOverrides.Registry.guard/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Registry.html#t:guards/0","title":"TypeCheck.DefaultOverrides.Registry.guards/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Registry.html#t:key/0","title":"TypeCheck.DefaultOverrides.Registry.key/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Registry.html#t:keys/0","title":"TypeCheck.DefaultOverrides.Registry.keys/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Registry.html#t:match_pattern/0","title":"TypeCheck.DefaultOverrides.Registry.match_pattern/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Registry.html#t:meta_key/0","title":"TypeCheck.DefaultOverrides.Registry.meta_key/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Registry.html#t:meta_value/0","title":"TypeCheck.DefaultOverrides.Registry.meta_value/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Registry.html#t:registry/0","title":"TypeCheck.DefaultOverrides.Registry.registry/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Registry.html#t:spec/0","title":"TypeCheck.DefaultOverrides.Registry.spec/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Registry.html#t:start_option/0","title":"TypeCheck.DefaultOverrides.Registry.start_option/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Registry.html#t:value/0","title":"TypeCheck.DefaultOverrides.Registry.value/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Stream.html","title":"TypeCheck.DefaultOverrides.Stream","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: acc ( ) :: any ( )","ref":"TypeCheck.DefaultOverrides.Stream.html#t:acc/0","title":"TypeCheck.DefaultOverrides.Stream.acc/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: default ( ) :: any ( )","ref":"TypeCheck.DefaultOverrides.Stream.html#t:default/0","title":"TypeCheck.DefaultOverrides.Stream.default/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: element ( ) :: any ( )","ref":"TypeCheck.DefaultOverrides.Stream.html#t:element/0","title":"TypeCheck.DefaultOverrides.Stream.element/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: index ( ) :: non_neg_integer ( )","ref":"TypeCheck.DefaultOverrides.Stream.html#t:index/0","title":"TypeCheck.DefaultOverrides.Stream.index/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: timer ( ) :: non_neg_integer ( ) | :infinity","ref":"TypeCheck.DefaultOverrides.Stream.html#t:timer/0","title":"TypeCheck.DefaultOverrides.Stream.timer/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.String.html","title":"TypeCheck.DefaultOverrides.String","type":"module"},{"doc":"","ref":"TypeCheck.DefaultOverrides.String.html#printable_string_gen/0","title":"TypeCheck.DefaultOverrides.String.printable_string_gen/0","type":"function"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: codepoint ( ) :: t ( )","ref":"TypeCheck.DefaultOverrides.String.html#t:codepoint/0","title":"TypeCheck.DefaultOverrides.String.codepoint/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: grapheme ( ) :: t ( )","ref":"TypeCheck.DefaultOverrides.String.html#t:grapheme/0","title":"TypeCheck.DefaultOverrides.String.grapheme/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: pattern ( ) :: t ( ) | [ t ( ) ] | TypeCheck.DefaultOverrides.Erlang.Binary . cp ( )","ref":"TypeCheck.DefaultOverrides.String.html#t:pattern/0","title":"TypeCheck.DefaultOverrides.String.pattern/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t ( ) :: wrap_with_gen ( binary ( ) , &amp; TypeCheck.DefaultOverrides.String . printable_string_gen / 0 )","ref":"TypeCheck.DefaultOverrides.String.html#t:t/0","title":"TypeCheck.DefaultOverrides.String.t/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.String.Chars.html","title":"TypeCheck.DefaultOverrides.String.Chars","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t ( ) :: impl ( String.Chars )","ref":"TypeCheck.DefaultOverrides.String.Chars.html#t:t/0","title":"TypeCheck.DefaultOverrides.String.Chars.t/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Supervisor.html","title":"TypeCheck.DefaultOverrides.Supervisor","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: child ( ) :: pid ( ) | :undefined","ref":"TypeCheck.DefaultOverrides.Supervisor.html#t:child/0","title":"TypeCheck.DefaultOverrides.Supervisor.child/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: child_spec ( ) :: %{ :id =&gt; atom ( ) | term ( ) , :start =&gt; { module ( ) , atom ( ) , [ term ( ) ] } , optional ( :restart ) =&gt; :permanent | :transient | :temporary , optional ( :shutdown ) =&gt; timeout ( ) | :brutal_kill , optional ( :type ) =&gt; :worker | :supervisor , optional ( :modules ) =&gt; [ module ( ) ] | :dynamic }","ref":"TypeCheck.DefaultOverrides.Supervisor.html#t:child_spec/0","title":"TypeCheck.DefaultOverrides.Supervisor.child_spec/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: init_option ( ) :: { :strategy , strategy ( ) } | { :max_restarts , non_neg_integer ( ) } | { :max_seconds , pos_integer ( ) }","ref":"TypeCheck.DefaultOverrides.Supervisor.html#t:init_option/0","title":"TypeCheck.DefaultOverrides.Supervisor.init_option/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: name ( ) :: atom ( ) | { :global , term ( ) } | { :via , module ( ) , term ( ) }","ref":"TypeCheck.DefaultOverrides.Supervisor.html#t:name/0","title":"TypeCheck.DefaultOverrides.Supervisor.name/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: on_start ( ) :: { :ok , pid ( ) } | :ignore | { :error , { :already_started , pid ( ) } | { :shutdown , term ( ) } | term ( ) }","ref":"TypeCheck.DefaultOverrides.Supervisor.html#t:on_start/0","title":"TypeCheck.DefaultOverrides.Supervisor.on_start/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: on_start_child ( ) :: { :ok , child ( ) } | { :ok , child ( ) , info :: term ( ) } | { :error , { :already_started , child ( ) } | :already_present | term ( ) }","ref":"TypeCheck.DefaultOverrides.Supervisor.html#t:on_start_child/0","title":"TypeCheck.DefaultOverrides.Supervisor.on_start_child/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: option ( ) :: { :name , name ( ) }","ref":"TypeCheck.DefaultOverrides.Supervisor.html#t:option/0","title":"TypeCheck.DefaultOverrides.Supervisor.option/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: strategy ( ) :: :one_for_one | :one_for_all | :rest_for_one","ref":"TypeCheck.DefaultOverrides.Supervisor.html#t:strategy/0","title":"TypeCheck.DefaultOverrides.Supervisor.strategy/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: supervisor ( ) :: pid ( ) | name ( ) | { atom ( ) , node ( ) }","ref":"TypeCheck.DefaultOverrides.Supervisor.html#t:supervisor/0","title":"TypeCheck.DefaultOverrides.Supervisor.supervisor/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.System.html","title":"TypeCheck.DefaultOverrides.System","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: signal ( ) :: :sigabrt | :sigalrm | :sigchld | :sighup | :sigquit | :sigstop | :sigterm | :sigtstp | :sigusr1 | :sigusr2","ref":"TypeCheck.DefaultOverrides.System.html#t:signal/0","title":"TypeCheck.DefaultOverrides.System.signal/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: time_unit ( ) :: :second | :millisecond | :microsecond | :nanosecond | pos_integer ( )","ref":"TypeCheck.DefaultOverrides.System.html#t:time_unit/0","title":"TypeCheck.DefaultOverrides.System.time_unit/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Task.html","title":"TypeCheck.DefaultOverrides.Task","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t ( ) :: % Task { owner : pid ( ) , pid : pid ( ) | nil , ref : reference ( ) }","ref":"TypeCheck.DefaultOverrides.Task.html#t:t/0","title":"TypeCheck.DefaultOverrides.Task.t/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Task.Supervisor.html","title":"TypeCheck.DefaultOverrides.Task.Supervisor","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: option ( ) :: TypeCheck.DefaultOverrides.DynamicSupervisor . option ( ) | TypeCheck.DefaultOverrides.DynamicSupervisor . init_option ( )","ref":"TypeCheck.DefaultOverrides.Task.Supervisor.html#t:option/0","title":"TypeCheck.DefaultOverrides.Task.Supervisor.option/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Time.html","title":"TypeCheck.DefaultOverrides.Time","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t ( ) :: % Time { calendar : TypeCheck.DefaultOverrides.Calendar . calendar ( ) , hour : TypeCheck.DefaultOverrides.Calendar . hour ( ) , microsecond : TypeCheck.DefaultOverrides.Calendar . microsecond ( ) , minute : TypeCheck.DefaultOverrides.Calendar . minute ( ) , second : TypeCheck.DefaultOverrides.Calendar . second ( ) }","ref":"TypeCheck.DefaultOverrides.Time.html#t:t/0","title":"TypeCheck.DefaultOverrides.Time.t/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.URI.html","title":"TypeCheck.DefaultOverrides.URI","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: port_number ( ) :: %{ __struct__ : Range , first : 0 , last : 65535 , step : 1 }","ref":"TypeCheck.DefaultOverrides.URI.html#t:port_number/0","title":"TypeCheck.DefaultOverrides.URI.port_number/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t ( ) :: % URI { authority : nil | binary ( ) , fragment : nil | binary ( ) , host : nil | binary ( ) , path : nil | binary ( ) , port : nil | port_number ( ) , query : nil | binary ( ) , scheme : nil | binary ( ) , userinfo : nil | binary ( ) }","ref":"TypeCheck.DefaultOverrides.URI.html#t:t/0","title":"TypeCheck.DefaultOverrides.URI.t/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Version.html","title":"TypeCheck.DefaultOverrides.Version","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: build ( ) :: TypeCheck.DefaultOverrides.String . t ( ) | nil","ref":"TypeCheck.DefaultOverrides.Version.html#t:build/0","title":"TypeCheck.DefaultOverrides.Version.build/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: major ( ) :: non_neg_integer ( )","ref":"TypeCheck.DefaultOverrides.Version.html#t:major/0","title":"TypeCheck.DefaultOverrides.Version.major/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: minor ( ) :: non_neg_integer ( )","ref":"TypeCheck.DefaultOverrides.Version.html#t:minor/0","title":"TypeCheck.DefaultOverrides.Version.minor/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: patch ( ) :: non_neg_integer ( )","ref":"TypeCheck.DefaultOverrides.Version.html#t:patch/0","title":"TypeCheck.DefaultOverrides.Version.patch/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: pre ( ) :: [ TypeCheck.DefaultOverrides.String . t ( ) | non_neg_integer ( ) ]","ref":"TypeCheck.DefaultOverrides.Version.html#t:pre/0","title":"TypeCheck.DefaultOverrides.Version.pre/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: requirement ( ) :: TypeCheck.DefaultOverrides.String . t ( ) | TypeCheck.DefaultOverrides.Version.Requirement . t ( )","ref":"TypeCheck.DefaultOverrides.Version.html#t:requirement/0","title":"TypeCheck.DefaultOverrides.Version.requirement/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t ( ) :: % Version { build : build ( ) , major : major ( ) , minor : minor ( ) , patch : patch ( ) , pre : pre ( ) }","ref":"TypeCheck.DefaultOverrides.Version.html#t:t/0","title":"TypeCheck.DefaultOverrides.Version.t/0","type":"type"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: version ( ) :: TypeCheck.DefaultOverrides.String . t ( ) | t ( )","ref":"TypeCheck.DefaultOverrides.Version.html#t:version/0","title":"TypeCheck.DefaultOverrides.Version.version/0","type":"type"},{"doc":"","ref":"TypeCheck.DefaultOverrides.Version.Requirement.html","title":"TypeCheck.DefaultOverrides.Version.Requirement","type":"module"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: matchable() (opaque type)","ref":"TypeCheck.DefaultOverrides.Version.Requirement.html#t:matchable/0","title":"TypeCheck.DefaultOverrides.Version.Requirement.matchable/0","type":"opaque"},{"doc":"(This type is managed by TypeCheck , which allows checking values against the type at runtime.) Full definition: t() (opaque type)","ref":"TypeCheck.DefaultOverrides.Version.Requirement.html#t:t/0","title":"TypeCheck.DefaultOverrides.Version.Requirement.t/0","type":"opaque"},{"doc":"Raised when during compilation of types or specifications, an irrecoverable error occurs.","ref":"TypeCheck.CompileError.html","title":"TypeCheck.CompileError","type":"exception"},{"doc":"A custom Credo check which supports the @spec! syntax. NOTE: This module is only compiled if you've added :credo as a (dev/test) dependency to your app. To use this check in your project, make sure you have a .credo.exs config file (which you can generate with mix credo gen.config ) and make sure to add it to the :checks :enabled list: %{ configs : [ %{ checks : %{ enabled : [ { Elixir.TypeCheck.Credo.Check.Readability.Specs , [ ] } , # ... ] # ... } # ... } ] } This check is an alternative to Credo's own experimental Credo.Check.Readability.Specs , so be sure to turn that check off. This check has a base priority of 0 and works with any version of Elixir. Explanation Functions, callbacks and macros need typespecs. Adding typespecs gives tools like Dialyzer and TypeCheck more information when performing checks for type errors in function calls and definitions. Typespecs will also be shown in generated documentation, and can be a great way to concisely convey how a function should be used. Using TypeCheck's @spec! syntax which will also enable runtime type-checking: (Don't forget to use TypeCheck in your module!) @spec! sub ( integer , integer ) :: integer def sub ( a , b ) , do : a + b If you do not want runtime checks, write a normal @spec : @spec add ( integer , integer ) :: integer def add ( a , b ) , do : a + b Functions with multiple arities need to have a spec defined for each arity: @spec! foo ( integer ) :: boolean @spec! foo ( integer , integer ) :: boolean def foo ( a ) , do : a &gt; 0 def foo ( a , b ) , do : a &gt; b The check only considers whether the specification is present, it doesn't perform any actual type checking while reading your code. Like all Readability issues, this one is not a technical concern. But you can improve the odds of others reading and liking your code by making it easier to follow. Check-Specific Parameters Use the following parameters to configure this check: :include_defp Include private functions. This parameter defaults to false . General Parameters Like with all checks, general params can be applied. Parameters can be configured via the .credo.exs config file .","ref":"TypeCheck.Credo.Check.Readability.Specs.html","title":"TypeCheck.Credo.Check.Readability.Specs","type":"module"},{"doc":"TypeCheck: Fast and flexible runtime type-checking for your Elixir projects.","ref":"readme.html","title":"Guide/Readme","type":"extras"},{"doc":"Type- and function specifications are constructed using (essentially) the same syntax as built-in Elixir Typespecs. When a value does not match a type check, the user is shown human-friendly error messages . Types and type-checks are generated at compiletime. This means type-checking code is optimized rigorously by the compiler. Property-checking generators can be extracted from type specifications without extra work. Automatically create a spectest which checks for each function if it adheres to its spec. Flexibility to add custom checks : Subparts of a type can be named, and 'type guards' can be specified to restrict what values are allowed to match that refer to these types.","ref":"readme.html#core-ideas","title":"Guide/Readme - Core ideas","type":"extras"},{"doc":"We add use TypeCheck to a module and wherever we want to add runtime type-checks we replace the normal calls to @type and @spec with @type! and @spec! respectively. defmodule User do use TypeCheck defstruct [ :name , :age ] @type! t :: % User { name : binary , age : integer } end defmodule AgeCheck do use TypeCheck @spec! user_older_than? ( User . t , integer ) :: boolean def user_older_than? ( user , age ) do user . age &gt;= age end end Now we can try the following: iex&gt; AgeCheck . user_older_than? ( % User { name : &quot;Qqwy&quot; , age : 11 } , 10 ) true iex&gt; AgeCheck . user_older_than? ( % User { name : &quot;Qqwy&quot; , age : 9 } , 10 ) false So far so good. Now let's see what happens when we pass values that are incorrect: iex&gt; AgeCheck . user_older_than? ( &quot;foobar&quot; , 42 ) ** (TypeCheck.TypeError) At lib/type_check_example.ex:28: The call to ` user_older_than? / 2 ` failed , because parameter no . 1 does not adhere to the spec ` % User { age : integer ( ) , name : binary ( ) } ` . Rather , its value is : ` &quot;foobar&quot; ` . Details : The call ` user_older_than? ( &quot;foobar&quot; , 42 ) ` does not adhere to spec ` user_older_than? ( % User { age : integer ( ) , name : binary ( ) } , integer ( ) ) :: boolean ( ) ` . Reason : parameter no . 1 : ` &quot;foobar&quot; ` does not check against ` % User { age : integer ( ) , name : binary ( ) } ` . Reason : ` &quot;foobar&quot; ` is not a map . ( type_check_example 0.1 . 0 ) lib / type_check_example . ex : 28 : AgeCheck . user_older_than? / 2 iex&gt; AgeCheck . user_older_than? ( % User { name : nil , age : 11 } , 10 ) ** (TypeCheck.TypeError) At lib/type_check_example.ex:28: The call to ` user_older_than? / 2 ` failed , because parameter no . 1 does not adhere to the spec ` % User { age : integer ( ) , name : binary ( ) } ` . Rather , its value is : ` % User { age : 11 , name : nil } ` . Details : The call ` user_older_than? ( % User { age : 11 , name : nil } , 10 ) ` does not adhere to spec ` user_older_than? ( % User { age : integer ( ) , name : binary ( ) } , integer ( ) ) :: boolean ( ) ` . Reason : parameter no . 1 : ` % User { age : 11 , name : nil } ` does not check against ` % User { age : integer ( ) , name : binary ( ) } ` . Reason : under key ` :name ` : ` nil ` is not a binary . ( type_check_example 0.1 . 0 ) lib / type_check_example . ex : 28 : AgeCheck . user_older_than? / 2 iex&gt; AgeCheck . user_older_than? ( % User { name : &quot;Aaron&quot; , age : nil } , 10 ) * * ( TypeCheck.TypeError ) At lib / type_check_example . ex : 28 : The call to ` user_older_than? / 2 ` failed , because parameter no . 1 does not adhere to the spec ` % User { age : integer ( ) , name : binary ( ) } ` . Rather , its value is : ` % User { age : nil , name : &quot;Aaron&quot; } ` . Details : The call ` user_older_than? ( % User { age : nil , name : &quot;Aaron&quot; } , 10 ) ` does not adhere to spec ` user_older_than? ( % User { age : integer ( ) , name : binary ( ) } , integer ( ) ) :: boolean ( ) ` . Reason : parameter no . 1 : ` % User { age : nil , name : &quot;Aaron&quot; } ` does not check against ` % User { age : integer ( ) , name : binary ( ) } ` . Reason : under key ` :age ` : ` nil ` is not an integer . ( type_check_example 0.1 . 0 ) lib / type_check_example . ex : 28 : AgeCheck . user_older_than? / 2 iex&gt; AgeCheck . user_older_than? ( % User { name : &quot;José&quot; , age : 11 } , 10.0 ) * * ( TypeCheck.TypeError ) At lib / type_check_example . ex : 28 : The call to ` user_older_than? / 2 ` failed , because parameter no . 2 does not adhere to the spec ` integer ( ) ` . Rather , its value is : ` 10.0 ` . Details : The call ` user_older_than? ( % User { age : 11 , name : &quot;José&quot; } , 10.0 ) ` does not adhere to spec ` user_older_than? ( % User { age : integer ( ) , name : binary ( ) } , integer ( ) ) :: boolean ( ) ` . Reason : parameter no . 2 : ` 10.0 ` is not an integer . ( type_check_example 0.1 . 0 ) lib / type_check_example . ex : 28 : AgeCheck . user_older_than? / 2 And if we were to introduce an error in the function definition: defmodule AgeCheck do use TypeCheck @spec! user_older_than? ( User . t , integer ) :: boolean def user_older_than? ( user , age ) do user . age end end Then we get a nice error message explaining that problem as well: ** (TypeCheck.TypeError) The call to `user_older_than?/2` failed, because the returned result does not adhere to the spec ` boolean ( ) ` . Rather , its value is : ` 26 ` . Details : The result of calling ` user_older_than? ( % User { age : 26 , name : &quot;Marten&quot; } , 10 ) ` does not adhere to spec ` user_older_than? ( % User { age : integer ( ) , name : binary ( ) } , integer ( ) ) :: boolean ( ) ` . Reason : Returned result : ` 26 ` is not a boolean . ( type_check_example 0.1 . 0 ) lib / type_check_example . ex : 28 : AgeCheck . user_older_than? / 2","ref":"readme.html#usage-example","title":"Guide/Readme - Usage Example","type":"extras"},{"doc":"Implemented [x] Proof and implementation of the basic concept [x] Custom type definitions (type, typep, opaque) [x] Basic [x] Parameterized [x] Hide implementation of opaque from documentation [x] Spec argument types checking [x] Spec return type checking [x] Spec possibly named arguments [x] Implementation of Elixir's builtin types [x] Primitive types [x] More primitive types [x] Compound types [x] special forms like | , a..b etc. [x] Literal lists [x] Maps with keys =&gt; types [x] Structs with keys =&gt; types [x] More map/list-based structures. [x] Bitstring type syntax ( &lt;&lt;&gt;&gt; , &lt;&lt;_ :: size&gt;&gt; , &lt;&lt;_ :: _ * unit&gt;&gt; , &lt;&lt;_ :: size, _ :: _ * unit&gt;&gt; ) [x] A when to add guards to typedefs for more power. [x] Make errors raised when types do not match humanly readable [x] Improve readability of spec-errors by repeating spec and which parameter did not match. [x] Creating generators from types [x] Don't warn on zero-arity types used without parentheses. [x] Hide structure of opaque and typep from documentation [x] Make sure to handle recursive (and mutually recursive) types without hanging. [x] A compile-error is raised when a type is expanded more than a million times [x] A macro called lazy is introduced to allow to defer type expansion to runtime (to within the check). [x] the Elixir formatter likes the way types+specs are constructed [x] A type impl(ProtocolName) to work with 'any type implementing protocol Protocolname '. [x] Type checks. [x] StreamData generator. [x] High code-coverage to ensure stability of implementation. [x] Make sure we handle most (if not all) of Typespec's primitive types and syntax. (With the exception of functions and binary pattern matching) [x] Option to turn @type/@opaque/@typep -injection off for the cases in which it generates improper results. [x] Manually overriding generators for user-specified types if so desired. [x] Creating generators from specs [x] Wrap spec-generators so you have a single statement to call in the test suite which will prop-test your function against all allowed inputs/outputs. [x] Option to turn the generation of runtime checks off for a given module in a particular environment ( enable_runtime_checks ). [x] Support for function-types (for typechecks as well as property-testing generators): (-&gt; result_type) (...-&gt; result_type) (param_type, param2_type -&gt; result_type) [x] Basic support for maps with a single required(type) or optional(type) . [x] Overrides for builtin remote types ( String.t , Enum.t , Range.t , MapSet.t etc.) (75% done) Details [x] Overrides for more builtin remote types [x] Support for maps with mixed required(type) and optional(type) syntaxes. [x] Configurable setting to turn checks on/off at compile-time, on a per-OTP-app basis (so you have control over your dependencies) as well as your individual modules. [x] Hide named types from opaque types. [x] A way to define structs and their field types at the same time. [x] Finalize formatter specification and make a generator for this so that people can easily test their own formatters. Pre-stable Longer-term future ideas [ ] Per-module or even per-spec settings to turn on/off, configure formatter, etc. Changelog 0.13.1 - Fixes: Fixes issues with aliases to remote types not being followed. (c.f. #160) 0.13.0 - Additions: Support for maps with multiple optional atom keys. Thank you very much, @dvic! (c.f. #165) Fixes: Fixes 'Enumerable not implemented' crash when passing a struct to a type expecting a fixed map. (c.f. #161) 0.12.4 - Fixes: Allow syntax %{String.t() =&gt; any()} as shorthand for %{required(String.t()) =&gt; any()} . (c.f. #152) Situations in which the fully qualified name of a type defined in the current module were not picked up ( Example.t() not being found inside specs in Example . even though Example defined a type called t() ). (c.f. #154) A problem with using types with type guards in other modules. (c.f. #147) 0.12.3 - Fixes: Compatibility problems with Elixir v1.14. ( Code.Identifier.inspect_as_key , working with the AST of imported functions) Thank you very much, @marcandre! (c.f. #150) 0.12.2 - Fixes: Compilation error in a new Phoenix 1.16 project. Thank you very much, @assimelha! (c.f. #114) 0.12.1 - Performance: Significant reduction on generated code size of runtime type checks. This will speed up compilation roughly ~10-50%. Fixes: No longer use white ANSI color in exception formatting, ensuring messages of the DefaultFormatter are also visible on white backgrounds (Livebook, anyone? :D) Skip OTP app lookup (used to fetch per OTP app compile-time configuration) if we're not compiled inside an OTP app (such as inside a Livebook). Improve generator for Regex.t() to generate only correct regular expressions. Thank you very much, @skwerlman!(c.f. #133) 0.12.0 - Additions: The default options used are now fetched from the application configuration. This means that you can configure a default for your app as well as for each of your dependencies(!) by adding config :app_name, :type_check [...] to your configuration file(s). (c.f. #61) TypeCheck.External module, with functions to work with typespecs in modules outside of your control. Thank you very much, @orsinium! (c.f. #113) fetch_spec to build a TypeCheck type from any function that has a @spec . fetch_type to build a TypeCheck type from any @type . enforce_spec! to wrap a call to any function that has a @spec with a runtime type-check on the input parameters and return value. apply and apply! to wrap a call to any function with the function spec type that you give it. TypeCheck.Defstruct.defstruct! , a way to combine defstruct , @enforce_keys and the creation of the struct's type, reducing boilerplate and the possibility of mistakes. (c.f. #118) Fixes: Long-standing issue where Dialyzer would sometimes complain in apps using TypeCheck is resolved. (c.f. #95) Creation of the new maybe_nonempty_list type will no longer get stuck in an infinite loop on creation. (c.f. #120) 0.11.0 - Additions: Support for fancier map syntaxes: %{required(key_type) =&gt; value_type} Maps with a single kind of required key-value type. %{optional(key_type) =&gt; value_type} Maps with a single kind of optional key-value type. %{:some =&gt; a(), :fixed =&gt; b(), :keys =&gt; c(), optional(atom()) =&gt; any()} Maps with any number of fixed keys and a single optional key-value type. TypeCheck now supports nearly all kinds of map types that see use. Archaic combinations of optional and required are not supported, but also not very useful types in practice. Because of this, the inspection of the builtin type map(key, value) has been changed to look the same as an optional map. This is a minor backwards-incompatible change. Desugaring %{} has changed from 'any map' to 'the empty map' in line with Elixir's Typespecs. This is a minor backwards-incompatible change. Support for the builtin types port() , reference() and (based on these) identifier() . Support for the builtin type struct() . Support for the builtin type timeout() . Support for the builtin type nonempty_charlist() and maybe_improper_list and (based on these) iolist() and iodata() . Adding types depending on these builtins to the default type overrides. We now support all modules of the full standard library! TypeCheck.Credo.Check.Readability.Specs : an opt-in alternative Credo check which will check whether all functions have either a @spec! or 'normal' @spec . (Fixes #102). 0.10.8 - Fixes: Ensures that the Inspect protocol is properly implemented for sized bitstring types (c.f. #104). Thank you very much, @trarbr! 0.10.7 - Fixes: Ensures fixed-maps are checked for superfluous keys (c.f. #96). Thank you very much, @spatrik! 0.10.6 - Fixes: Fixes issue where Dialyzer would sometimes complain about the generated return-type checking code if the implementation of the function itself was trivial. (c.f. #85). 0.10.5 - Additions: Support for the builtin types of the System module. Thank you, @paulswartz! 0.10.4 - Fixes: Fixes issue where sometimes results of specs returning a struct would remove some or all struct fields. (c.f. #78) Related: Ensures that when a @type! containing a %__MODULE__{} is used before that module's defstruct , a clear CompileError is created (c.f. #83). 0.10.3 - Fixes: Fixes issue when TypeCheck specs were used inside a defprotocol module or other modules where the normal def / defp macros are hidden/overridden. 0.10.2 - Fixes: Fixes issue where FixedMaps would accept maps any maps (even those missing the required keys) sometimes. (c.f. #74) 0.10.1 - Fixes: Swaps Murmur out for Erlang's builtin :erlang.phash2/2 to generate data for function-types, allowing the removal of the optional dependency on the :murmur library. 0.10.0 - Additions Support for function-types (for typechecks as well as property-testing generators): (-&gt; result_type) (...-&gt; result_type) (param_type, param2_type -&gt; result_type) Fixes: Wrapping private functions no longer make the function public. (c.f. #64) Wrapping macros now works correctly. (also related to #64) Using __MODULE__ inside a struct inside a type now expands correctly. (c.f. #66) 0.9.0 - Support for bitstring type syntax: &lt;&lt;&gt;&gt; , &lt;&lt;_ :: size&gt;&gt; , &lt;&lt;_ :: _ * unit&gt;&gt; , &lt;&lt;_ :: size, _ :: _ * unit&gt;&gt; (both as types and as generators) 0.8.2 - Fixed compiler warnings when optional dependency StreamData is not installed. Fixed pretty-printing of typedoc of opaque types. 0.8.1 - Improved documentation with a page comparing TypeCheck with Elixir's plain typespecs. (Thank you very much, @baldwindavid ) Addition of missing override for the type t Range.t/0 . (Thank you very much, @baldwindavid ) 0.8.0 - Fixes prettyprinting of TypeCheck.Builtin.Range . Addition of require TypeCheck.Type to use TypeCheck so there no longer is a need to call this manually if you want to e.g. use TypeCheck.Type.build/1 . Pretty-printing of types and TypeError output in multiple colors. Nicer indentation of errors. named types are now printed in abbreviated fashion if they are repeated multiple times in an error message. This makes a nested error message much easier to read, especially for larger specs. [type] no longer creates a fixed_list(type) but instead a list(type) (just as Elixir's own typespecs.) Support for [...] and [type, ...] as alias for nonempty_list() and nonempty_list(type) respectively. Remove support for list literals with multiple elements. Improved documentation. Thank you, @0ourobor0s! 0.7.0 - Addition of the option enable_runtime_checks . When false, all runtime checks in the given module are completely disabled. Adding DateTime.t to the default overrides, as it was still missing. 0.6.0 - Addition of spectest &amp; 'default overrides' Elixir's standard library types: Adding TypeCheck.ExUnit , with the function spectest to test function-specifications. Possibility to use options :except , :only , :initial_seed . Possibility to pass custom options to StreamData. Adding TypeCheck.DefaultOverrides with many sub-modules containing checked typespecs for the types in Elixir's standard library (75% done). Ensure that these types are correct also on older Elixir versions (1.9, 1.10, 1.11) By default load these 'DefaultOverrides', but have the option to turn this behaviour off in TypeCheck.Option . Nice generators for Enum.t , Collectable.t , String.t . Support for the builtin types: pid() nonempty_list() , nonempty_list(type) . Allow use TypeCheck in IEx or other non-module contexts, to require TypeCheck and import TypeCheck.Builtin in the current scope (without importing/using the macros that only work at the module level.) The introspection function __type_check__/1 is now added to any module that contains a use TypeCheck . Fixes the Inspect implementation of custom structs, by falling back to Any , which is more useful than attempting to use a customized implementation that would try to read the values in the struct and failing because the struct-type containing types in the fields. Fixes conditional compilation warnings when optional dependency :stream_data was not included in your project. 0.5.0 - Stability improvements: Adding Typecheck.Option debug: true , which will (at compile-time) print the checks that TypeCheck is generating. Actually autogenerate a @spec , which did not yet happen before. When writing @autogen_typespec false , no typespec is exported for the next @type! / @opaque / @spec! encountered in a module. Code coverage increased to 85% Bugfixes w.r.t. generating typespecs Fixes compiler-warnings on unused named types when using a type guard. (c.f. #25) Fixes any warnings that were triggered during the test suite before. 0.4.0 - Support for impl(ProtocolName) to accept any type implementing a particular protocol. Also adds rudimentary support for overriding remote types. Bugfix when inspecting lazy( ...) -types. 0.3.2 - Support for unquote fragments inside types and specs. (c.f. #39) 0.3.1 - Fixed link in the documentation. 0.3.0 - Improve DefaultFormatter output when used with long function- or type-signatures (c.f. #32). Also, bugfix for Builtin.tuple/1 . 0.2.3 - Bugfix release: Ensure TypeCheck compiles on Elixir v1.11 (#30), Ensure StreamData truly is an optional dependency (#27). 0.2.2 - Support for literal strings should no longer break in Elixir's builtin typespecs. 0.2.1 - Improved parsing of types that have a type-guard at the root level. (c.f. #24), support for custom generators. 0.2.0 - Improved (and changed) API that works better with the Elixir formatter: Use @type! / @spec! instead, support named types in specs. 0.1.2 - Added missing keyword type to TypeCheck.Builtin (#20) 0.1.1 - Fixing some documentation typos 0.1.0 - Initial Release","ref":"readme.html#features-roadmap","title":"Guide/Readme - Features &amp; Roadmap","type":"extras"},{"doc":"TypeCheck is available in Hex . The package can be installed by adding type_check to your list of dependencies in mix.exs : def deps do [ { :type_check , &quot;~&gt; 0.12.1&quot; } , # To allow spectesting and property-testing data generators (optional): { :stream_data , &quot;~&gt; 0.5.0&quot; , only : :test } , ] end The documentation can be found at https://hexdocs.pm/type_check . Formatter TypeCheck exports a couple of macros that you might want to use without parentheses. To make mix format respect this setting, add import_deps: [:type_check] to your .formatter.exs file.","ref":"readme.html#installation","title":"Guide/Readme - Installation","type":"extras"},{"doc":"TypeCheck is by no means the other solution out there to reduce the number of bugs in your code. Elixir's builtin typespecs and Dialyzer Elixir's builtin type-specifications use the same syntax as TypeCheck. They are however not used by the compiler or the runtime, and therefore mainly exist to improve your documentation. Besides documentation, extra external tools like Dialyzer can be used to perform static analysis of the types used in your application. Dialyzer is an opt-in static analysis tool. This means that it can point out some inconsistencies or bugs, but because of its opt-in nature, there are also many problems it cannot detect, and it requires your dependencies to have written all of their typespecs correctly. Dialyzer is also (unfortunately) infamous for its at times difficult-to-understand error messages. An advantage that Dialyzer has over TypeCheck is that its checking is done without having to execute your program code (thus not having any effect on the runtime behaviour or efficiency of your projects). Because TypeCheck adds @type , @typep , @opaque and @spec -attributes based on the types that are defined, it is possible to use Dialyzer together with TypeCheck. Norm Norm is an Elixir library for specifying the structure of data that can be used for both validation and data-generation. On a superficial level, Norm and TypeCheck seem similar. However, there are important differences in their design considerations .","ref":"readme.html#typecheck-compared-to-other-tools","title":"Guide/Readme - TypeCheck compared to other tools","type":"extras"},{"doc":"yes","ref":"readme.html#is-it-any-good","title":"Guide/Readme - Is it any good?","type":"extras"},{"doc":"Type-checking and spec-testing with TypeCheck TypeCheck is an elixir library to, you guessed it, check the types of the values, variables and functions in your Elixir projects. Elixir is a strong, dynamically typed programming language. 'Strong' (as opposed to 'weak') means that when we try to perform an unsupported operation on a value (like 'multiplying strings'), we get an error, rather than silent faulty behaviour. 'Dynamic' (as opposed to 'Static') means that which operations we do with our values is not checked at compile-time, but only once the program itself is running. However, when such a failure happens at runtime, the resulting error (and more importantly: what the cause was of an error) is not always very clear. Is it a bug in your code? Or a bug in a library you are using? Where in the deeply nested tree of structs is the problem that results in a ** (BadFunctionError) expected a function, got: nil somewhere deep inside the codebase? Related to this, we would like to catch problems early in the development cycle: if we find an error while writing our code initially, then fixing it takes significantly less effort (and is much less costly) than if the problem happens in an application which is already deployed to production. TypeCheck gives you the handholds to tackle this situation.","ref":"type-checking-and-spec-testing-with-typecheck.html","title":"Introducing TypeCheck","type":"extras"},{"doc":"Elixir (and Erlang) come with a nice description for the types of the values passed between functions, called 'Typespecs' . However, by default these typespecs are just used for documentation. They are not used in any way to restrict, or even warn when your code is not following them correctly. Somewhat more recently, tools like Dialyzer / Dialyxir have been introduced in the ecosystem. These perform a static check of the source-code (that is, they look at the code as written without executing it) to catch some potential mis-uses. Dialyzer has however three commonly mentioned drawbacks: On larger codebases it becomes prohibitively slow to run; It contains both 'false negatives' (improper usage that it missed) as well as 'false positives' (warnings about things which are actually OK); Some warnings can be outright cryptic to resolve. Dialyzer is still a very useful tool, but it is not a cure-all. As such, there was definite room in this space for improvement.","ref":"type-checking-and-spec-testing-with-typecheck.html#what-about-elixir-s-built-in-typespecs-and-dialyzer","title":"Introducing TypeCheck - What about Elixir's built-in typespecs and Dialyzer?","type":"extras"},{"doc":"Using TypeCheck is as simple as adding a use TypeCheck to the module(s) you want to add checks to. For each type-specification ('type') and function-specification ('spec') which is defined using TypeCheck, the following four features become available: 'Normal' Elixir Typespecs for usage with pre-existing external tools (like e.g. Dialyzer). Documentation with extra detail not found in 'Normal' Elixir typespecs. Run-time type checks for all parameters to a function (if there is a failure, the function is used improperly) and of the returned value (if this fails, the function has a mistake). Data generators for all types (and specs), for usage in testing, especially property-tests and spectests (explained below). Let's take a look at how TypeCheck can be used in practice. This will help to see how the run-time type checks work, as well as how you can use TypeCheck's spectests to supercharge your testing.","ref":"type-checking-and-spec-testing-with-typecheck.html#introducing-typecheck","title":"Introducing TypeCheck - Introducing TypeCheck","type":"extras"},{"doc":"Let's say we are writing a module to work with five-star ratings. defmodule Rating do @type t ( ) :: % Rating { value : 1 .. 5 , author : String . t ( ) } defstruct [ :value , :author ] @spec average ( list ( t ( ) ) ) :: number ( ) def average ( ratings ) do values = Enum . map ( ratings , &amp; ( &amp;1 . value ) ) Enum . sum ( values ) / Enum . count ( values ) end end In good Elixir style, the functions and the struct have already received type signatures, even though they are currently only used for documentation. (Speaking of documentation: it is good practice to add documentation to all public modules and functions. To keep the examples in this article brief, they have been elided.) From the function-specification, you can already see how the function average is intended to be used: When passing a list of ratings, we will return a single number: the average. Let's try calling it with a couple of inputs: iex(8)&gt; Rating . average ( [ % Rating { author : &quot;Joe&quot; , value : 5 } , % Rating { author : &quot;Mike&quot; , value : 4 } ] ) 4.5 iex(8)&gt; Rating . average ( [ % Rating { author : &quot;Robert&quot; , value : 3 } ] ) 3.0 So far so good. Now, let's try what happens when someone makes a mistake: iex&gt; Rating . average ( % Rating { author : &quot;José&quot; , value : 3 } ) ** (Protocol.UndefinedError) protocol Enumerable not implemented for %Rating{author: &quot;José&quot;, value: 3} of type Rating (a struct). This protocol is implemented for the following type(s): Map, Range, List, MapSet, GenEvent.Stream, Stream, Date.Range, HashDict, IO.Stream, HashSet, Function, File.Stream (elixir 1.12.0) lib/enum.ex:1: Enumerable.impl_for!/1 (elixir 1.12.0) lib/enum.ex:141: Enumerable.reduce/3 (elixir 1.12.0) lib/enum.ex:3923: Enum.map/2 iex:19: Rating.average/1 Oof! Clearly something is going wrong here, but if one were to encounter this error somewhere deep in a codebase, it would not be immediately obvious that the problem was that we were calling the function incorrectly. Even worse is if we happen to pass a list of non-ratings to the function: iex&gt; Rating . average ( [ 1 , 2 , 3 ] ) ** (ArgumentError) you attempted to apply :value on 1. If you are using apply/3, make sure the module is an atom. If you are using the dot syntax, such as map.field or module.function(), make sure the left side of the dot is an atom or a map :erlang.apply(1, :value, []) iex:19: anonymous fn/1 in Rating.average/1 (elixir 1.12.0) lib/enum.ex:1553: Enum.&quot;-map/2-lists^map/1-0-&quot;/2 iex:19: Rating.average/1 And finally, there is nothing preventing the creation of malformed rating-objects. While we have specified in our type that the rating's value should only ever be in the range 1..5, this is not constrained anywhere in the code. And if someone passes a nil rating, we'd get a ** (ArithmeticError) bad argument in arithmetic expression: nil + 0 error. Also not very clear. While we could sprinkle checks for this everywhere, this would in the best case result in extremely 'defensive' and badly readable code. And in the worst case, we might forget to add a check at certain places, still not giving us certainty. Let's see how TypeCheck can improve this situation.","ref":"type-checking-and-spec-testing-with-typecheck.html#an-example","title":"Introducing TypeCheck - An Example","type":"extras"},{"doc":"In general, adding TypeCheck to a module only requires the following two steps: Add use TypeCheck at the top of the module. Replace all usage of @type with @type! and all usage of @spec with @spec! . (For the curious: There indeed are similarly overloaded versions of @typep! and @opaque! available.) You're done! Our example ratings module now looks like this: defmodule Rating do use TypeCheck @type! t ( ) :: % Rating { value : 1 .. 5 , author : String . t ( ) } defstruct [ :value , :author ] @spec! average ( list ( t ( ) ) ) :: number ( ) def average ( ratings ) do values = Enum . map ( ratings , &amp; ( &amp;1 . value ) ) Enum . sum ( values ) / Enum . count ( values ) end end With this change, correct usage of the function still returns the expected results: iex(8)&gt; Rating . average ( [ % Rating { author : &quot;Joe&quot; , value : 5 } , % Rating { author : &quot;Mike&quot; , value : 4 } ] ) 4.5 iex(8)&gt; Rating . average ( [ % Rating { author : &quot;Robert&quot; , value : 3 } ] ) 3.0 So far, so good. Now let's look at what happens when the function is used incorrectly: iex(20)&gt; Rating . average ( % Rating { author : &quot;José&quot; , value : 3 } ) ** (TypeCheck.TypeError) At iex:17: The call to `average/1` failed, because parameter no. 1 does not adhere to the spec `list(%Rating{author: binary(), value: number()})`. Rather, its value is: `%Rating{author: &quot;José&quot;, value: 3}`. Details: The call `average(%Rating{author: &quot;José&quot;, value: 3})` does not adhere to spec `average(list(%Rating{author: binary(), value: number()})) :: number()`. Reason: parameter no. 1: `%Rating{author: &quot;José&quot;, value: 3}` does not check against `list(%Rating{author: binary(), value: number()})`. Reason: `%Rating{author: &quot;José&quot;, value: 3}` is not a list. lib/type_check/spec.ex:165: Rating.average/1 Look at that! Not only do we see that the problem is is caused by the call to the function itself, but the cause of the problem is also very clear: the passed value is not a list. Here's our second mistake: iex&gt; Rating . average ( [ 1 , 2 , 3 ] ) ** (TypeCheck.TypeError) At iex:17: The call to `average/1` failed, because parameter no. 1 does not adhere to the spec `list(%Rating{author: binary(), value: number()})`. Rather, its value is: `[1, 2, 3]`. Details: The call `average([1, 2, 3])` does not adhere to spec `average(list(%Rating{author: binary(), value: number()})) :: number()`. Reason: parameter no. 1: `[1, 2, 3]` does not check against `list(%Rating{author: binary(), value: number()})`. Reason: at index 0: `1` does not check against `%Rating{author: binary(), value: number()}`. Reason: `1` is not a map. lib/type_check/spec.ex:165: Rating.average/1 This time too, the error is much clearer, showing that the innermost reason of the type-checking failure is that the passed number is not a map. Now, let's also look at what happens when we pass a malformed Rating struct: iex&gt; Rating . average ( [ % Rating { author : &quot;root&quot; , value : - 100 } ] ) ** (TypeCheck.TypeError) At iex:21: The call to `average/1` failed, because parameter no. 1 does not adhere to the spec `list(%Rating{author: binary(), value: 1..5})`. Rather, its value is: `[%Rating{author: &quot;root&quot;, value: -100}]`. Details: The call `average([%Rating{author: &quot;root&quot;, value: -100}])` does not adhere to spec `average(list(%Rating{author: binary(), value: 1..5})) :: number()`. Reason: parameter no. 1: `[%Rating{author: &quot;root&quot;, value: -100}]` does not check against `list(%Rating{author: binary(), value: 1..5})`. Reason: at index 0: `%Rating{author: &quot;root&quot;, value: -100}` does not check against `%Rating{author: binary(), value: 1..5}`. Reason: under key `:value`: `-100` does not check against `1..5`. Reason: `-100` falls outside the range 1..5. lib/type_check/spec.ex:165: Rating.average/1 Much nicer! Now, all functions which have a @spec! will have there inputs checked properly, and passing them a malformed rating-struct is impossible without cluttering our code. As a final example, say we alter the implementation of the function to something which is clearly wrong, such as always returning the string &quot;something else&quot; : iex&gt; Rating . average ( [ % Rating { author : &quot;Joe&quot; , value : 5 } , % Rating { author : &quot;Mike&quot; , value : 4 } ] ) ** (TypeCheck.TypeError) The call to `average/1` failed, because the returned result does not adhere to the spec `number()`. Rather, its value is: `&quot;something else&quot;`. Details: The result of calling `average([%Rating{author: &quot;Marten&quot;, value: 1}])` does not adhere to spec `average( list(%Rating{author: binary(), value: 1..5})) :: number()`. Reason: Returned result: `&quot;something else&quot;` is not a number. lib/type_check/spec.ex:194: Rating.average/1 In this case, you can clearly see that the problem is caused by the return value, and why.","ref":"type-checking-and-spec-testing-with-typecheck.html#adding-typecheck","title":"Introducing TypeCheck - Adding TypeCheck","type":"extras"},{"doc":"TypeCheck adds its runtime checks by wrapping your functions (using defoverridable ). This means that the Elixir and Erlang compilers are able to optimize the checks to their liking. Therefore, code added by TypeCheck is at the very least not slower than any hand-written parameter-checking code. In many cases, the compiler is even smart enough to combine the type-check with a case , if or cond -expression that your function implementation itself contains, so even in those cases no duplicate checks are done. That said, there are certain cases in which the type-checks might still be too slow, since by default TypeCheck performs a deep check for all parameters to a function. In the case of for instance a large collection or a deeply-nested 'tree of structs' this might still be too slow. In those cases, you might want to turn off TypeCheck in certain environments (such as production) while still keeping it available in the development and testing environments. Be sure to benchmark before making the decision to turn TypeCheck off, as there is a high probability that the bottlenecks in your code are actually found elsewhere. And finally, regardless of whether the checks are used or not, there is one more way in which TypeCheck's types and specs are useful: Automated testing.","ref":"type-checking-and-spec-testing-with-typecheck.html#efficiency-of-runtime-checks","title":"Introducing TypeCheck - Efficiency of runtime checks","type":"extras"},{"doc":"Runtime type-checks give an early notice whether a function is being used properly or improperly. In essence, we check 'Does the caller of the function adhere to the function's specification?' However, to get more certainty about the correctness¹ of our code, we'd also like to check the opposite: 'Does the function itself adhere to its specification?' This is where 'function-specification tests', or spectests for short, come in. What is a spectest? A spectest is a property-based test in which we check whether the function adheres to its specification's invariants (also known as the function's contract or its preconditions and postconditions ). This is done by generating a large amount of possible function inputs, and for each of these, check whether the function: Does not raise an exception. Returns a result that type-checks against the spec's return-type. Spectests are given its own test-category in ExUnit, for easier recognition (Just like 'doctests' and 'properties' are different from normal tests, so are 'spectests'.) If you're new to property-based testing, then 'Overview of Property-based testing' -section of the ExUnitProperties' documentation might be a good place for a general overview. ¹: Because of the nature of property-based testing, we can never know for 100% sure that a function is correct. However, with every new randomly-generated test-case, the level of confidence grows a little. So while we can never by fully sure, we are able to get asymptotically close to it. Spectesting our example To add a spectest, we need to use TypeCheck.ExUnit in our testing file, and then call spectest YourModuleName . This accepts options similarly to doctest which you might already be familiar with. To use spectests (and data generation in general), TypeCheck requires on the optional dependency StreamData , so be sure to add it to your project's mix deps. Let's spectest the Rating module, to make sure our code does not contain any mistakes. defmodule RatingTest do use ExUnit.Case , async : true use TypeCheck.ExUnit spectest Rating end $ mix test 1 ) spectest average ( list ( % Rating { author : binary ( ) , value : 1 .. 5 } ) ) :: number ( ) ( RatingTest ) test / rating_test . exs : 5 Spectest failed ( after 0 successful runs ) Input : Rating . average ( [ ] ) ** (ArithmeticError) bad argument in arithmetic expression code : # TypeCheck.Spec &lt; average ( list ( % Rating { author : binary ( ) , value : 1 .. 5 } ) ) :: number ( ) &gt; stacktrace : ( type_check_guide 0.1 . 0 ) lib / rating . ex : 10 : Rating . &quot;average (overridable 1)&quot; / 1 lib / type_check / ex_unit . ex : 5 : anonymous fn / 1 in RatingTest . &quot;spectest average( \\n list(%Rating{author: binary(), value: 1..5}) \\n ) \\n :: \\n number()&quot; / 1 ( stream_data 0.5 . 0 ) lib / stream_data . ex : 2102 : StreamData . check_all / 7 lib / type_check / ex_unit . ex : 5 : ( test ) .. Finished in 0.1 seconds ( 0.1 s async , 0.00 s sync ) 1 doctest , 1 spectest , 1 test , 1 failure Oof! Our code contained a mistake! It turns out that when Rating.average is given an empty list, we end up dividing by zero! Some math-savvy readers might have seen this coming from a mile away. However, I'm sure that some of you will have been surprised by this problem 😇. Of course this is only an educational example of the kind of issues one might encounter in a real codebase. We might resolve this issue in two ways: Decide that an empty list should never be passed, and therefore restrict the parameter types further. For instance, we could change it from a list(Rating.t()) to a nonempty_list(Rating.t()) . This means that when someone tries passing it an empty list, they will immediately be notified that this is not supported, using the clear error messages as seen in the earlier examples. Changing the spec this way will also make the spectest pass, as empty lists will no longer be generated. Decide that empty lists are a correct input, but that the output will be changed from number() to {:ok, number()} | {:error, :empty} , asking code which uses average to handle the possibility of an error-result being returned. In either case, after these changes the spectest will pass.","ref":"type-checking-and-spec-testing-with-typecheck.html#spectests","title":"Introducing TypeCheck - Spectests","type":"extras"},{"doc":"While spectests are a very simple and code-light way to test your functions, it is also possible to generate arbitrary values from any type outside of a spectest , for use in other, more specialized property-based tests. See TypeCheck.Type.StreamData.to_gen/1 for more info.","ref":"type-checking-and-spec-testing-with-typecheck.html#more-general-properties","title":"Introducing TypeCheck - More general properties","type":"extras"},{"doc":"We have covered the most important features of TypeCheck, but there is more to discover, such as: The error-formatter which is reconfigurable and could be overloaded by custom implementations. TypeCheck itself is extensively property-tested using its own types. Most of TypeCheck's type-construction functions use TypeCheck's checks itself, which mean that you will get the same clear error messages when making a mistake while constructing a type (but this time during compile-time . nifty, eh?!) TypeCheck supports 'type guards' for if you want to add extra custom constraints to any particular type. TypeCheck supports 'unquote fragments' for when you want to go ham with metaprogramming.","ref":"type-checking-and-spec-testing-with-typecheck.html#there-is-more","title":"Introducing TypeCheck - There is more","type":"extras"},{"doc":"In this guide, you have seen how TypeCheck can be used and what value it can add to your projects. We have seen how TypeCheck can be used in a general project to add runtime checks to your functions, as well as how to use the spectest macro to get automatic property-tests that check whether your functions follow their specs. TypeCheck currently is at version 0.7.0 and in active development. Feedback, issues and pull requests are very welcome ! Thank you very much for sticking through this long read with me 💚. I wish you a wonderful day! ~Marten/Qqwy","ref":"type-checking-and-spec-testing-with-typecheck.html#summary","title":"Introducing TypeCheck - Summary","type":"extras"},{"doc":"TypeCheck is intended to be a drop-in supplement to Elixir typespecs. With very few exceptions, all of Elixir's typespecs are supported by TypeCheck. Below is a breakdown of supported typespec syntax in TypeCheck. In the tables below: '✅' indicates that something is supported '❌' indicates that something is not supported. '⌛' indicates that something is not currently supported, but there are plans to add support for it in the near future.","ref":"comparing-typecheck-and-elixir-typespecs.html","title":"Comparison to Plain Typespecs","type":"extras"},{"doc":"Type Supported? Notes any() ✅ the top type, the set of all terms none() ✅ the bottom type, contains no terms atom() ✅ map() ✅ any map pid() ✅ process identifier port() ✅ port identifier reference() ✅ tuple() ✅ tuple of any size float() ✅ integer() ✅ neg_integer() ✅ ..., -3, -2, -1 non_neg_integer() ✅ 0, 1, 2, 3, ... pos_integer() ✅ 1, 2, 3, ... list(type) ✅ proper list nonempty_list(type) ✅ non-empty proper list maybe_improper_list(content_type, termination_type) ✅ proper or improper list nonempty_improper_list(content_type, termination_type) ✅ improper list nonempty_maybe_improper_list(content_type, termination_type) ✅ non-empty proper or improper list","ref":"comparing-typecheck-and-elixir-typespecs.html#basic-types","title":"Comparison to Plain Typespecs - Basic Types","type":"extras"},{"doc":"Type Supported? Notes true ✅ false ✅ nil ✅ &lt;&lt;&gt;&gt; ✅ empty bitstring &lt;&lt;_::size&gt; ✅ size is 0 or a positive integer &lt;&lt;_::_*unit&gt;&gt; ✅ unit is an integer from 1 to 256 &lt;&lt;_::size, _::_*unit&gt;&gt; ✅ (-&gt; type) ✅¹ 0-arity, returns type (type1, type2 -&gt; type) ✅¹ 2-arity, returns type (... -&gt; type) ✅¹ any arity, returns type 1 ✅ integer 1..10 ✅ range [type] ✅ list with any number of type elements [] ✅ empty list [...] ✅ shorthand for nonempty_list(any()) [type, ...] ✅ shorthand for nonempty_list(type) [key: value_type] ✅ keyword list with key :key of value_type %{} ✅ empty map %{key: value_type} ✅ map with required (atom) key :key of value_type %{key_type =&gt; value_type} ✅ map with required pairs of key_type and value_type %{required(key_type) =&gt; value_type} ✅² map with required pairs of key_type and value_type %{optional(key_type) =&gt; value_type} ✅² map with optional pairs of key_type and value_type %SomeStruct{} ✅ struct with all fields of any type %SomeStruct{key: value_type} ✅ struct with required key :key of value_type %{key: value_type, optional(opt_key_type) =&gt; opt_value_type} ✅³ struct with required key :key of value_type, and zero or more pairs of opt_key_type and opt_value_type {} ✅ empty tuple {:ok, type} ✅ two-element tuple with an atom and any type ¹: Functions passed as parameters can only be fully checked once they are called. TypeCheck wraps them in a 'wrapper function' which performs the correct check on their input/output. This wrapper will only run once the the function actually is called. ²: Only a single 'required' or 'optional' field in a map is currently supported. (Types which need more are fortunately very rare.) ³: Only optional is currently supported, and only a single one. (Types which need more are fortunately very rare.)","ref":"comparing-typecheck-and-elixir-typespecs.html#literals","title":"Comparison to Plain Typespecs - Literals","type":"extras"},{"doc":"Type Supported? Notes term() ✅ any() arity() ✅ 0..255 as_boolean(t) ✅ t binary() ✅ &lt;&lt;_::_*8&gt;&gt; bitstring() ✅ &lt;&lt;_::_*1&gt;&gt; boolean() ✅ true | false byte() ✅ 0..255 char() ✅ 0..0x10FFFF charlist() ✅ [char()] nonempty_charlist() ✅ [char(), ...] fun() ✅ (... -&gt; any) function() ✅ fun() identifier() ✅ pid() | port() | reference() iodata() ✅ iolist() | binary() iolist() ✅ maybe_improper_list(byte() | binary() | iolist(), binary() | []) keyword() ✅ [{atom(), any()}] keyword(t) ✅ [{atom(), t}] list() ✅ [any()] nonempty_list() ✅ nonempty_list(any()) maybe_improper_list() ✅ maybe_improper_list(any(), any()) nonempty_maybe_improper_list() ✅ nonempty_maybe_improper_list(any(), any()) mfa() ✅ {module(), atom(), arity()} module() ✅ atom() no_return() ✅ none() node() ✅ atom() number() ✅ integer() | float() struct() ✅ %{:__struct__ =&gt; atom(), optional(atom()) =&gt; any()} timeout() ✅ :infinity | non_neg_integer()","ref":"comparing-typecheck-and-elixir-typespecs.html#built-in-types","title":"Comparison to Plain Typespecs - Built-in types","type":"extras"},{"doc":"TypeCheck adds the following extensions on Elixir's builtin typespec syntax: Type Notes impl(protocol_name) Checks whether the given value implements the particular protocol fixed_list(element_types) Fixed size where element_types dictate types tuple(size) Any types, but which has exactly size elements map(key_type, value_type) Zero or more keys of key_type and values of value_type type when guard A 'type guard', an extra check implemented in arbitrary code (see below)","ref":"comparing-typecheck-and-elixir-typespecs.html#typecheck-additions","title":"Comparison to Plain Typespecs - 🚀 TypeCheck Additions","type":"extras"},{"doc":"Type Supported? Notes @type ✅ as @type! @opaque ✅ as @opaque! @typep ✅ as @typep! @spec ✅ as @spec! ✅ Basic Spec Definition # typespecs @spec function_name ( type1 , type2 ) :: return_type # TypeCheck @spec! function_name ( type1 , type2 ) :: return_type ❌ Spec Guards # typespecs @spec function ( arg ) :: [ arg ] when arg : atom @spec function ( arg1 , arg2 ) :: { arg1 , arg2 } when arg1 : atom , arg2 : integer @spec function ( arg ) :: [ arg ] when arg : var # TypeCheck - unsupported TypeCheck currently solely allows the usage of when to specify 'type guards' (see below). Support for spec guards could be added. However: Their usage is quite rare. Each place where it is used, it matches 'exactly the same value'. Building this check is relatively tricky. If you have a strong desire for this feature, please open an issue for it. ✅ Named Arguments # typespecs @spec days_since_epoch ( year :: integer , month :: integer , day :: integer ) :: integer @type color :: { red :: integer , green :: integer , blue :: integer } # TypeCheck @spec! days_since_epoch ( year :: integer , month :: integer , day :: integer ) :: integer @type! color :: { red :: integer , green :: integer , blue :: integer } ❌ Specification Overloads # typespecs @spec function ( integer ) :: atom @spec function ( atom ) :: integer # TypeCheck - unsupported There is no intention to support this. The implementation would be very difficult, and it is arguably good practice to have a single specification anyway. User Defined Types ✅ Basic Definition # typespecs @type type_name :: type @typep type_name :: type @opaque type_name :: type # TypeCheck @type! type_name :: type @typep! type_name :: type @opaque! type_name :: type ✅ Parameterized Types # typespecs @type dict ( key , value ) :: [ { key , value } ] # TypeCheck @type! dict ( key , value ) :: [ { key , value } ] 🚀 Type Guards To add extra custom checks to a type, you can use a so-called 'type guard'. This is arbitrary code that is executed during a type-check once the type itself already matches. You can use &quot;named types&quot; to refer to (parts of) the value that matched the type, and refer to these from a type-guard: # typespecs - unsupported # TypeCheck @type! sorted_pair :: { lower :: number ( ) , higher :: number ( ) } when lower &lt;= higher","ref":"comparing-typecheck-and-elixir-typespecs.html#defining-specifications","title":"Comparison to Plain Typespecs - Defining Specifications","type":"extras"},{"doc":"It is often useful to refer to types defined in other modules. These are called 'Remote types'. Elixir's typespecs and TypeCheck both support remote types. # typespecs defmodule User do @type t ( ) :: % User { name : String . t ( ) , age : non_negative_integer ( ) } end defmodule Greeter do @spec greet ( User . t ( ) ) :: String . t ( ) def greet ( user ) do # ... end end # TypeCheck defmodule User do use TypeCheck @type! t ( ) :: % User { name : String . t ( ) , age : non_negative_integer ( ) } end defmodule Greeter do use TypeCheck @spec! greet ( User . t ( ) ) :: String . t ( ) def greet ( user ) do # ... end end Remote Type Overrides From time to time we need to interface with modules written in other libraries (or the Elixir standard library) which do not expose their types through TypeCheck yet. We want to be able to use those types in our checks, but they exist in modules that we cannot change ourselves. The solution is to allow a list of ‘type overrides’ to be given as part of the options passed to use TypeCheck, which allow you to use the original type in your types and documentation, but have it be checked (and potentially property-generated) as the given TypeCheck-type. Example: defmodule Original do @type t ( ) :: any ( ) end defmodule Replacement do use TypeCheck @type! t ( ) :: integer ( ) end defmodule Example do use TypeCheck , overrides : [ { &amp; Original . t / 0 , &amp; Replacement . t / 0 } ] @spec! times_two ( Original . t ( ) ) :: integer ( ) def times_two ( input ) do input * 2 end end Elixir Standard Library Types TypeCheck helpfully ships with the majority of the types in Elixir's Standard Library already implemented as default overrides. This means that your @spec! definitions can reference types like String.t() , Date.t() and Range.t() out of the box. Type Supported? Notes Access ✅ Agent ✅ Application ✅ Calendar ✅ Calendar.ISO ✅ Calendar.TimeZoneDatabase ✅ Code ✅ Collectable ✅ Config.Provider ✅ Date ✅ Date.Range ✅ DateTime ✅ Dict ❌ deprecated DynamicSupervisor ✅ Enum ✅ Enumerable ✅ Exception ✅ File ✅ File.Stat ✅ File.Stream ✅ Float ✅ Function ✅ GenEvent ❌ deprecated GenServer ✅ HashDict ❌ deprecated HashSet ❌ deprecated IO ✅ IO.ANSI ✅ IO.Stream ✅ Inspect ✅ Inspect.Algebra ✅ Inspect.Opts ✅ Keyword ✅ List.Chars ✅ Macro ✅ Macro.Env ✅ Map ✅ MapSet ✅ NaiveDateTime ✅ Node ✅ OptionParser ✅ Path ✅ Port ✅ Process ✅ Range ✅ Regex ✅ Registry ✅ Set ❌ deprecated Stream ✅ String ✅ String.Chars ✅ Supervisor ✅ Supervisor.Spec ❌ deprecated System ✅ Task ✅ Task.Supervisor ✅ Time ✅ URI ✅ Version ✅ Version.Requirement ✅","ref":"comparing-typecheck-and-elixir-typespecs.html#remote-types","title":"Comparison to Plain Typespecs - ✅ Remote Types","type":"extras"},{"doc":"Norm Norm is an Elixir library for specifying the structure of data that can be used for both validation and data-generation. On a superficial level, Norm and TypeCheck seem similar. However, there are important differences in their design considerations: Primary Focus Norm focuses on conforming values to specifications by re-using your existing validations. Norm also has a focus on 'open' schemas that are designed to allow systems to grow over time. TypeCheck focuses on making the types your program is using explicit. This is done by retrofitting Elixir's built-in type syntax to allow you to use a single statement to create an expanded type-/function-specification for in the documentation, runtime type-checking code and data-generation for property-testing. Syntax Norm uses a new, guard-like function-call syntax. Norm is light on the little syntactic sugar: Literal atoms and tuples containing specs as elements are treated as specs themselves. Anything else requires you to write a (dedicated or anonymous) validation function. TypeCheck uses the same syntax that Elixir's built-in typespecs use, which is already familiar to many Elixir developers. This makes a TypeCheck @type! or @spec! often much shorter than the equivalent Norm spec or @contract . A couple of syntactical examples: A simple 'manual' validation. (This is common Norm usage but manual validations are more rare in TypeCheck.) # Norm: iex&gt; Norm . conform! ( 123 , spec ( is_integer ( ) and &amp; ( &amp;1 &gt; 0 ) ) ) 123 # TypeCheck iex&gt; TypeCheck . conforms! ( 123 , non_neg_integer ( ) ) # or: iex&gt; TypeCheck . conforms! ( 123 , x :: integer ( ) when x &gt;= 0 ) Defining custom type-specifications ('specs' in Norm parlance) and function-specifications ('contracts' in Norm parlance): # Norm: defmodule Color do import Norm def rgb ( ) , do : spec ( is_integer ( ) and &amp; ( &amp;1 in 0 .. 255 ) ) def hex ( ) , do : spec ( is_binary ( ) and &amp; String . starts_with? ( &amp;1 , &quot;#&quot; ) ) @contract rgb_to_hex ( r :: rgb ( ) , g :: rgb ( ) , b :: rgb ( ) ) :: hex ( ) def rgb_to_hex ( r , g , b ) do # ... end end # TypeCheck: defmodule Color do use TypeCheck @type! rgb :: 0 .. 255 @type! hex :: ( str :: binary ( ) when String . starts_with? ( str , &quot;#&quot; ) ) @spec! rgb_to_hex ( rgb ( ) , rgb ( ) , rgb ( ) ) :: hex ( ) def rgb_to_hex ( r , g , b ) do # ... end end Defining a more complicated specification of a custom structure with multiple fields: # Norm: defmodule User do use Norm defstruct [ :name , :age ] def age_spec ( ) , do : spec ( is_integer ( ) and &amp; ( &amp;1 &gt;= 0 ) ) def s ( ) do schema ( %{ name : spec ( is_binary ( ) ) , age : age_spec ( ) , } ) end @contract new ( name :: spec ( is_binary ( ) ) , age :: age_spec ( ) ) :: s ( ) def new ( name , age ) do % __MODULE__ { name : name , age : age } end @contract ensure_old_enough ( user :: s ( ) , limit :: age_spec ( ) ) :: alt ( success : { :ok , s ( ) } , problem : { :error , spec ( is_binary ( ) ) } ) def ensure_old_enough ( user , limit ) do if user . age &gt;= limit do { :ok , user } else { :error , &quot;not old enough&quot; } end end end # TypeCheck: defmodule User do use TypeCheck defstruct [ :name , :age ] @type! age :: non_neg_integer ( ) @type! t :: % User { name : binary ( ) , age : age ( ) } @spec! new ( binary ( ) , age ( ) ) :: t ( ) def new ( name , age ) do % User { name : name , age : age } end @spec! ensure_old_enough ( t ( ) , age ( ) ) :: { :ok , t ( ) } | { :error , binary ( ) } def ensure_old_enough ( user , limit ) do if user . age &gt;= limit do { :ok , user } else { :error , &quot;not old enough&quot; } end end end Error messages Norm does not particulary focus on readable error messages (although that might change in the future). TypeCheck heavily focuses on creating humanly-readable error-messages when a value does not type-check, creating a deeply nested error message indicating the cause of a top-level error (see the main README for some examples of this). TypeCheck's error messages are based on a pluggable formatter for which custom alternatives can be provided. As an example, consider the execution using the respective definitions of User.ensure_old_enough above: User . ensure_old_enough ( % User { name : &quot;Marten&quot; , age : 0.5 } , 21 ) ** (Norm.MismatchError) Could not conform input: val : 0.5 in : :age fails : is_integer ( ) ( norm 0.13 . 0 ) lib / norm . ex : 65 : Norm . conform! / 2 ( norm_example 0.1 . 0 ) lib / norm_example . ex : 38 : anonymous fn / 2 in User . ensure_old_enough / 2 ( elixir 1.12 . 0 ) lib / enum . ex : 2356 : Enum . &quot;-reduce/3-lists^foldl/2-0-&quot; / 3 ( norm_example 0.1 . 0 ) lib / norm_example . ex : 38 : User . ensure_old_enough / 2 ** (TypeCheck.TypeError) At lib/type_check_example.ex:21: The call to ` ensure_old_enough / 2 ` failed , because parameter no . 1 does not adhere to the spec ` % User { age : non_neg_integer ( ) , name : binary ( ) } ` . Rather , its value is : ` % User { age : 0.5 , name : &quot;Marten&quot; } ` . Details : The call ` ensure_old_enough ( % User { age : 0.5 , name : &quot;Marten&quot; } , 21 ) ` does not adhere to spec ` ensure_old_enough ( % User { age : non_neg_integer ( ) , name : binary ( ) } , non_neg_integer ( ) ) :: { :ok , % User { age : non_neg_integer ( ) , name : binary ( ) } } | { :error , binary ( ) } ` . Reason : parameter no . 1 : ` % User { age : 0.5 , name : &quot;Marten&quot; } ` does not check against ` % User { age : non_neg_integer ( ) , name : binary ( ) } ` . Reason : under key ` :age ` : ` 0.5 ` is not a non - negative integer . ( type_check_example 0.1 . 0 ) lib / type_check / spec . ex : 165 : User . ensure_old_enough / 2 Another example, displaying the difference between error messages when the problem is in the value returned by the function: # Given the `rgb_to_hex` function as defined above, but with an implementation not adhering to its spec/contract def rgb_to_hex ( r , g , b ) do { :ok , 42 } end iex&gt; Color . rgb_to_hex ( 10 , 20 , 30 ) ** (Norm.MismatchError) Could not conform input: val : { :ok , 42 } fails : is_binary ( ) ( norm 0.13 . 0 ) lib / norm . ex : 65 : Norm . conform! / 2 iex&gt; Color . rgb_to_hex ( 10 , 20 , 30 ) ** (TypeCheck.TypeError) The call to `rgb_to_hex/3` failed, because the returned result does not adhere to the spec ` ( str :: binary ( ) when String . starts_with? ( str , &quot;#&quot; ) ) ` . Rather , its value is : ` { :ok , 42 } ` . Details : The result of calling ` rgb_to_hex ( 10 , 20 , 30 ) ` does not adhere to spec ` rgb_to_hex ( 0 .. 255 , 0 .. 255 , 0 .. 255 ) :: ( str :: binary ( ) when String . starts_with? ( str , &quot;#&quot; ) ) ` . Reason : Returned result : ` { :ok , 42 } ` does not check against ` ( str :: binary ( ) when String . starts_with? ( str , &quot;#&quot; ) ) ` . Reason : ` { :ok , 42 } ` does not check against ` str :: binary ( ) ` . Reason : ` { :ok , 42 } ` is not a binary . ( type_check_example 0.1 . 0 ) lib / type_check / spec . ex : 194 : Color . rgb_to_hex / 3 Execution &amp; Efficiency In Norm, while wrapping functions with a contract happens at compile-time, all contracts and specs are resolved at runtime. This makes Norm's internals less metaprogramming-heavy and easily allows specs to be created and manipulated dynamically at runtime, but it does mean that the compiler is not able to optimize the type-checking code at all, and specs are re-evaluated every time a function is called. TypeCheck requires¹ types to be defined at compile-time and injects the type-checking code into your functions and modules before they are compiled, allowing type-checking to be optimized. If there is overlap between the parts of your parameters being checked by TypeCheck and the logic of your function, the BEAM compiler will in most cases be able to combine these into a single check. ¹: In normal usage. Escape hatches to work with types defined at runtime exist. (c.f. TypeCheck.dynamic_conforms/2 and variants) Documentation Norm does not focus on documentation. Norm's spec s are normal functions which you can document manually using @doc if you wish. Norm's @contract s are not used for documentation purposes. TypeCheck adds @type / @typep / @opaque attributes for the types you specify, making them show up in your documentation and allowing you to use the same type definitions for tools like Dialyzer . You can also use the t helper to look them up in IEx. Documentation can be added to these types by using @typedoc (just like for normal typespecs). Function-specifications created with TypeCheck will also add @spec -attributes, which will end up in the documentation of your functions and are similarly useful for e.g. Dialyzer . Data Generation &amp; Testing It is very useful to generate examples of good data to be used for property testing. Both Norm and TypeCheck have this capability, by using :stream_data as an optional dependency. Norm's generators (only) work when the first predicate in a spec(...) is one of (a subset of) Elixir's built-in guard-clauses. If your spec is too restrictive, you'll have to manually provide a custom data generator as well. TypeCheck builds more complicated generators out of simple ones just as it builds complicated types out of simple ones. This means that virtually all Elixir types can be turned into generators without extra effort of the user. It is only when 'type guards' are used to add arbitrary checks to a type that you might up with a generator that is too restrictive. TypeCheck supports nearly all of Elixir's builtin types, as well as many of the remote types that are part of Elixir's standard library ( Range.t , MapSet.t , List.t , Enum.t etc.) Furthermore, TypeCheck allows overriding the generator for a type (c.f. the TypeCheck.Type.StreamData.wrap_with_gen/2 macro). Finally and maybe most importantly, TypeCheck ships with the spectest macro. (c.f. TypeCheck.ExUnit.spectest/2 ) spectest works similarly to doctest but rather than testing all code snippets in the module's documentation, it will automatically run a property-test to check for each @spec! -ced function in a module, whether it correctly follows its spec. Norm and TypeCheck are but two different dots in the datastructure-validation design space. Norm is definitely worth checking out!","ref":"comparing-typecheck-and-norm.html","title":"Comparing TypeCheck and Norm","type":"extras"}]